/**************************************************************************
*  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
*  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
*                                                                         *
*  Merc Diku Mud improvements copyright (C) 1992, 1993 by Michael         *
*  Chastain, Michael Quan, and Mitchell Tse.                              *
*                                                                         *
*  In order to use any part of this Merc Diku Mud, you must comply with   *
*  both the original Diku license in 'license.doc' as well the Merc       *
*  license in 'license.txt'.  In particular, you may not remove either of *
*  these copyright notices.                                               *
*                                                                         *
*  Much time and thought has gone into this software and you are          *
*  benefiting.  We hope that you share your changes too.  What goes       *
*  around, comes around.                                                  *
***************************************************************************
*       ROM 2.4 is copyright 1993-1998 Russ Taylor                        *
*       ROM has been brought to you by the ROM consortium                 *
*           Russ Taylor (rtaylor@hypercube.org)                           *
*           Gabrielle Taylor (gtaylor@hypercube.org)                      *
*           Brian Moore (zump@rom.org)                                    *
*       By using this code, you have agreed to follow the terms of the    *
*       ROM license, in the file Rom24/doc/rom.license                    *
***************************************************************************
*       1stMUD ROM Derivative (c) 2001-2003 by Ryan Jennings              *
*            http://1stmud.dlmud.com/  <r-jenn@shaw.ca>                   *
***************************************************************************/

#define	IN_DB_C
#include "merc.h"
#include "db.h"
#include "recycle.h"
#include "music.h"
#include "tables.h"
#include "lookup.h"
#include "olc.h"
#include "interp.h"
#include "gcn.h"
#include "tablesave.h"
#include "save.h"
#undef IN_DB_C

/*
 * Locals.
 */
MOB_INDEX_DATA *mob_index_hash[MAX_KEY_HASH];
OBJ_INDEX_DATA *obj_index_hash[MAX_KEY_HASH];
ROOM_INDEX_DATA *room_index_hash[MAX_KEY_HASH];

AREA_DATA *current_area;

char str_empty[1];

/*
 * Semi-locals.
 */
READ_DATA *fpArea;
char strArea[MAX_INPUT_LENGTH];

/*
 * Local booting procedures.
*/
PROTOTYPE(void init_mm, (void));
PROTOTYPE(void load_area, (READ_DATA *));
PROTOTYPE(void new_load_area, (READ_DATA *));	/* OLC */
PROTOTYPE(void load_helps, (READ_DATA *));
PROTOTYPE(void load_old_mob, (READ_DATA *));
PROTOTYPE(void load_old_obj, (READ_DATA *));
PROTOTYPE(void load_resets, (READ_DATA *));
PROTOTYPE(void load_rooms, (READ_DATA *));
PROTOTYPE(void load_shops, (READ_DATA *));
PROTOTYPE(void load_specials, (READ_DATA *));
PROTOTYPE(void load_mobprogs, (READ_DATA *));
PROTOTYPE(void load_objprogs, (READ_DATA *));
PROTOTYPE(void load_roomprogs, (READ_DATA *));
PROTOTYPE(void fix_objprogs, (void));
PROTOTYPE(void fix_roomprogs, (void));
PROTOTYPE(void init_area_weather, (void));
PROTOTYPE(void fix_exits, (void));
PROTOTYPE(void fix_mobprogs, (void));
PROTOTYPE(void init_www_history, (void));

void load_time(void)
{
	long lhour, lday, lmonth;
	int total;
	bool fail = FALSE;
	READ_DATA *time_file;

	if ((time_file = open_read(TIME_FILE)) != NULL)
	{

		total =
			sscanf(time_file->str, "%d %d %d %d", &(time_info.hour),
				   &(time_info.day), &(time_info.month), &(time_info.year));
		if (total != 4)
		{
			log_string("loading stored time failed, using default");
			fail = TRUE;
		}
		close_read(time_file);
	}
	else
	{
		log_string("failed open of time_file, loading default time.");
		fail = TRUE;
	}

	if (fail)
	{
		lhour = (current_time - 650336715) / (PULSE_TICK / PULSE_PER_SECOND);
		time_info.hour = lhour % 24;
		lday = lhour / 24;
		time_info.day = lday % 35;
		lmonth = lday / 35;
		time_info.month = lmonth % 17;
		time_info.year = lmonth / 17;
	}
	if (time_info.hour < 5)
	{
		time_info.sunlight = SUN_DARK;
	}
	else if (time_info.hour < 6)
	{
		time_info.sunlight = SUN_RISE;
	}
	else if (time_info.hour < 19)
	{
		time_info.sunlight = SUN_LIGHT;
	}
	else if (time_info.hour < 20)
	{
		time_info.sunlight = SUN_SET;
	}
	else
	{
		time_info.sunlight = SUN_DARK;
	}

	mud_info.weath_unit = 10;
	mud_info.rand_factor = 2;
	mud_info.climate_factor = 1;
	mud_info.max_vector = mud_info.weath_unit * 3;
}

void load_area_db(void)
{
	READ_DATA *fpList;

	if ((fpList = open_read(AREA_LIST)) == NULL)
	{
		log_error(AREA_LIST);
		exit(1);
	}

	for (;;)
	{
		strcpy(strArea, read_word(fpList));

		if (strArea[0] == '$')
			break;

		if ((fpArea = open_read(AREA_DIR "%s", strArea)) == NULL)
		{
			log_error(strArea);
			exit(1);
		}

		logf("Loading %s...", strArea);
		current_area = NULL;

		for (;;)
		{
			char *word;

			if (read_letter(fpArea) != '#')
			{
				bug("Boot_db: # not found.");
				exit(1);
			}

			word = read_word(fpArea);

			if (word[0] == '$')
				break;
			else if (!str_cmp(word, "AREA"))
				load_area(fpArea);
			else if (!str_cmp(word, "AREADATA"))
				new_load_area(fpArea);
			else if (!str_cmp(word, "HELPS"))
				load_helps(fpArea);
			else if (!str_cmp(word, "MOBOLD"))
				load_old_mob(fpArea);
			else if (!str_cmp(word, "MOBILES"))
				load_mobiles(fpArea);
			else if (!str_cmp(word, "MOBPROGS"))
				load_mobprogs(fpArea);
			else if (!str_cmp(word, "OBJPROGS"))
				load_objprogs(fpArea);
			else if (!str_cmp(word, "ROOMPROGS"))
				load_roomprogs(fpArea);
			else if (!str_cmp(word, "OBJOLD"))
				load_old_obj(fpArea);
			else if (!str_cmp(word, "OBJECTS"))
				load_objects(fpArea);
			else if (!str_cmp(word, "RESETS"))
				load_resets(fpArea);
			else if (!str_cmp(word, "ROOMS"))
				load_rooms(fpArea);
			else if (!str_cmp(word, "SHOPS"))
				load_shops(fpArea);
			else if (!str_cmp(word, "SPECIALS"))
				load_specials(fpArea);
			else
			{
				bug("Boot_db: bad section name.");
				exit(1);
			}
		}
		close_read(fpArea);
	}
	close_read(fpList);
	fpArea = NULL;
}

/*
 * Big mama top level function.
 */
void boot_db(void)
{

	run_level = RUNLEVEL_BOOTING;

	/*
	 * Init random number generator.
	 */
	log_string("Starting random number generator...");
	init_mm();

	log_string("Loading mud data...");
	rw_mud_data(action_read);

	logf("Loading game time.");
	load_time();

	log_string("Loading classes...");
	rw_classes(action_read);

	log_string("Loading skills...");
	rw_skills(action_read);
	log_string("Loading groups...");
	rw_groups(action_read);
	log_string("Loading channels...");
	rw_channels(action_read);
#if !defined(NO_WEB)
	init_www_history();
#endif

	/*
	 * Assign gsn's for skills which have them.
	 */
	{
		int sn;

		log_string("Assigning GSN's...");
		for (sn = 0; sn < maxSkill; sn++)
		{
			if (skill_table[sn].pgsn != NULL
				&& skill_table[sn].pgsn != &gsn_null)
				*skill_table[sn].pgsn = sn;
		}
		for (sn = 0; sn < maxChannel; sn++)
		{
			if (channel_table[sn].index != NULL
				&& channel_table[sn].index != &gcn_null)
				*channel_table[sn].index = sn;
		}
	}

	crs_info.status = CRS_NONE;
	crs_info.who = &str_empty[0];
	crs_info.reason = &str_empty[0];
	crs_info.timer = -1;

	log_string("Loading races...");
	rw_races(action_read);
	log_string("Loading clans...");
	rw_clans(action_read);
	rw_members(action_read);

	/*
	 * Read in all the area files.
	 */
	load_area_db();

	/*
	 * Fix up exits.
	 * Declare db booting over.
	 * Reset all areas once.
	 * Load up the songs, notes and ban files.
	 */
	fix_exits();
	fix_mobprogs();
	fix_objprogs();
	fix_roomprogs();
	run_level = RUNLEVEL_BOOT_NOEXIT;
	convert_objects();			/* ROM OLC */
	area_update();
	log_string("Loading boards...");
	load_boards();				/* Load all boards */
	save_notes();
	rw_bans(action_read);
	rw_music(action_read);
	load_disabled();
	log_string("Loading socials...");
	rw_socials(action_read);
	rw_gquest_data(action_read);
	rw_war_data(action_read);
	log_string("Loading commands...");
	rw_commands(action_read);
	load_corpses();
	load_donation_pit();
	log_string("Loading deities...");
	rw_deities(action_read);
	log_string("Loading web passwords...");
	rw_webpasswds(action_read);
	logf("Initializing weather data.");
	init_area_weather();
	logf("Stored %d Areas, %d Rooms, %d Mobs & %d Objects", top_area,
		 top_room, top_mob_index, top_obj_index);
	log_string("Finished.");

	return;
}

PROTOTYPE(void convert_area_credits, (AREA_DATA *));

/*
 * Snarf an 'area' header line.
 */
void load_area(READ_DATA * fp)
{
	AREA_DATA *pArea;

	pArea = new_area();
	free_read_string(pArea->file_name, fp);

	pArea->area_flags = AREA_LOADING;	/* OLC */
	free_read_string(pArea->name, fp);
	free_read_string(pArea->credits, fp);
	pArea->min_vnum = read_number(fp);
	pArea->max_vnum = read_number(fp);

	add_area(pArea);
	current_area = pArea;
	convert_area_credits(pArea);

	top_area++;
	return;
}

/* OLC
 * Snarf an 'area' header line.   Check this format.  MUCH better.  Add fields
 * too.
 *
 * #AREAFILE
 * Name   { All } Locke    Newbie School~
 * Repop  A teacher pops in the room and says, 'Repop coming!'~
 * Recall 3001
 * End
 */
void new_load_area(READ_DATA * fp)
{
	AREA_DATA *pArea;
	const char *word;
	bool fMatch;

	pArea = new_area();
	replace_string(pArea->file_name, strArea);

	for (;;)
	{
		word = steof(fp) ? "End" : read_word(fp);
		fMatch = FALSE;

		switch (UPPER(word[0]))
		{
		case 'L':
			KEYS("LvlComment", pArea->lvl_comment);
			break;
		case 'M':
			KEY("MinLevel", pArea->min_level, read_number(fp));
			KEY("MaxLevel", pArea->max_level, read_number(fp));
			break;
		case 'N':
			KEYS("Name", pArea->name);
			break;
		case 'F':
			KEY("Flags", pArea->area_flags, read_flag(fp));
			break;
		case 'S':
			KEY("Security", pArea->security, read_number(fp));
			break;
		case 'V':
			KEY("Version", pArea->version, read_number(fp));
			if (!str_cmp(word, "VNUMs"))
			{
				pArea->min_vnum = read_number(fp);
				pArea->max_vnum = read_number(fp);
				fMatch = TRUE;
				break;
			}
			break;
		case 'E':
			if (!str_cmp(word, "End"))
			{
				fMatch = TRUE;
				add_area(pArea);
				current_area = pArea;
				top_area++;
				convert_area_credits(pArea);

				return;
			}
			break;
		case 'B':
			KEYS("Builders", pArea->builders);
			break;
		case 'C':
			KEYS("Credits", pArea->credits);
			if (!str_cmp(word, "Climate"))
			{
				pArea->weather.climate_temp = read_number(fp);
				pArea->weather.climate_precip = read_number(fp);
				pArea->weather.climate_wind = read_number(fp);
				fMatch = TRUE;
				break;
			}
			break;
		}
	}
}

/*
 * Sets vnum range for area using OLC protection features.
 */
void assign_area_vnum(vnum_t vnum)
{
	if (current_area->min_vnum == 0 || current_area->max_vnum == 0)
		current_area->min_vnum = current_area->max_vnum = vnum;
	if (vnum != URANGE(current_area->min_vnum, vnum, current_area->max_vnum))
	{
		if (vnum < current_area->min_vnum)
			current_area->min_vnum = vnum;
		else
			current_area->max_vnum = vnum;
	}
	return;
}

/*
 * Snarf a help section.
 */
void load_helps(READ_DATA * fp)
{
	HELP_DATA *pHelp;
	int level;
	const char *keyword;

	for (;;)
	{
		level = read_number(fp);
		keyword = read_string(fp);

		if (keyword[0] == '$')
			break;

		pHelp = new_help();
		pHelp->level = level;
		free_string(pHelp->keyword);
		pHelp->keyword = keyword;
		free_read_string(pHelp->text, fp);

		if (!str_cmp(pHelp->keyword, "greeting"))
			help_greeting = pHelp->text;

		add_help(pHelp);

		top_help++;
	}

	return;
}

/*
 * Snarf a mob section.  old style 
 */
void load_old_mob(READ_DATA * fp)
{
	MOB_INDEX_DATA *pMobIndex;
	/* for race updating */
	RACE_DATA *race;
	char name[MAX_STRING_LENGTH];

	if (!current_area)			/* OLC */
	{
		bug("Load_mobiles: no #AREA seen yet.");
		exit(1);
	}

	for (;;)
	{
		vnum_t vnum;
		char letter;
		int iHash;

		letter = read_letter(fp);
		if (letter != '#')
		{
			bug("Load_mobiles: # not found.");
			exit(1);
		}

		vnum = read_number(fp);
		if (vnum == 0)
			break;

		run_level = RUNLEVEL_BOOT_NOEXIT;
		if (get_mob_index(vnum) != NULL)
		{
			bugf("Load_mobiles: vnum %ld duplicated.", vnum);
			exit(1);
		}
		run_level = RUNLEVEL_BOOTING;

		pMobIndex = new_mob_index();
		pMobIndex->vnum = vnum;
		pMobIndex->area = current_area;	/* OLC */
		pMobIndex->new_format = FALSE;
		free_read_string(pMobIndex->player_name, fp);
		free_read_string(pMobIndex->short_descr, fp);
		free_read_string(pMobIndex->long_descr, fp);
		free_read_string(pMobIndex->description, fp);

/*
		pMobIndex->long_descr[0] = UPPER (pMobIndex->long_descr[0]);
		pMobIndex->description[0] = UPPER (pMobIndex->description[0]);
*/

		pMobIndex->act = read_flag(fp) | ACT_IS_NPC;
		pMobIndex->affected_by = read_flag(fp);
		pMobIndex->pShop = NULL;
		pMobIndex->alignment = read_number(fp);
		letter = read_letter(fp);
		pMobIndex->level = read_number(fp);

		/*
		 * The unused stuff is for imps who want to use the old-style
		 * stats-in-files method.
		 */
		read_number(fp);		/* Unused */
		read_number(fp);		/* Unused */
		read_number(fp);		/* Unused */
		/* 'd'      */ read_letter(fp);
		/* Unused */
		read_number(fp);		/* Unused */
		/* '+'      */ read_letter(fp);
		/* Unused */
		read_number(fp);		/* Unused */
		read_number(fp);		/* Unused */
		/* 'd'      */ read_letter(fp);
		/* Unused */
		read_number(fp);		/* Unused */
		/* '+'      */ read_letter(fp);
		/* Unused */
		read_number(fp);		/* Unused */
		pMobIndex->wealth = read_number(fp) / 20;
		/* xp can't be used! */ read_number(fp);
		/* Unused */
		pMobIndex->start_pos = read_enum(position_t, fp);	/* Unused */
		pMobIndex->default_pos = read_enum(position_t, fp);	/* Unused */

		if (pMobIndex->start_pos < POS_SLEEPING)
			pMobIndex->start_pos = POS_STANDING;
		if (pMobIndex->default_pos < POS_SLEEPING)
			pMobIndex->default_pos = POS_STANDING;

		/*
		 * Back to meaningful values.
		 */
		pMobIndex->sex = read_number(fp);

		/* compute the race BS */
		one_argument(pMobIndex->player_name, name);

		if (IS_NULLSTR(name) || (race = race_lookup(name)) == NULL)
		{
			/* fill in with blanks */
			pMobIndex->race = default_race;
			pMobIndex->off_flags =
				OFF_DODGE | OFF_DISARM | OFF_TRIP | ASSIST_VNUM;
			pMobIndex->imm_flags = 0;
			pMobIndex->res_flags = 0;
			pMobIndex->vuln_flags = 0;
			pMobIndex->form =
				FORM_EDIBLE | FORM_SENTIENT | FORM_BIPED | FORM_MAMMAL;
			pMobIndex->parts =
				PART_HEAD | PART_ARMS | PART_LEGS | PART_HEART |
				PART_BRAINS | PART_GUTS;
		}
		else
		{
			pMobIndex->race = race;
			pMobIndex->off_flags =
				OFF_DODGE | OFF_DISARM | OFF_TRIP | ASSIST_RACE | race->off;
			pMobIndex->imm_flags = race->imm;
			pMobIndex->res_flags = race->res;
			pMobIndex->vuln_flags = race->vuln;
			pMobIndex->form = race->form;
			pMobIndex->parts = race->parts;
		}

		if (letter != 'S')
		{
			bugf("Load_mobiles: vnum %ld non-S.", vnum);
			exit(1);
		}

		convert_mobile(pMobIndex);	/* ROM OLC */

		iHash = vnum % MAX_KEY_HASH;
		LINK_SINGLE(pMobIndex, next, mob_index_hash[iHash]);
		top_mob_index++;
		top_vnum_mob = top_vnum_mob < vnum ? vnum : top_vnum_mob;	/* OLC */
		assign_area_vnum(vnum);	/* OLC */
		kill_table[URANGE(0, pMobIndex->level, MAX_LEVEL - 1)].number++;
	}

	return;
}

/*
 * Snarf an obj section.  old style 
 */
void load_old_obj(READ_DATA * fp)
{
	OBJ_INDEX_DATA *pObjIndex;

	if (!current_area)			/* OLC */
	{
		bug("Load_objects: no #AREA seen yet.");
		exit(1);
	}

	for (;;)
	{
		vnum_t vnum;
		char letter;
		int iHash;

		letter = read_letter(fp);
		if (letter != '#')
		{
			bug("Load_objects: # not found.");
			exit(1);
		}

		vnum = read_number(fp);
		if (vnum == 0)
			break;

		run_level = RUNLEVEL_BOOT_NOEXIT;
		if (get_obj_index(vnum) != NULL)
		{
			bugf("Load_objects: vnum %ld duplicated.", vnum);
			exit(1);
		}
		run_level = RUNLEVEL_BOOTING;

		pObjIndex = new_obj_index();
		pObjIndex->vnum = vnum;
		pObjIndex->area = current_area;	/* OLC */
		pObjIndex->new_format = FALSE;
		pObjIndex->reset_num = 0;
		free_read_string(pObjIndex->name, fp);
		free_read_string(pObjIndex->short_descr, fp);
		free_read_string(pObjIndex->description, fp);
		/* Action description */ read_string(fp);

/*		pObjIndex->short_descr[0] = LOWER (pObjIndex->short_descr[0]);
		pObjIndex->description[0] = UPPER (pObjIndex->description[0]); */

		pObjIndex->item_type = read_number(fp);
		pObjIndex->extra_flags = read_flag(fp);
		pObjIndex->wear_flags = read_flag(fp);
		pObjIndex->value[0] = read_number(fp);
		pObjIndex->value[1] = read_number(fp);
		pObjIndex->value[2] = read_number(fp);
		pObjIndex->value[3] = read_number(fp);
		pObjIndex->value[4] = 0;
		pObjIndex->level = 0;
		pObjIndex->condition = 100;
		pObjIndex->weight = read_number(fp);
		pObjIndex->cost = read_number(fp);	/* Unused */
		/* Cost per day */ read_number(fp);

		if (pObjIndex->item_type == ITEM_WEAPON)
		{
			if (is_name("two", pObjIndex->name) ||
				is_name("two-handed", pObjIndex->name) ||
				is_name("claymore", pObjIndex->name))
				SET_BIT(pObjIndex->value[4], WEAPON_TWO_HANDS);
		}

		for (;;)
		{
			char pletter;

			pletter = read_letter(fp);

			if (pletter == 'A')
			{
				AFFECT_DATA *paf;

				paf = new_affect();
				paf->where = TO_OBJECT;
				paf->type = -1;
				paf->level = 20;	/* RT temp fix */
				paf->duration = -1;
				paf->location = read_enum(apply_t, fp);
				paf->modifier = read_number(fp);
				paf->bitvector = 0;
				LINK(paf, pObjIndex->first_affect, pObjIndex->last_affect, next,
					 prev);
				top_affect++;
			}

			else if (pletter == 'E')
			{
				EXTRA_DESCR_DATA *ed;

				ed = new_extra_descr();
				free_read_string(ed->keyword, fp);
				free_read_string(ed->description, fp);
				LINK(ed, pObjIndex->first_extra_descr,
					 pObjIndex->last_extra_descr, next, prev);
				top_ed++;
			}

			else
			{
				fp->pos--;
				break;
			}
		}

		/* fix armors */
		if (pObjIndex->item_type == ITEM_ARMOR)
		{
			pObjIndex->value[1] = pObjIndex->value[0];
			pObjIndex->value[2] = pObjIndex->value[1];
		}

		/*
		 * Translate spell "slot numbers" to internal "skill numbers."
		 */
		switch (pObjIndex->item_type)
		{
		case ITEM_PILL:
		case ITEM_POTION:
		case ITEM_SCROLL:
			break;

		case ITEM_STAFF:
		case ITEM_WAND:
			break;
		}

		iHash = vnum % MAX_KEY_HASH;
		LINK_SINGLE(pObjIndex, next, obj_index_hash[iHash]);
		top_obj_index++;
		top_vnum_obj = top_vnum_obj < vnum ? vnum : top_vnum_obj;	/* OLC */
		assign_area_vnum(vnum);	/* OLC */
	}

	return;
}

/*
 * Adds a reset to a room.  OLC
 * Similar to add_reset in olc.c
 */
void new_reset(ROOM_INDEX_DATA * pR, RESET_DATA * pReset)
{
	if (!pR)
		return;

	LINK(pReset, pR->reset_first, pR->reset_last, next, prev);

	return;
}

/*
 * Snarf a reset section.
 */
void load_resets(READ_DATA * fp)
{
	RESET_DATA *pReset;
	EXIT_DATA *pexit;
	ROOM_INDEX_DATA *pRoomIndex;
	vnum_t rVnum = -1;

	if (!current_area)
	{
		bug("Load_resets: no #AREA seen yet.");
		exit(1);
	}

	for (;;)
	{
		char letter;

		if ((letter = read_letter(fp)) == 'S')
			break;

		if (letter == '*')
		{
			read_to_eol(fp);
			continue;
		}

		pReset = new_reset_data();
		pReset->command = letter;
		/* if_flag */ read_number(fp);
		pReset->arg1 = read_number(fp);
		pReset->arg2 = read_number(fp);
		pReset->arg3 = (letter == 'G'
						|| (current_area->version < 3
							&& letter == 'R')) ? 0 : read_number(fp);
		pReset->arg4 = (letter == 'P' || letter == 'M') ? read_number(fp) : 0;
		read_to_eol(fp);

		switch (pReset->command)
		{
		case 'M':
		case 'O':
			rVnum = pReset->arg3;
			break;

		case 'P':
		case 'G':
		case 'E':
			break;

		case 'D':
			pRoomIndex = get_room_index((rVnum = pReset->arg1));
			if (pReset->arg2 < 0 || pReset->arg2 >= MAX_DIR ||
				!pRoomIndex ||
				!(pexit = pRoomIndex->exit[pReset->arg2]) ||
				!IS_SET(pexit->rs_flags, EX_ISDOOR))
			{
				bugf("Load_resets: 'D': exit %d, room %ld not door.",
					 pReset->arg2, pReset->arg1);
				exit(1);
			}

			switch (pReset->arg3)
			{
			default:
				bugf("Load_resets: 'D': bad 'locks': %ld.", pReset->arg3);
				break;
			case 0:
				break;
			case 1:
				SET_BIT(pexit->rs_flags, EX_CLOSED);
				SET_BIT(pexit->exit_info, EX_CLOSED);
				break;
			case 2:
				SET_BIT(pexit->rs_flags, EX_CLOSED | EX_LOCKED);
				SET_BIT(pexit->exit_info, EX_CLOSED | EX_LOCKED);
				break;
			}
			break;

		case 'R':
			rVnum = pReset->arg1;
			break;
		}

		if (rVnum == -1)
		{
			bugf("load_resets : rVnum == -1");
			exit(1);
		}

		if (pReset->command != 'D')
			new_reset(get_room_index(rVnum), pReset);
		else
			free_reset_data(pReset);
	}

	return;
}

/*
 * Snarf a room section.
 */
void load_rooms(READ_DATA * fp)
{
	ROOM_INDEX_DATA *pRoomIndex;

	if (current_area == NULL)
	{
		bug("Load_resets: no #AREA seen yet.");
		exit(1);
	}

	for (;;)
	{
		vnum_t vnum;
		char letter;
		int door;
		int iHash;

		letter = read_letter(fp);
		if (letter != '#')
		{
			bug("Load_rooms: # not found.");
			exit(1);
		}

		vnum = read_number(fp);
		if (vnum == 0)
			break;

		run_level = RUNLEVEL_BOOT_NOEXIT;
		if (get_room_index(vnum) != NULL)
		{
			bugf("Load_rooms: vnum %ld duplicated.", vnum);
			exit(1);
		}
		run_level = RUNLEVEL_BOOTING;

		pRoomIndex = new_room_index();
		pRoomIndex->first_person = NULL;
		pRoomIndex->clan = NULL;
		pRoomIndex->first_content = NULL;
		pRoomIndex->first_extra_descr = NULL;
		pRoomIndex->area = current_area;
		pRoomIndex->vnum = vnum;
		free_read_string(pRoomIndex->name, fp);
		free_read_string(pRoomIndex->description, fp);
		/* Area number */ read_number(fp);
		pRoomIndex->room_flags = read_flag(fp);
		/* horrible hack */
		if (3000 <= vnum && vnum < 3400)
			SET_BIT(pRoomIndex->room_flags, ROOM_LAW);
		pRoomIndex->sector_type = read_enum(sector_t, fp);
		pRoomIndex->light = 0;
		for (door = 0; door <= 5; door++)
			pRoomIndex->exit[door] = NULL;

		/* defaults */
		pRoomIndex->heal_rate = 100;
		pRoomIndex->mana_rate = 100;

		for (;;)
		{
			letter = read_letter(fp);

			if (letter == 'S')
				break;

			if (letter == 'H')	/* healing room */
				pRoomIndex->heal_rate = read_number(fp);

			else if (letter == 'M')	/* mana room */
				pRoomIndex->mana_rate = read_number(fp);

			else if (letter == 'C')	/* clan */
			{
				const char *tmp;

				if (pRoomIndex->clan != NULL)
				{
					bug("Load_rooms: duplicate clan fields.");
					exit(1);
				}
				tmp = read_string(fp);
				pRoomIndex->clan = clan_lookup(tmp);
				free_string(tmp);
				SET_BIT(pRoomIndex->room_flags, ROOM_NOEXPLORE);
			}

			else if (letter == 'D')
			{
				EXIT_DATA *pexit;

				door = read_number(fp);
				if (door < 0 || door > 5)
				{
					bugf("Fread_rooms: vnum %ld has bad door number.", vnum);
					exit(1);
				}

				pexit = new_exit();
				free_read_string(pexit->description, fp);
				free_read_string(pexit->keyword, fp);
				pexit->exit_info = 0;
				pexit->rs_flags = 0;	/* OLC */
				if (pRoomIndex->area->version < 2)
				{
					switch (read_number(fp))
					{
					case 1:
						pexit->exit_info = EX_ISDOOR;
						pexit->rs_flags = EX_ISDOOR;
						break;
					case 2:
						pexit->exit_info = EX_ISDOOR | EX_PICKPROOF;
						pexit->rs_flags = EX_ISDOOR | EX_PICKPROOF;
						break;
					case 3:
						pexit->exit_info = EX_ISDOOR | EX_NOPASS;
						pexit->rs_flags = EX_ISDOOR | EX_NOPASS;
						break;
					case 4:
						pexit->exit_info = EX_ISDOOR | EX_NOPASS | EX_PICKPROOF;
						pexit->rs_flags = EX_ISDOOR | EX_NOPASS | EX_PICKPROOF;
						break;
					}
				}
				else
					pexit->rs_flags = read_flag(fp);

				if (IS_SET(pexit->rs_flags, EX_TEMP))
					REMOVE_BIT(pexit->rs_flags, EX_TEMP);

				pexit->exit_info = pexit->rs_flags;
				pexit->key = read_number(fp);
				pexit->u1.vnum = read_number(fp);
				pexit->orig_door = door;	/* OLC */

				pRoomIndex->exit[door] = pexit;
				top_exit++;
			}
			else if (letter == 'E')
			{
				EXTRA_DESCR_DATA *ed;

				ed = new_extra_descr();
				free_read_string(ed->keyword, fp);
				free_read_string(ed->description, fp);
				LINK(ed, pRoomIndex->first_extra_descr,
					 pRoomIndex->last_extra_descr, next, prev);
				top_ed++;
			}

			else if (letter == 'O')
			{
				if (!IS_NULLSTR(pRoomIndex->owner))
				{
					bug("Load_rooms: duplicate owner.");
					exit(1);
				}

				pRoomIndex->owner = read_string(fp);
				SET_BIT(pRoomIndex->room_flags, ROOM_NOEXPLORE);
			}
			else if (letter == 'R')
			{
				PROG_LIST *pRprog;
				const char *word;
				flag_t trigger = 0;

				pRprog = new_prog();
				word = read_word(fp);
				if ((trigger = flag_value(rprog_flags, word)) == NO_FLAG)
				{
					bug("ROOMprogs: invalid trigger.");
					exit(1);
				}
				SET_BIT(pRoomIndex->rprog_flags, trigger);
				pRprog->trig_type = trigger;
				pRprog->vnum = read_number(fp);
				free_read_string(pRprog->trig_phrase, fp);
				LINK(pRprog, pRoomIndex->first_rprog, pRoomIndex->last_rprog,
					 next, prev);
			}
			else
			{
				bugf("Load_rooms: vnum %ld has flag not 'DES'.", vnum);
				exit(1);
			}
		}

		iHash = vnum % MAX_KEY_HASH;
		LINK_SINGLE(pRoomIndex, next, room_index_hash[iHash]);
		top_room++;
		if (!IS_SET(pRoomIndex->room_flags, ROOM_NOEXPLORE))
			top_explored++;
		top_vnum_room = top_vnum_room < vnum ? vnum : top_vnum_room;	/* OLC */
		assign_area_vnum(vnum);	/* OLC */
	}

	return;
}

/*
 * Snarf a shop section.
 */
void load_shops(READ_DATA * fp)
{
	SHOP_DATA *pShop;

	for (;;)
	{
		MOB_INDEX_DATA *pMobIndex;
		int iTrade;

		pShop = new_shop();
		pShop->keeper = read_number(fp);
		if (pShop->keeper == 0)
			break;
		for (iTrade = 0; iTrade < MAX_TRADE; iTrade++)
			pShop->buy_type[iTrade] = read_number(fp);
		pShop->profit_buy = read_number(fp);
		pShop->profit_sell = read_number(fp);
		pShop->open_hour = read_number(fp);
		pShop->close_hour = read_number(fp);
		read_to_eol(fp);
		pMobIndex = get_mob_index(pShop->keeper);
		pMobIndex->pShop = pShop;
		LINK(pShop, shop_first, shop_last, next, prev);
		top_shop++;
	}

	return;
}

/*
 * Snarf spec proc declarations.
 */
void load_specials(READ_DATA * fp)
{
	for (;;)
	{
		MOB_INDEX_DATA *pMobIndex;
		char letter;

		switch (letter = read_letter(fp))
		{
		default:
			bugf("Load_specials: letter '%c' not *MS.", letter);
			exit(1);

		case 'S':
			return;

		case '*':
			break;

		case 'M':
			pMobIndex = get_mob_index(read_number(fp));
			pMobIndex->spec_fun = spec_lookup(read_word(fp));
			if (pMobIndex->spec_fun == 0)
			{
				bugf("Load_specials: 'M': vnum %ld.", pMobIndex->vnum);
				exit(1);
			}
			break;
		}

		read_to_eol(fp);
	}
}

/*
 * Translate all room exits from virtual to real.
 * Has to be done after all rooms are read in.
 * Check for bad reverse exits.
 */
void fix_exits(void)
{
	ROOM_INDEX_DATA *pRoomIndex;
	/* Remove Markanth - 21/09/02
	   char buf[MAX_STRING_LENGTH];
	   ROOM_INDEX_DATA *to_room;
	   EXIT_DATA *pexit_rev; */
	EXIT_DATA *pexit;
	RESET_DATA *pReset;
	ROOM_INDEX_DATA *iLastRoom, *iLastObj;
	int iHash;
	int door;

	for (iHash = 0; iHash < MAX_KEY_HASH; iHash++)
	{
		for (pRoomIndex = room_index_hash[iHash]; pRoomIndex != NULL;
			 pRoomIndex = pRoomIndex->next)
		{
			bool fexit;

			iLastRoom = iLastObj = NULL;

			/* OLC : nuevo chequeo de resets */
			for (pReset = pRoomIndex->reset_first; pReset;
				 pReset = pReset->next)
			{
				switch (pReset->command)
				{
				default:
					bugf("fix_exits : cuarto %ld con reset cmd %c",
						 pRoomIndex->vnum, pReset->command);
					exit(1);
					break;

				case 'M':
					get_mob_index(pReset->arg1);
					iLastRoom = get_room_index(pReset->arg3);
					break;

				case 'O':
					get_obj_index(pReset->arg1);
					iLastObj = get_room_index(pReset->arg3);
					break;

				case 'P':
					get_obj_index(pReset->arg1);
					if (iLastObj == NULL)
					{
						bugf
							("fix_exits : reset en cuarto %ld con iLastObj NULL",
							 pRoomIndex->vnum);
						exit(1);
					}
					break;

				case 'G':
				case 'E':
					get_obj_index(pReset->arg1);
					if (iLastRoom == NULL)
					{
						bugf
							("fix_exits : reset en cuarto %ld con iLastRoom NULL",
							 pRoomIndex->vnum);
						exit(1);
					}
					iLastObj = iLastRoom;
					break;

				case 'D':
					bugf("???");
					break;

				case 'R':
					get_room_index(pReset->arg1);
					if (pReset->arg2 < 0 || pReset->arg2 > MAX_DIR)
					{
						bugf
							("fix_exits : reset en cuarto %ld con arg2 %d >= MAX_DIR",
							 pRoomIndex->vnum, pReset->arg2);
						exit(1);
					}
					break;
				}				/* switch */
			}					/* for */

			fexit = FALSE;
			for (door = 0; door <= 5; door++)
			{
				if ((pexit = pRoomIndex->exit[door]) != NULL)
				{
					if (pexit->u1.vnum <= 0 ||
						get_room_index(pexit->u1.vnum) == NULL)
						pexit->u1.to_room = NULL;
					else
					{
						fexit = TRUE;
						pexit->u1.to_room = get_room_index(pexit->u1.vnum);
					}
				}
			}
			if (!fexit)
				SET_BIT(pRoomIndex->room_flags, ROOM_NO_MOB);
		}
	}

	/* Removed - Markanth 21/09/02
	   for (iHash = 0; iHash < MAX_KEY_HASH; iHash++)
	   {
	   for (pRoomIndex = room_index_hash[iHash]; pRoomIndex != NULL;
	   pRoomIndex = pRoomIndex->next)
	   {
	   for (door = 0; door <= 5; door++)
	   {
	   if ((pexit = pRoomIndex->exit[door]) != NULL &&
	   (to_room = pexit->u1.to_room) != NULL &&
	   (pexit_rev =
	   to_room->exit[rev_dir[door]]) != NULL
	   && pexit_rev->u1.to_room != pRoomIndex
	   && (pRoomIndex->vnum < 1200
	   || pRoomIndex->vnum > 1299))
	   {
	   sprintf (buf,
	   "Fix_exits: %ld:%d -> %ld:%d -> %ld.",
	   pRoomIndex->vnum, door,
	   to_room->vnum, rev_dir[door],
	   (pexit_rev->u1.to_room ==
	   NULL) ? 0 : pexit_rev->u1.
	   to_room->vnum);
	   bug (buf, 0);
	   }
	   }
	   }
	   }
	 */

	return;
}

/*
 * Load mobprogs section
 */
void load_mobprogs(READ_DATA * fp)
{
	PROG_CODE *pMprog;

	if (current_area == NULL)
	{
		bug("Load_mobprogs: no #AREA seen yet.");
		exit(1);
	}

	for (;;)
	{
		vnum_t vnum;
		char letter;

		letter = read_letter(fp);
		if (letter != '#')
		{
			bug("Load_mobprogs: # not found.");
			exit(1);
		}

		vnum = read_number(fp);
		if (vnum == 0)
			break;

		run_level = RUNLEVEL_BOOT_NOEXIT;
		if (get_prog_index(vnum, PRG_MPROG) != NULL)
		{
			bugf("Load_mobprogs: vnum %ld duplicated.", vnum);
			exit(1);
		}
		run_level = RUNLEVEL_BOOTING;

		pMprog = new_pcode();
		pMprog->vnum = vnum;
		free_read_string(pMprog->code, fp);
		pMprog->area = current_area;
		LINK(pMprog, mprog_first, mprog_last, next, prev);
		top_mprog_index++;
	}
	return;
}

/*
 *  Translate mobprog vnums pointers to real code
 */
void fix_mobprogs(void)
{
	MOB_INDEX_DATA *pMobIndex;
	PROG_LIST *list;
	PROG_CODE *prog;
	int iHash;

	for (iHash = 0; iHash < MAX_KEY_HASH; iHash++)
	{
		for (pMobIndex = mob_index_hash[iHash]; pMobIndex != NULL;
			 pMobIndex = pMobIndex->next)
		{
			for (list = pMobIndex->first_mprog; list != NULL; list = list->next)
			{
				if ((prog = get_prog_index(list->vnum, PRG_MPROG)) != NULL)
					replace_string(list->code, prog->code);
				else
				{
					bugf("Fix_mobprogs: code vnum %ld not found.", list->vnum);
					exit(1);
				}
			}
		}
	}
}

/*
 * Repopulate areas periodically.
 */
void area_update(void)
{
	AREA_DATA *pArea;
	char buf[MAX_STRING_LENGTH];
	int hash;
	ROOM_INDEX_DATA *room;

	for (pArea = area_first; pArea != NULL; pArea = pArea->next)
	{

		if (++pArea->age < 3)
			continue;

		/*
		 * Check age and reset.
		 * Note: Mud School resets every 3 minutes (not 15).
		 */
		if ((!pArea->empty && (pArea->nplayer == 0 || pArea->age >= 15))
			|| pArea->age >= 31)
		{
			ROOM_INDEX_DATA *pRoomIndex;

			reset_area(pArea);
			sprintf(buf, "%s has just been reset.", pArea->name);
			wiznet(buf, NULL, NULL, WIZ_RESETS, 0, 0);

			pArea->age = number_range(0, 3);
			pRoomIndex = get_room_index(ROOM_VNUM_SCHOOL);
			if (pRoomIndex != NULL && pArea == pRoomIndex->area)
				pArea->age = 15 - 2;
			else if (pArea->nplayer == 0)
				pArea->empty = TRUE;
		}
	}

	/*
	 * ROOMprog Triggers!
	 */
	for (hash = 0; hash < MAX_KEY_HASH; hash++)
		for (room = room_index_hash[hash]; room; room = room->next)
		{
			if (room->area->empty)
				continue;

			if (HAS_TRIGGER_ROOM(room, TRIG_DELAY) && room->rprog_delay > 0)
			{
				if (--room->rprog_delay <= 0)
					p_percent_trigger(NULL, NULL, room, NULL, NULL, NULL,
									  TRIG_DELAY);
			}
			else if (HAS_TRIGGER_ROOM(room, TRIG_RANDOM))
				p_percent_trigger(NULL, NULL, room, NULL, NULL, NULL,
								  TRIG_RANDOM);
		}
	return;
}

/* OLC
 * Reset one room.  Called by reset_area and olc.
 */
void reset_room(ROOM_INDEX_DATA * pRoom)
{
	RESET_DATA *pReset;
	CHAR_DATA *pMob;
	CHAR_DATA *mob;
	OBJ_DATA *pObj;
	CHAR_DATA *LastMob = NULL;
	OBJ_DATA *LastObj = NULL;
	int iExit;
	int level = 0;
	bool last;

	if (!pRoom)
		return;

	pMob = NULL;
	last = FALSE;

	for (iExit = 0; iExit < MAX_DIR; iExit++)
	{
		EXIT_DATA *pExit;
		if ((pExit = pRoom->exit[iExit])
			/*  && !IS_SET( pExit->exit_info, EX_BASHED )   ROM OLC */ )
		{
			pExit->exit_info = pExit->rs_flags;
			if ((pExit->u1.to_room != NULL) &&
				((pExit = pExit->u1.to_room->exit[rev_dir[iExit]])))
			{
				/* nail the other side */
				pExit->exit_info = pExit->rs_flags;
			}
		}
	}

	for (pReset = pRoom->reset_first; pReset != NULL; pReset = pReset->next)
	{
		MOB_INDEX_DATA *pMobIndex;
		OBJ_INDEX_DATA *pObjIndex;
		OBJ_INDEX_DATA *pObjToIndex;
		ROOM_INDEX_DATA *pRoomIndex;
		char buf[MAX_STRING_LENGTH];
		int count, limit = 0;

		switch (pReset->command)
		{
		default:
			bugf("Reset_room: bad command %c.", pReset->command);
			break;

		case 'M':
			if (!(pMobIndex = get_mob_index(pReset->arg1)))
			{
				bugf("Reset_room: 'M': bad vnum %ld.", pReset->arg1);
				continue;
			}

			if ((pRoomIndex = get_room_index(pReset->arg3)) == NULL)
			{
				bugf("Reset_area: 'R': bad vnum %ld.", pReset->arg3);
				continue;
			}
			if (pMobIndex->count >= pReset->arg2)
			{
				last = FALSE;
				break;
			}
/* */
			count = 0;
			for (mob = pRoomIndex->first_person; mob != NULL;
				 mob = mob->next_in_room)
				if (mob->pIndexData == pMobIndex)
				{
					count++;
					if (count >= pReset->arg4)
					{
						last = FALSE;
						break;
					}
				}

			if (count >= pReset->arg4)
				break;

/* */

			pMob = create_mobile(pMobIndex);

			/*
			 * Some more hard coding.
			 */
			if (room_is_dark(pRoom))
				SET_BIT(pMob->affected_by, AFF_INFRARED);

			/*
			 * Pet shop mobiles get ACT_PET set.
			 */
			{
				ROOM_INDEX_DATA *pRoomIndexPrev;

				pRoomIndexPrev = get_room_index(pRoom->vnum - 1);
				if (pRoomIndexPrev &&
					IS_SET(pRoomIndexPrev->room_flags, ROOM_PET_SHOP))
					SET_BIT(pMob->act, ACT_PET);
			}

			char_to_room(pMob, pRoom);

			LastMob = pMob;
			level = URANGE(0, pMob->level - 2, MAX_MORTAL_LEVEL - 1);	/* -1 ROM */
			last = TRUE;
			break;

		case 'O':
			if (!(pObjIndex = get_obj_index(pReset->arg1)))
			{
				bugf("Reset_room: 'O' 1 : bad vnum %ld", pReset->arg1);
				sprintf(buf, "%ld %d %ld %d", pReset->arg1,
						pReset->arg2, pReset->arg3, pReset->arg4);
				bugf(buf, 1);
				continue;
			}

			if (!(pRoomIndex = get_room_index(pReset->arg3)))
			{
				bugf("Reset_room: 'O' 2 : bad vnum %ld.", pReset->arg3);
				sprintf(buf, "%ld %d %ld %d", pReset->arg1,
						pReset->arg2, pReset->arg3, pReset->arg4);
				bugf(buf, 1);
				continue;
			}

			if (pRoom->area->nplayer > 0 ||
				count_obj_list(pObjIndex, pRoom->first_content) > 0)
			{
				last = FALSE;
				break;
			}

			pObj = create_object(pObjIndex,	/* UMIN - ROM OLC */
								 UMIN(number_fuzzy(level),
									  MAX_MORTAL_LEVEL - 1));
			pObj->cost = 0;
			obj_to_room(pObj, pRoom);
			last = TRUE;
			break;

		case 'P':
			if (!(pObjIndex = get_obj_index(pReset->arg1)))
			{
				bugf("Reset_room: 'P': bad vnum %ld.", pReset->arg1);
				continue;
			}

			if (!(pObjToIndex = get_obj_index(pReset->arg3)))
			{
				bugf("Reset_room: 'P': bad vnum %ld.", pReset->arg3);
				continue;
			}

			if (pReset->arg2 > 50)	/* old format */
				limit = 6;
			else if (pReset->arg2 == -1)	/* no limit */
				limit = 999;
			else
				limit = pReset->arg2;

			if (pRoom->area->nplayer > 0
				|| (LastObj = get_obj_type(pObjToIndex)) == NULL
				|| (LastObj->in_room == NULL && !last) || (pObjIndex->count >= limit	/* && number_range(0,4) != 0 */
				)
				|| (count =
					count_obj_list(pObjIndex,
								   LastObj->first_content)) > pReset->arg4)
			{
				last = FALSE;
				break;
			}
			/* lastObj->level  -  ROM */

			while (count < pReset->arg4)
			{
				pObj = create_object(pObjIndex, number_fuzzy(LastObj->level));
				obj_to_obj(pObj, LastObj);
				count++;
				if (pObjIndex->count >= limit)
					break;
			}

			/* fix object lock state! */
			LastObj->value[1] = LastObj->pIndexData->value[1];
			last = TRUE;
			break;

		case 'G':
		case 'E':
			if (!(pObjIndex = get_obj_index(pReset->arg1)))
			{
				bugf("Reset_room: 'E' or 'G': bad vnum %ld.", pReset->arg1);
				continue;
			}

			if (!last)
				break;

			if (!LastMob)
			{
				bugf("Reset_room: 'E' or 'G': null mob for vnum %ld.",
					 pReset->arg1);
				last = FALSE;
				break;
			}

			if (LastMob->pIndexData->pShop)	/* Shop-keeper? */
			{
				int olevel = 0, i, j;

				if (!pObjIndex->new_format)
					switch (pObjIndex->item_type)
					{
					default:
						olevel = 0;
						break;
					case ITEM_PILL:
					case ITEM_POTION:
					case ITEM_SCROLL:
						olevel = ANGEL;
						for (i = 1; i < 5; i++)
						{
							if (pObjIndex->value[i] > 0)
							{
								for (j = 0; j < maxClass; j++)
								{
									olevel =
										UMIN
										(olevel,
										 skill_table
										 [pObjIndex->value[i]].skill_level[j]);
								}
							}
						}

						olevel = UMAX(0, (olevel * 3 / 4) - 2);
						break;

					case ITEM_WAND:
						olevel = number_range(10, 20);
						break;
					case ITEM_STAFF:
						olevel = number_range(15, 25);
						break;
					case ITEM_ARMOR:
						olevel = number_range(5, 15);
						break;
						/* ROM patch weapon, treasure */
					case ITEM_WEAPON:
						olevel = number_range(5, 15);
						break;
					case ITEM_TREASURE:
						olevel = number_range(10, 20);
						break;

					}

				pObj = create_object(pObjIndex, olevel);
				SET_BIT(pObj->extra_flags, ITEM_INVENTORY);	/* ROM OLC */

			}
			else				/* ROM OLC else version */
			{
				int plimit;
				if (pReset->arg2 > 50)	/* old format */
					plimit = 6;
				else if (pReset->arg2 == -1 || pReset->arg2 == 0)	/* no limit */
					plimit = 999;
				else
					plimit = pReset->arg2;

				if (pObjIndex->count < plimit || number_range(0, 4) == 0)
				{
					pObj =
						create_object(pObjIndex,
									  UMIN(number_fuzzy
										   (level), MAX_MORTAL_LEVEL - 1));
					/* error message if it is too high 
					   if (pObj->level > LastMob->level + 3
					   ||  (pObj->item_type == ITEM_WEAPON 
					   &&   pReset->command == 'E' 
					   &&   pObj->level < LastMob->level -5 && pObj->level < 45))
					   logf(
					   "Err: obj %s (%d) -- %d, mob %s (%d) -- %d\n",
					   pObj->short_descr,pObj->pIndexData->vnum,pObj->level,
					   LastMob->short_descr,LastMob->pIndexData->vnum,LastMob->level);
					 */
				}
				else
					break;
			}

			obj_to_char(pObj, LastMob);
			if (pReset->command == 'E')
				equip_char(LastMob, pObj, (wloc_t) pReset->arg3);
			last = TRUE;
			break;

		case 'D':
			break;

		case 'R':
			if (!(pRoomIndex = get_room_index(pReset->arg1)))
			{
				bugf("Reset_room: 'R': bad vnum %ld.", pReset->arg1);
				continue;
			}

			{
				EXIT_DATA *pExit;
				int d0;
				int d1;

				switch (pReset->arg3)
				{
				default:
					for (d0 = 0; d0 < pReset->arg2 - 1; d0++)
					{
						d1 = number_range(d0, pReset->arg2 - 1);
						pExit = pRoomIndex->exit[d0];
						pRoomIndex->exit[d0] = pRoomIndex->exit[d1];
						pRoomIndex->exit[d1] = pExit;
					}
					break;
				case 1:
					add_random_exit(pRoomIndex, pReset, TRUE);
					break;
				case 2:
					add_random_exit(pRoomIndex, pReset, FALSE);
					break;
				}
			}
			break;
		}
	}

	return;
}

/* OLC
 * Reset one area.
 */
void reset_area(AREA_DATA * pArea)
{
	ROOM_INDEX_DATA *pRoom;
	vnum_t vnum;

	for (vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++)
	{
		if ((pRoom = get_room_index(vnum)))
			reset_room(pRoom);
	}

	return;
}

/*
 * Create an instance of a mobile.
 */
CHAR_DATA *create_mobile(MOB_INDEX_DATA * pMobIndex)
{
	CHAR_DATA *mob;
	int i;
	AFFECT_DATA af;

	mobile_count++;

	if (pMobIndex == NULL)
	{
		bug("Create_mobile: NULL pMobIndex.");
		exit(1);
	}

	mob = new_char();

	mob->pIndexData = pMobIndex;

	mob->name = str_dup(pMobIndex->player_name);	/* OLC */
	mob->short_descr = str_dup(pMobIndex->short_descr);	/* OLC */
	mob->long_descr = str_dup(pMobIndex->long_descr);	/* OLC */
	mob->description = str_dup(pMobIndex->description);	/* OLC */
	mob->id = get_mob_id();
	mob->spec_fun = pMobIndex->spec_fun;
	mob->prompt = NULL;
	mob->mprog_target = NULL;

	if (pMobIndex->wealth == 0)
	{
		mob->silver = 0;
		mob->gold = 0;
	}
	else
	{
		long wealth;

		wealth = number_range(pMobIndex->wealth / 2, 3 * pMobIndex->wealth / 2);
		mob->gold = number_range(wealth / 200, wealth / 100);
		mob->silver = wealth - (mob->gold * 100);
	}

	if (pMobIndex->new_format)
		/* load in new style */
	{
		/* read from prototype */
		mob->group = pMobIndex->group;
		mob->act = pMobIndex->act;
		mob->comm = COMM_NOCHANNELS | COMM_NOSHOUT | COMM_NOTELL;
		mob->affected_by = pMobIndex->affected_by;
		mob->alignment = pMobIndex->alignment;
		mob->level = pMobIndex->level;
		mob->hitroll = pMobIndex->hitroll;
		mob->damroll = pMobIndex->damage[DICE_BONUS];
		mob->max_hit =
			dice(pMobIndex->hit[DICE_NUMBER],
				 pMobIndex->hit[DICE_TYPE]) + pMobIndex->hit[DICE_BONUS];
		mob->hit = mob->max_hit;
		mob->max_mana =
			dice(pMobIndex->mana[DICE_NUMBER],
				 pMobIndex->mana[DICE_TYPE]) + pMobIndex->mana[DICE_BONUS];
		mob->mana = mob->max_mana;
		mob->damage[DICE_NUMBER] = pMobIndex->damage[DICE_NUMBER];
		mob->damage[DICE_TYPE] = pMobIndex->damage[DICE_TYPE];
		mob->dam_type = pMobIndex->dam_type;
		if (mob->dam_type == 0)
			switch (number_range(1, 3))
			{
			case (1):
				mob->dam_type = 3;
				break;			/* slash */
			case (2):
				mob->dam_type = 7;
				break;			/* pound */
			case (3):
				mob->dam_type = 11;
				break;			/* pierce */
			}
		for (i = 0; i < 4; i++)
			mob->armor[i] = pMobIndex->ac[i];
		mob->off_flags = pMobIndex->off_flags;
		mob->imm_flags = pMobIndex->imm_flags;
		mob->res_flags = pMobIndex->res_flags;
		mob->vuln_flags = pMobIndex->vuln_flags;
		mob->start_pos = pMobIndex->start_pos;
		mob->default_pos = pMobIndex->default_pos;
		mob->sex = pMobIndex->sex;
		if (mob->sex == 3)		/* random sex */
			mob->sex = number_range(1, 2);
		mob->race = pMobIndex->race;
		mob->form = pMobIndex->form;
		mob->parts = pMobIndex->parts;
		mob->size = pMobIndex->size;
		mob->material = str_dup(pMobIndex->material);

		/* computed on the spot */

		for (i = 0; i < MAX_STATS; i++)
			mob->perm_stat[i] = UMIN(25, 11 + mob->level / 4);

		{
			int stance = number_range(STANCE_NORMAL, STANCE_SWALLOW);
			SET_STANCE(mob, STANCE_CURRENT, STANCE_NONE);
			SET_STANCE(mob, STANCE_AUTODROP, stance);
		}

		if (IS_SET(mob->act, ACT_WARRIOR))
		{
			mob->perm_stat[STAT_STR] += 3;
			mob->perm_stat[STAT_INT] -= 1;
			mob->perm_stat[STAT_CON] += 2;
		}

		if (IS_SET(mob->act, ACT_THIEF))
		{
			mob->perm_stat[STAT_DEX] += 3;
			mob->perm_stat[STAT_INT] += 1;
			mob->perm_stat[STAT_WIS] -= 1;
		}

		if (IS_SET(mob->act, ACT_CLERIC))
		{
			mob->perm_stat[STAT_WIS] += 3;
			mob->perm_stat[STAT_DEX] -= 1;
			mob->perm_stat[STAT_STR] += 1;
		}

		if (IS_SET(mob->act, ACT_MAGE))
		{
			mob->perm_stat[STAT_INT] += 3;
			mob->perm_stat[STAT_STR] -= 1;
			mob->perm_stat[STAT_DEX] += 1;
		}

		if (IS_SET(mob->off_flags, OFF_FAST))
			mob->perm_stat[STAT_DEX] += 2;

		mob->perm_stat[STAT_STR] += mob->size - SIZE_MEDIUM;
		mob->perm_stat[STAT_CON] += (mob->size - SIZE_MEDIUM) / 2;

		/* let's get some spell action */
		if (IS_AFFECTED(mob, AFF_SANCTUARY))
		{
			af.where = TO_AFFECTS;
			af.type = skill_lookup("sanctuary");
			af.level = mob->level;
			af.duration = -1;
			af.location = APPLY_NONE;
			af.modifier = 0;
			af.bitvector = AFF_SANCTUARY;
			affect_to_char(mob, &af);
		}

		if (IS_AFFECTED(mob, AFF_HASTE))
		{
			af.where = TO_AFFECTS;
			af.type = skill_lookup("haste");
			af.level = mob->level;
			af.duration = -1;
			af.location = APPLY_DEX;
			af.modifier =
				1 + (mob->level >= 18) + (mob->level >=
										  25) + (mob->level >= 32);
			af.bitvector = AFF_HASTE;
			affect_to_char(mob, &af);
		}

		if (IS_AFFECTED(mob, AFF_PROTECT_EVIL))
		{
			af.where = TO_AFFECTS;
			af.type = skill_lookup("protection evil");
			af.level = mob->level;
			af.duration = -1;
			af.location = APPLY_SAVES;
			af.modifier = -1;
			af.bitvector = AFF_PROTECT_EVIL;
			affect_to_char(mob, &af);
		}

		if (IS_AFFECTED(mob, AFF_PROTECT_GOOD))
		{
			af.where = TO_AFFECTS;
			af.type = skill_lookup("protection good");
			af.level = mob->level;
			af.duration = -1;
			af.location = APPLY_SAVES;
			af.modifier = -1;
			af.bitvector = AFF_PROTECT_GOOD;
			affect_to_char(mob, &af);
		}
	}
	else						/* read in old format and convert */
	{
		mob->act = pMobIndex->act;
		mob->affected_by = pMobIndex->affected_by;
		mob->alignment = pMobIndex->alignment;
		mob->level = pMobIndex->level;
		mob->hitroll = pMobIndex->hitroll;
		mob->damroll = 0;
		mob->max_hit =
			mob->level * 8 + number_range(mob->level * mob->level / 4,
										  mob->level * mob->level);
		mob->max_hit = mob->max_hit * 9 / 10;
		mob->hit = mob->max_hit;
		mob->max_mana = 100 + dice(mob->level, 10);
		mob->mana = mob->max_mana;
		switch (number_range(1, 3))
		{
		case (1):
			mob->dam_type = 3;
			break;				/* slash */
		case (2):
			mob->dam_type = 7;
			break;				/* pound */
		case (3):
			mob->dam_type = 11;
			break;				/* pierce */
		}
		for (i = 0; i < 3; i++)
			mob->armor[i] = interpolate(mob->level, 100, -100);
		mob->armor[3] = interpolate(mob->level, 100, 0);
		mob->race = pMobIndex->race;
		mob->off_flags = pMobIndex->off_flags;
		mob->imm_flags = pMobIndex->imm_flags;
		mob->res_flags = pMobIndex->res_flags;
		mob->vuln_flags = pMobIndex->vuln_flags;
		mob->start_pos = pMobIndex->start_pos;
		mob->default_pos = pMobIndex->default_pos;
		mob->sex = pMobIndex->sex;
		mob->form = pMobIndex->form;
		mob->parts = pMobIndex->parts;
		mob->size = SIZE_MEDIUM;
		mob->material = "";

		for (i = 0; i < MAX_STATS; i++)
			mob->perm_stat[i] = 11 + mob->level / 4;
	}

	mob->position = mob->start_pos;

	/* link the mob to the world list */
	LINK(mob, char_first, char_last, next, prev);
	pMobIndex->count++;
	return mob;
}

/* duplicate a mobile exactly -- except inventory */
void clone_mobile(CHAR_DATA * parent, CHAR_DATA * clone)
{
	int i;
	AFFECT_DATA *paf;

	if (parent == NULL || clone == NULL || !IS_NPC(parent))
		return;

	/* start fixing values */
	replace_string(clone->name, parent->name);
	clone->version = parent->version;
	replace_string(clone->short_descr, parent->short_descr);
	replace_string(clone->long_descr, parent->long_descr);
	replace_string(clone->description, parent->description);
	clone->group = parent->group;
	clone->sex = parent->sex;
	for (i = 0; i < MAX_MCLASS; i++)
		clone->Class[i] = parent->Class[i];
	clone->race = parent->race;
	clone->level = parent->level;
	clone->trust = 0;
	clone->timer = parent->timer;
	clone->wait = parent->wait;
	clone->hit = parent->hit;
	clone->max_hit = parent->max_hit;
	clone->mana = parent->mana;
	clone->max_mana = parent->max_mana;
	clone->move = parent->move;
	clone->max_move = parent->max_move;
	clone->gold = parent->gold;
	clone->silver = parent->silver;
	clone->exp = parent->exp;
	clone->act = parent->act;
	clone->comm = parent->comm;
	clone->imm_flags = parent->imm_flags;
	clone->res_flags = parent->res_flags;
	clone->vuln_flags = parent->vuln_flags;
	clone->invis_level = parent->invis_level;
	clone->affected_by = parent->affected_by;
	clone->position = parent->position;
	clone->practice = parent->practice;
	clone->train = parent->train;
	clone->saving_throw = parent->saving_throw;
	clone->alignment = parent->alignment;
	clone->hitroll = parent->hitroll;
	clone->damroll = parent->damroll;
	clone->wimpy = parent->wimpy;
	clone->form = parent->form;
	clone->parts = parent->parts;
	clone->size = parent->size;
	replace_string(clone->material, parent->material);
	clone->off_flags = parent->off_flags;
	clone->dam_type = parent->dam_type;
	clone->start_pos = parent->start_pos;
	clone->default_pos = parent->default_pos;
	clone->spec_fun = parent->spec_fun;

	for (i = 0; i < 4; i++)
		clone->armor[i] = parent->armor[i];

	for (i = 0; i < MAX_STATS; i++)
	{
		clone->perm_stat[i] = parent->perm_stat[i];
		clone->mod_stat[i] = parent->mod_stat[i];
	}

	for (i = 0; i < 3; i++)
		clone->damage[i] = parent->damage[i];

	/* now add the affects */
	for (paf = parent->first_affect; paf != NULL; paf = paf->next)
		affect_to_char(clone, paf);

}

/*
 * Create an instance of an object.
 */
OBJ_DATA *create_object(OBJ_INDEX_DATA * pObjIndex, int level)
{
	AFFECT_DATA *paf;
	OBJ_DATA *obj;
	int i;

	if (pObjIndex == NULL)
	{
		bug("Create_object: NULL pObjIndex.");
		exit(1);
	}

	obj = new_obj();

	obj->pIndexData = pObjIndex;
	obj->in_room = NULL;
	obj->enchanted = FALSE;

	if (pObjIndex->new_format)
		obj->level = pObjIndex->level;
	else
		obj->level = UMAX(0, level);
	obj->wear_loc = WEAR_NONE;

	obj->name = str_dup(pObjIndex->name);	/* OLC */
	obj->short_descr = str_dup(pObjIndex->short_descr);	/* OLC */
	obj->description = str_dup(pObjIndex->description);	/* OLC */
	obj->material = str_dup(pObjIndex->material);
	obj->item_type = pObjIndex->item_type;
	obj->extra_flags = pObjIndex->extra_flags;
	obj->wear_flags = pObjIndex->wear_flags;
	obj->value[0] = pObjIndex->value[0];
	obj->value[1] = pObjIndex->value[1];
	obj->value[2] = pObjIndex->value[2];
	obj->value[3] = pObjIndex->value[3];
	obj->value[4] = pObjIndex->value[4];
	obj->weight = pObjIndex->weight;

	if (level == -1 || pObjIndex->new_format)
		obj->cost = pObjIndex->cost;
	else
		obj->cost =
			number_fuzzy(10) * number_fuzzy(level) * number_fuzzy(level);

	/*
	 * Mess with object properties.
	 */
	switch (obj->item_type)
	{
	default:
		bugf("Read_object: vnum %ld bad type.", pObjIndex->vnum);
		break;

	case ITEM_LIGHT:
		if (obj->value[2] == 999)
			obj->value[2] = -1;
		break;

	case ITEM_FURNITURE:
	case ITEM_TRASH:
	case ITEM_CONTAINER:
	case ITEM_DRINK_CON:
	case ITEM_KEY:
	case ITEM_FOOD:
	case ITEM_BOAT:
	case ITEM_CORPSE_NPC:
	case ITEM_CORPSE_PC:
	case ITEM_FOUNTAIN:
	case ITEM_MAP:
	case ITEM_CLOTHING:
	case ITEM_PORTAL:
		if (!pObjIndex->new_format)
			obj->cost /= 5;
		break;

	case ITEM_TREASURE:
	case ITEM_WARP_STONE:
	case ITEM_ROOM_KEY:
	case ITEM_GEM:
	case ITEM_JEWELRY:
		break;

	case ITEM_JUKEBOX:
		for (i = 0; i < 5; i++)
			obj->value[i] = -1;
		break;

	case ITEM_SCROLL:
		if (level != -1 && !pObjIndex->new_format)
			obj->value[0] = number_fuzzy(obj->value[0]);
		break;

	case ITEM_WAND:
	case ITEM_STAFF:
		if (level != -1 && !pObjIndex->new_format)
		{
			obj->value[0] = number_fuzzy(obj->value[0]);
			obj->value[1] = number_fuzzy(obj->value[1]);
			obj->value[2] = obj->value[1];
		}
		if (!pObjIndex->new_format)
			obj->cost *= 2;
		break;

	case ITEM_WEAPON:
		if (level != -1 && !pObjIndex->new_format)
		{
			obj->value[1] = number_fuzzy(number_fuzzy(1 * level / 4 + 2));
			obj->value[2] = number_fuzzy(number_fuzzy(3 * level / 4 + 6));
		}
		break;

	case ITEM_ARMOR:
		if (level != -1 && !pObjIndex->new_format)
		{
			obj->value[0] = number_fuzzy(level / 5 + 3);
			obj->value[1] = number_fuzzy(level / 5 + 3);
			obj->value[2] = number_fuzzy(level / 5 + 3);
		}
		break;

	case ITEM_POTION:
	case ITEM_PILL:
		if (level != -1 && !pObjIndex->new_format)
			obj->value[0] = number_fuzzy(number_fuzzy(obj->value[0]));
		break;

	case ITEM_MONEY:
		if (!pObjIndex->new_format)
			obj->value[0] = obj->cost;
		break;
	}

	for (paf = pObjIndex->first_affect; paf != NULL; paf = paf->next)
		if (paf->location == APPLY_SPELL_AFFECT)
			affect_to_obj(obj, paf);

	LINK(obj, object_first, object_last, next, prev);
	pObjIndex->count++;

	return obj;
}

/* duplicate an object exactly -- except contents */
void clone_object(OBJ_DATA * parent, OBJ_DATA * clone)
{
	int i;
	AFFECT_DATA *paf;
	EXTRA_DESCR_DATA *ed, *ed_new;

	if (parent == NULL || clone == NULL)
		return;

	/* start fixing the object */
	replace_string(clone->name, parent->name);
	replace_string(clone->short_descr, parent->short_descr);
	replace_string(clone->description, parent->description);
	clone->item_type = parent->item_type;
	clone->extra_flags = parent->extra_flags;
	clone->wear_flags = parent->wear_flags;
	clone->weight = parent->weight;
	clone->cost = parent->cost;
	clone->level = parent->level;
	clone->condition = parent->condition;
	replace_string(clone->material, parent->material);
	clone->timer = parent->timer;

	for (i = 0; i < 5; i++)
		clone->value[i] = parent->value[i];

	/* affects */
	clone->enchanted = parent->enchanted;

	for (paf = parent->first_affect; paf != NULL; paf = paf->next)
		affect_to_obj(clone, paf);

	/* extended desc */
	for (ed = parent->first_extra_descr; ed != NULL; ed = ed->next)
	{
		ed_new = new_extra_descr();
		ed_new->keyword = str_dup(ed->keyword);
		ed_new->description = str_dup(ed->description);
		LINK(ed_new, clone->first_extra_descr, clone->last_extra_descr, next,
			 prev);
	}

}

/*
 * Clear a new character.
 */
void clear_char(CHAR_DATA * ch)
{
	static CHAR_DATA ch_zero;
	int i;

	*ch = ch_zero;
	ch->name = &str_empty[0];
	ch->short_descr = &str_empty[0];
	ch->long_descr = &str_empty[0];
	ch->description = &str_empty[0];
	ch->prompt = &str_empty[0];
	ch->logon = current_time;
	ch->lines = 0;
	ch->columns = 0;
	for (i = 0; i < 4; i++)
		ch->armor[i] = 100;
	ch->position = POS_STANDING;
	ch->hit = 20;
	ch->max_hit = 20;
	ch->mana = 100;
	ch->max_mana = 100;
	ch->move = 100;
	ch->max_move = 100;
	ch->on = NULL;
	for (i = 0; i < MAX_STATS; i++)
	{
		ch->perm_stat[i] = 13;
		ch->mod_stat[i] = 0;
	}
	return;
}

/*
 * Get an extra description from a list.
 */
const char *get_extra_descr(const char *name, EXTRA_DESCR_DATA * ed)
{
	for (; ed != NULL; ed = ed->next)
	{
		if (is_name(name, ed->keyword))
			return ed->description;
	}
	return NULL;
}

/*
 * Translates mob virtual number to its mob index struct.
 * Hash table lookup.
 */
MOB_INDEX_DATA *get_mob_index(vnum_t vnum)
{
	MOB_INDEX_DATA *pMobIndex;

	for (pMobIndex = mob_index_hash[vnum % MAX_KEY_HASH]; pMobIndex != NULL;
		 pMobIndex = pMobIndex->next)
	{
		if (pMobIndex->vnum == vnum)
			return pMobIndex;
	}

	if (run_level == RUNLEVEL_BOOTING)
	{
		bugf("Get_mob_index: bad vnum %ld.", vnum);
		exit(1);
	}

	return NULL;
}

/*
 * Translates mob virtual number to its obj index struct.
 * Hash table lookup.
 */
OBJ_INDEX_DATA *get_obj_index(vnum_t vnum)
{
	OBJ_INDEX_DATA *pObjIndex;

	for (pObjIndex = obj_index_hash[vnum % MAX_KEY_HASH]; pObjIndex != NULL;
		 pObjIndex = pObjIndex->next)
	{
		if (pObjIndex->vnum == vnum)
			return pObjIndex;
	}

	if (run_level == RUNLEVEL_BOOTING)
	{
		bugf("Get_obj_index: bad vnum %ld.", vnum);
		exit(1);
	}

	return NULL;
}

/*
 * Translates mob virtual number to its room index struct.
 * Hash table lookup.
 */
ROOM_INDEX_DATA *get_room_index(vnum_t vnum)
{
	ROOM_INDEX_DATA *pRoomIndex;

	for (pRoomIndex = room_index_hash[vnum % MAX_KEY_HASH];
		 pRoomIndex != NULL; pRoomIndex = pRoomIndex->next)
	{
		if (pRoomIndex->vnum == vnum)
			return pRoomIndex;
	}

	if (run_level == RUNLEVEL_BOOTING)
	{
		bugf("Get_room_index: bad vnum %ld.", vnum);
		exit(1);
	}

	return NULL;
}

PROG_CODE *get_prog_index(vnum_t vnum, prog_t type)
{
	PROG_CODE *prg;

	switch (type)
	{
	case PRG_MPROG:
		prg = mprog_first;
		break;
	case PRG_OPROG:
		prg = oprog_first;
		break;
	case PRG_RPROG:
		prg = rprog_first;
		break;
	default:
		return NULL;
	}

	for (; prg; prg = prg->next)
	{
		if (prg->vnum == vnum)
			return (prg);
	}
	return NULL;
}

flag_t flag_convert(char letter)
{
	flag_t bitsum = 0;
	char i;

	if ('A' <= letter && letter <= 'Z')
	{
		bitsum = ((flag_t) 1 << 0);
		for (i = letter; i > 'A'; i--)
			bitsum <<= 1;
	}
	else if ('a' <= letter && letter <= 'z')
	{
		bitsum = ((flag_t) 1 << 26);
		for (i = letter; i > 'a'; i--)
			bitsum <<= 1;
	}

	return bitsum;
}

/*
 * Duplicate a string into dynamic memory.
 * Fread_strings are read-only and shared.
 */
const char *str_dup(const char *pstr)
{
	char *str_new;
	size_t size;

	if (IS_NULLSTR(pstr))
		return &str_empty[0];

	size = strlen(pstr) + 1;
	alloc_mem(str_new, char, size);

	if (str_new == NULL)
	{
		bug("Failed to grab memory for str_dup...");
		return &str_empty[0];
	}
	strcpy(str_new, pstr);
	sAllocString += size;
	nAllocString += 1;
	return str_new;
}

/*
 * Free a string.
 * Null is legal here to simplify callers.
 * Read-only shared strings are not touched.
 */
void _free_string(const char *pstr)
{
	if (IS_NULLSTR(pstr))
		return;

	sAllocString -= strlen(pstr);
	nAllocString -= 1;
	free_mem(pstr);
	return;
}

AREA_DATA *area_first_sorted;

void newarea_insert_level_sort(AREA_DATA * a)
{
	AREA_DATA *lsort = area_first_sorted;
	AREA_DATA *lsort_prev = NULL;

	if (!area_first_sorted)
	{
		area_first_sorted = a;
		return;
	}

	// sort areas by level

	for (; lsort; lsort_prev = lsort, lsort = lsort->next_sort)
	{
		if (IS_NULLSTR(a->lvl_comment))
		{
			if (IS_NULLSTR(lsort->lvl_comment))
			{
				if (lsort->min_level > 0)
				{
					if (a->min_level <= 0 || a->min_level > lsort->min_level)
					{
						continue;
					}

					if (a->min_level == lsort->min_level
						&& a->max_level > lsort->max_level)
					{
						continue;
					}
				}
				else
				{
					if (a->min_level < lsort->min_level)
					{
						continue;
					}
				}
			}
		}
		else
		{
			int i;

			if (IS_NULLSTR(lsort->lvl_comment))
			{
				continue;
			}

			i = str_cmp(a->lvl_comment, lsort->lvl_comment);
			if (i > 0)
			{
				continue;
			}

			if (i == 0 && str_cmp(a->name, lsort->name) > 0)
			{
				continue;
			}

		}

		// add area to this point in the list
		if (lsort_prev)
		{						// insert in the list 
			a->next_sort = lsort;
			lsort_prev->next_sort = a;
		}
		else					// we are at the head 
		{
			// insert at the head 
			a->next_sort = area_first_sorted;
			area_first_sorted = a;
		}
		return;
	}

	lsort_prev->next_sort = a;
}

void add_area(AREA_DATA * pArea)
{
	newarea_insert_level_sort(pArea);
	LINK(pArea, area_first, area_last, next, prev);
}

void unlink_area(AREA_DATA * pArea)
{
	UNLINK_SINGLE(pArea, next_sort, AREA_DATA, area_first_sorted);
	UNLINK(pArea, area_first, area_last, next, prev);
}

CH_CMD(do_areas)
{
	AREA_DATA *pArea1;
	AREA_DATA *pArea2;
	int iArea;
	int iAreaHalf;

	if (!IS_NULLSTR(argument))
	{
		chprintln(ch, "No argument is used with this command.");
		return;
	}

	iAreaHalf = (top_area + 1) / 2;
	pArea1 = area_first_sorted;
	pArea2 = area_first_sorted;
	for (iArea = 0; iArea < iAreaHalf; iArea++)
		pArea2 = pArea2->next_sort;

	for (iArea = 0; iArea < iAreaHalf; iArea++)
	{
		if (IS_NULLSTR(pArea1->lvl_comment))
			chprintf(ch, "[%03d %03d] %-7.7s %-30.30s ", pArea1->min_level,
					 pArea1->max_level, pArea1->credits, pArea1->name);
		else
			chprintf(ch, "[%-7.7s] %-7.7s %-30.30s ", pArea1->lvl_comment,
					 pArea1->credits, pArea1->name);

		pArea1 = pArea1->next_sort;
		if (pArea2 != NULL)
		{
			if (IS_NULLSTR(pArea2->lvl_comment))
				chprintlnf(ch, "[%03d %03d] %-7.7s %-30.30s", pArea2->min_level,
						   pArea2->max_level, pArea2->credits, pArea2->name);
			else
				chprintlnf(ch, "[%-7.7s] %-7.7s %-30.30s", pArea2->lvl_comment,
						   pArea2->credits, pArea2->name);

			pArea2 = pArea2->next_sort;
		}
		else
			chprintln(ch, "");
	}

	return;
}

#if defined(WIN32)
#pragma warning(disable:4101)
#endif

CH_CMD(do_dump)
{
	int count, count2, num_pcs, aff_count;
	CHAR_DATA *fch;
	MOB_INDEX_DATA *pMobIndex;
	PC_DATA *pc;
	OBJ_DATA *obj;
	OBJ_INDEX_DATA *pObjIndex;
	ROOM_INDEX_DATA *room;
	EXIT_DATA *pexit;
	DESCRIPTOR_DATA *d;
	AFFECT_DATA *af;
	WRITE_DATA *fp;
	vnum_t vnum;
	int nMatch = 0;

	/* open file */
	fp = open_write(LOG_DIR "mem.dmp");

	/* report use of data structures */

	num_pcs = 0;
	aff_count = 0;

	/* mobile prototypes */
	fprintf(fp->stream, "MobProt	%4d (%8d bytes)\n", top_mob_index,
			top_mob_index * (sizeof(*pMobIndex)));

	/* mobs */
	count = 0;
	count2 = 0;
	for (fch = char_first; fch != NULL; fch = fch->next)
	{
		count++;
		if (fch->pcdata != NULL)
			num_pcs++;
		for (af = fch->first_affect; af != NULL; af = af->next)
			aff_count++;
	}
	for (fch = char_free; fch != NULL; fch = fch->next)
		count2++;

	fprintf(fp->stream, "Mobs	%4d (%8d bytes), %2d free (%d bytes)\n", count,
			count * (sizeof(*fch)), count2, count2 * (sizeof(*fch)));

	/* pcdata */
	count = 0;
	for (pc = pcdata_free; pc != NULL; pc = pc->next)
		count++;

	fprintf(fp->stream, "Pcdata	%4d (%8d bytes), %2d free (%d bytes)\n",
			num_pcs, num_pcs * (sizeof(*pc)), count, count * (sizeof(*pc)));

	/* descriptors */
	count = 0;
	count2 = 0;
	for (d = descriptor_first; d != NULL; d = d->next)
		count++;
	for (d = descriptor_free; d != NULL; d = d->next)
		count2++;

	fprintf(fp->stream, "Descs	%4d (%8d bytes), %2d free (%d bytes)\n", count,
			count * (sizeof(*d)), count2, count2 * (sizeof(*d)));

	/* object prototypes */
	for (vnum = 0; nMatch < top_obj_index; vnum++)
		if ((pObjIndex = get_obj_index(vnum)) != NULL)
		{
			for (af = pObjIndex->first_affect; af != NULL; af = af->next)
				aff_count++;
			nMatch++;
		}

	fprintf(fp->stream, "ObjProt	%4d (%8d bytes)\n", top_obj_index,
			top_obj_index * (sizeof(*pObjIndex)));

	/* objects */
	count = 0;
	count2 = 0;
	for (obj = object_first; obj != NULL; obj = obj->next)
	{
		count++;
		for (af = obj->first_affect; af != NULL; af = af->next)
			aff_count++;
	}
	for (obj = obj_free; obj != NULL; obj = obj->next)
		count2++;

	fprintf(fp->stream, "Objs	%4d (%8d bytes), %2d free (%d bytes)\n", count,
			count * (sizeof(*obj)), count2, count2 * (sizeof(*obj)));

	/* affects */
	count = 0;
	for (af = affect_free; af != NULL; af = af->next)
		count++;

	fprintf(fp->stream, "Affects	%4d (%8d bytes), %2d free (%d bytes)\n",
			aff_count, aff_count * (sizeof(*af)), count, count * (sizeof(*af)));

	/* rooms */
	fprintf(fp->stream, "Rooms	%4d (%8d bytes)\n", top_room,
			top_room * (sizeof(*room)));

	/* exits */
	fprintf(fp->stream, "Exits	%4d (%8d bytes)\n", top_exit,
			top_exit * (sizeof(*pexit)));

	close_write(fp);

	/* start printing out mobile data */
	fp = open_write(LOG_DIR "mob.dmp");

	fprintf(fp->stream, "\nMobile Analysis\n");
	fprintf(fp->stream, "---------------\n");
	nMatch = 0;
	for (vnum = 0; nMatch < top_mob_index; vnum++)
		if ((pMobIndex = get_mob_index(vnum)) != NULL)
		{
			nMatch++;
			fprintf(fp->stream, "#%-4ld %3d active %3d killed     %s\n",
					pMobIndex->vnum, pMobIndex->count, pMobIndex->killed,
					pMobIndex->short_descr);
		}
	close_write(fp);

	/* start printing out object data */
	fp = open_write(LOG_DIR "obj.dmp");

	fprintf(fp->stream, "\nObject Analysis\n");
	fprintf(fp->stream, "---------------\n");
	nMatch = 0;
	for (vnum = 0; nMatch < top_obj_index; vnum++)
		if ((pObjIndex = get_obj_index(vnum)) != NULL)
		{
			nMatch++;
			fprintf(fp->stream, "#%-4ld %3d active %3d reset      %s\n",
					pObjIndex->vnum, pObjIndex->count,
					pObjIndex->reset_num, pObjIndex->short_descr);
		}

	/* close file */
	close_write(fp);
}

#if defined(WIN32)
#pragma warning(default: 4101)
#endif

/*
 * Stick a little fuzz on a number.
 */
int number_fuzzy(int number)
{
	switch (number_bits(2))
	{
	case 0:
		number -= 1;
		break;
	case 3:
		number += 1;
		break;
	}

	return UMAX(1, number);
}

int number_fuzzier(int number)
{
	switch (number_range(0, 4))
	{
	case 0:
		number -= 2;
		break;
	case 1:
		number -= 1;
		break;
	case 3:
		number += 1;
		break;
	case 4:
		number += 2;
		break;
	}

	return UMAX(1, number);
}

/*
 * Generate a random number.
 */
int number_range(int from, int to)
{
	int power;
	int number;

	if (from == 0 && to == 0)
		return 0;

	if ((to = to - from + 1) <= 1)
		return from;

	for (power = 2; power < to; power <<= 1)
		;

	while ((number = number_mm() & (power - 1)) >= to)
		;

	return from + number;
}

/*
 * Generate a percentile roll.
 */
int number_percent(void)
{
	int percent;

	while ((percent = number_mm() & (128 - 1)) > 99)
		;

	return 1 + percent;
}

/*
 * Generate a random door.
 */
int number_door(void)
{
	int door;

	while ((door = number_mm() & (8 - 1)) > 5)
		;

	return door;
}

int number_bits(int width)
{
	return number_mm() & ((1 << width) - 1);
}

/*
 * I've gotten too many bad reports on OS-supplied random number generators.
 * This is the Mitchell-Moore algorithm from Knuth Volume II.
 * Best to leave the constants alone unless you've read Knuth.
 * -- Furey
 */

/* I noticed streaking with this random number generator, so I switched
   back to the system srandom call.  If this doesn't work for you, 
   define OLD_RAND to use the old system -- Alander */

#if defined (OLD_RAND)
static int rgiState[2 + 55];
#endif

void init_mm()
{
#if defined (OLD_RAND)
	int *piState;
	int iState;

	piState = &rgiState[2];

	piState[-2] = 55 - 55;
	piState[-1] = 55 - 24;

	piState[0] = ((int) current_time) & ((1 << 30) - 1);
	piState[1] = 1;
	for (iState = 2; iState < 55; iState++)
	{
		piState[iState] =
			(piState[iState - 1] + piState[iState - 2]) & ((1 << 30) - 1);
	}
#else
	srandom(time(NULL) ^ getpid());
#endif
	return;
}

long number_mm(void)
{
#if defined (OLD_RAND)
	int *piState;
	int iState1;
	int iState2;
	int iRand;

	piState = &rgiState[2];
	iState1 = piState[-2];
	iState2 = piState[-1];
	iRand = (piState[iState1] + piState[iState2]) & ((1 << 30) - 1);
	piState[iState1] = iRand;
	if (++iState1 == 55)
		iState1 = 0;
	if (++iState2 == 55)
		iState2 = 0;
	piState[-2] = iState1;
	piState[-1] = iState2;
	return iRand >> 6;
#else
	return random() >> 6;
#endif
}

/*
 * Roll some dice.
 */
int dice(int number, int size)
{
	int idice;
	int sum;

	switch (size)
	{
	case 0:
		return 0;
	case 1:
		return number;
	}

	for (idice = 0, sum = 0; idice < number; idice++)
		sum += number_range(1, size);

	return sum;
}

/*
 * Simple linear interpolation.
 */
int interpolate(int level, int value_00, int value_32)
{
	return value_00 + level * (value_32 - value_00) / 32;
}

/*
 * Removes the tildes from a string.
 * Used for player-entered strings that go into disk files.
 */
void smash_tilde(const char *str)
{
	for (; *str != '\0'; str++)
	{
		if (*str == '~')
			*(char *) str = '-';
	}

	return;
}

/* Returns -1 if astr is shorter than bstr,
   0 if they are the same,
   1 if astr is longer than bstr.
   CaSe InSeNsiTiVe.
*/
int str_cmp(const char *astr, const char *bstr)
{
	if (astr == NULL)
		return bstr == NULL ? 0 : -1;
	if (bstr == NULL)
		return 1;
	return strcasecmp(astr, bstr);
}

/* Returns -1 if astr is shorter than bstr,
   0 if they are the same,
   1 if astr is longer than bstr.
   Case Sensitive.
*/
int str_casecmp(const char *astr, const char *bstr)
{
	if (astr == NULL)
		return bstr == NULL ? 0 : -1;
	if (bstr == NULL)
		return 1;
	return strcmp(astr, bstr);
}

/* Returns -1 if astr is shorter than bstr,
   0 if they are the same,
   1 if astr is longer than bstr.
   Only compares length of len.
   CaSe InSeNsiTiVe.
*/
int str_ncmp(const char *astr, const char *bstr, size_t len)
{
	if (astr == NULL)
		return bstr == NULL ? 0 : -1;
	if (bstr == NULL)
		return 1;
	return strncasecmp(astr, bstr, len);
}

/* Returns -1 if astr is shorter than bstr,
   0 if they are the same,
   1 if astr is longer than bstr.
   Only compares length of len.
   Case Sensitive.
*/
int str_ncasecmp(const char *astr, const char *bstr, size_t len)
{
	if (astr == NULL)
		return bstr == NULL ? 0 : -1;
	if (bstr == NULL)
		return 1;
	return strncmp(astr, bstr, len);
}

/*
 * Compare strings, case insensitive, for prefix matching.
 * Return TRUE if astr not a prefix of bstr
 *   (compatibility with historical functions).
 */
bool str_prefix(const char *astr, const char *bstr)
{
	if (astr == NULL)
	{
		bug("Str_prefix: null astr.");
		return TRUE;
	}

	if (bstr == NULL)
	{
		bug("Str_prefix: null bstr.");
		return TRUE;
	}

	for (; *astr; astr++, bstr++)
	{
		if (LOWER(*astr) != LOWER(*bstr))
			return TRUE;
	}

	return FALSE;
}

/*
 * Compare strings, case insensitive, for match anywhere.
 * Returns TRUE is astr not part of bstr.
 *   (compatibility with historical functions).
 */
bool str_infix(const char *astr, const char *bstr)
{
	int sstr1;
	int sstr2;
	int ichar;
	char c0;

	if ((c0 = LOWER(astr[0])) == '\0')
		return FALSE;

	sstr1 = strlen(astr);
	sstr2 = strlen(bstr);

	for (ichar = 0; ichar <= sstr2 - sstr1; ichar++)
	{
		if (c0 == LOWER(bstr[ichar]) && !str_prefix(astr, bstr + ichar))
			return FALSE;
	}

	return TRUE;
}

/*
 * Compare strings, case insensitive, for suffix matching.
 * Return TRUE if astr not a suffix of bstr
 *   (compatibility with historical functions).
 */
bool str_suffix(const char *astr, const char *bstr)
{
	int sstr1;
	int sstr2;

	sstr1 = strlen(astr);
	sstr2 = strlen(bstr);
	if (sstr1 <= sstr2 && !str_cmp(astr, bstr + sstr2 - sstr1))
		return FALSE;
	else
		return TRUE;
}

/*
 * Returns an initial-capped string.
 */
char *capitalize(const char *str)
{
	static char strcap[MAX_STRING_LENGTH];
	int i;

	for (i = 0; str[i] != '\0'; i++)
		strcap[i] = LOWER(str[i]);
	strcap[i] = '\0';
	strcap[0] = UPPER(strcap[0]);
	return strcap;
}

/*
 * Append a string to a file.
 */
void append_file(CHAR_DATA * ch, char *file, const char *str)
{
	WRITE_DATA *fp;

	if (IS_NPC(ch) || IS_NULLSTR(str))
		return;

	if ((fp = open_append(file)) == NULL)
	{
		log_error(file);
		chprintln(ch, "Could not open the file!");
	}
	else
	{
		fprintf(fp->stream, "[%5ld] %s: %s\n",
				ch->in_room ? ch->in_room->vnum : 0, ch->name, str);
		close_append(fp);
	}

	return;
}

/*
 * Reports a bug.
 */
void bug(const char *str)
{
	char buf[MAX_STRING_LENGTH];

	if (fpArea != NULL)
	{
		size_t iLine;
		int iCount = 0;

		for (iLine = 0; iLine < UMIN(fpArea->pos, fpArea->size); iLine++)
		{
			if (fpArea->str[iLine] == '\n')
				iCount++;
		}

		sprintf(buf, "[*****] FILE: %s LINE: %d", strArea, iCount);
		log_string(buf);
	}

	sprintf(buf, "[*****] BUG: %s", str);
	log_string(buf);
	wiznet(buf, NULL, NULL, WIZ_BUGS, 0, 0);
	return;
}

/*
 * Writes a string to the log.
 */
void log_string(const char *str)
{
	fprintf(stderr, "%s :: %s\n", str_time(-1, -1, NULL), str);
	return;
}

void log_error(const char *str)
{
	char buf[MSL];

	sprintf(buf, "%s :: %s", str_time(-1, -1, NULL), str);
	perror(buf);
	return;
}

/*
 * This function is here to aid in debugging.
 * If the last expression in a function is another function call,
 *   gcc likes to generate a JMP instead of a CALL.
 * This is called "tail chaining."
 * It hoses the debugger call stack for that call.
 * So I make this the last call in certain critical functions,
 *   where I really need the call stack to be right for debugging!
 *
 * If you don't understand this, then LEAVE IT ALONE.
 * Don't remove any calls to tail_chain anywhere.
 *
 * -- Furey
 */
void tail_chain(void)
{
	return;
}
