/**************************************************************************
*  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
*  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
*                                                                         *
*  Merc Diku Mud improvements copyright (C) 1992, 1993 by Michael         *
*  Chastain, Michael Quan, and Mitchell Tse.                              *
*                                                                         *
*  In order to use any part of this Merc Diku Mud, you must comply with   *
*  both the original Diku license in 'license.doc' as well the Merc       *
*  license in 'license.txt'.  In particular, you may not remove either of *
*  these copyright notices.                                               *
*                                                                         *
*  Much time and thought has gone into this software and you are          *
*  benefiting.  We hope that you share your changes too.  What goes       *
*  around, comes around.                                                  *
***************************************************************************
*       ROM 2.4 is copyright 1993-1998 Russ Taylor                        *
*       ROM has been brought to you by the ROM consortium                 *
*           Russ Taylor (rtaylor@hypercube.org)                           *
*           Gabrielle Taylor (gtaylor@hypercube.org)                      *
*           Brian Moore (zump@rom.org)                                    *
*       By using this code, you have agreed to follow the terms of the    *
*       ROM license, in the file Rom24/doc/rom.license                    *
***************************************************************************
*       1stMUD ROM Derivative (c) 2001-2003 by Ryan Jennings              *
*            http://1stmud.dlmud.com/  <r-jenn@shaw.ca>                   *
***************************************************************************/

#include "merc.h"
#include "interp.h"

/*
 * Local functions.
 */
PROTOTYPE(void check_assist, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(bool check_dodge, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(bool check_parry, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(bool check_shield_block, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(void dam_message, (CHAR_DATA *, CHAR_DATA *, int, int, bool));
PROTOTYPE(void death_cry, (CHAR_DATA *));
PROTOTYPE(void group_gain, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(int xp_compute, (CHAR_DATA *, CHAR_DATA *, int));
PROTOTYPE(void make_corpse, (CHAR_DATA *));
PROTOTYPE(void one_hit, (CHAR_DATA *, CHAR_DATA *, int, bool));
PROTOTYPE(void mob_hit, (CHAR_DATA *, CHAR_DATA *, int));
PROTOTYPE(void raw_kill, (CHAR_DATA *));
PROTOTYPE(void set_fighting, (CHAR_DATA *, CHAR_DATA *));
PROTOTYPE(void disarm, (CHAR_DATA *, CHAR_DATA *));

/*
 * Control the fights going on.
 * Called periodically by update_handler.
 */
void violence_update(void)
{
	CHAR_DATA *ch;
	CHAR_DATA *ch_next;
	CHAR_DATA *victim;
	OBJ_DATA *obj, *obj_next;
	bool room_trig = FALSE;

	for (ch = char_first; ch != NULL; ch = ch_next)
	{
		ch_next = ch->next;

		if (IS_NPC(ch) && ch->fighting == NULL
			&& IS_AWAKE(ch) && ch->hunting != NULL)
		{
			hunt_victim(ch);
			continue;
		}

		if ((victim = ch->fighting) == NULL || ch->in_room == NULL)
			continue;

		if (IS_AWAKE(ch) && ch->in_room == victim->in_room)
			multi_hit(ch, victim, TYPE_UNDEFINED);
		else
			stop_fighting(ch, FALSE);

		if ((victim = ch->fighting) == NULL)
			continue;

		/*
		 * Fun for the whole family!
		 */
		check_assist(ch, victim);

		if (IS_NPC(ch))
		{
			if (HAS_TRIGGER_MOB(ch, TRIG_FIGHT))
				p_percent_trigger(ch, NULL, NULL, victim, NULL, NULL,
								  TRIG_FIGHT);
			if (HAS_TRIGGER_MOB(ch, TRIG_HPCNT))
				p_hprct_trigger(ch, victim);
		}
		for (obj = ch->first_carrying; obj; obj = obj_next)
		{
			obj_next = obj->next_content;

			if (obj->wear_loc != WEAR_NONE && HAS_TRIGGER_OBJ(obj, TRIG_FIGHT))
				p_percent_trigger(NULL, obj, NULL, victim, NULL, NULL,
								  TRIG_FIGHT);
		}

		if (HAS_TRIGGER_ROOM(ch->in_room, TRIG_FIGHT) && room_trig == FALSE)
		{
			room_trig = TRUE;
			p_percent_trigger(NULL, NULL, ch->in_room, victim, NULL, NULL,
							  TRIG_FIGHT);
		}
	}

	return;
}

/* for auto assisting */
void check_assist(CHAR_DATA * ch, CHAR_DATA * victim)
{
	CHAR_DATA *rch, *rch_next;

	for (rch = ch->in_room->first_person; rch != NULL; rch = rch_next)
	{
		rch_next = rch->next_in_room;

		if (IS_AWAKE(rch) && rch->fighting == NULL)
		{

			/* quick check for ASSIST_PLAYER */
			if (!IS_NPC(ch) && IS_NPC(rch) &&
				IS_SET(rch->off_flags, ASSIST_PLAYERS) &&
				rch->level + 6 > victim->level)
			{
				do_function(rch, &do_emote, "screams and attacks!");
				multi_hit(rch, victim, TYPE_UNDEFINED);
				continue;
			}

			/* PCs next */
			if (!IS_NPC(ch) || IS_AFFECTED(ch, AFF_CHARM))
			{
				if (((!IS_NPC(rch) &&
					  IS_SET(rch->act, PLR_AUTOASSIST)) ||
					 IS_AFFECTED(rch, AFF_CHARM)) &&
					is_same_group(ch, rch) && !is_safe(rch, victim))
					multi_hit(rch, victim, TYPE_UNDEFINED);

				continue;
			}

			/* now check the NPC cases */

			if (IS_NPC(ch) && !IS_AFFECTED(ch, AFF_CHARM))

			{
				if ((IS_NPC(rch) &&
					 IS_SET(rch->off_flags, ASSIST_ALL)) ||
					(IS_NPC(rch) && rch->group &&
					 rch->group == ch->group) || (IS_NPC(rch) &&
												  rch->race ==
												  ch->race
												  &&
												  IS_SET
												  (rch->off_flags,
												   ASSIST_RACE))
					|| (IS_NPC(rch)
						&& IS_SET(rch->off_flags, ASSIST_ALIGN)
						&& ((IS_GOOD(rch) && IS_GOOD(ch))
							|| (IS_EVIL(rch) && IS_EVIL(ch))
							|| (IS_NEUTRAL(rch)
								&& IS_NEUTRAL(ch))))
					|| (rch->pIndexData == ch->pIndexData
						&& IS_SET(rch->off_flags, ASSIST_VNUM)))

				{
					CHAR_DATA *vch;
					CHAR_DATA *target;
					int number;

					if (number_bits(1) == 0)
						continue;

					target = NULL;
					number = 0;
					for (vch = ch->in_room->first_person; vch; vch = vch->next)
					{
						if (can_see(rch, vch) &&
							is_same_group(vch, victim) &&
							number_range(0, number) == 0)
						{
							target = vch;
							number++;
						}
					}

					if (target != NULL)
					{
						do_function(rch, &do_emote, "screams and attacks!");
						multi_hit(rch, target, TYPE_UNDEFINED);
					}
				}
			}
		}
	}
}

void special_move(CHAR_DATA * ch, CHAR_DATA * victim)
{
	if (!victim || victim->position == POS_DEAD)
		return;

	switch (number_range(1, 7))
	{
	default:
		return;
	case 1:
		act
			("{RYou pull your hands into your waist then snap them into $N's{R stomach.{x",
			 ch, NULL, victim, TO_CHAR);
		act
			("{R$n{R pulls $s{R hands into $s{R waist then snaps them into your stomach.{x",
			 ch, NULL, victim, TO_VICT);
		act
			("{R$n{R pulls $s{R hands into $s{R waist then snaps them into $N's{R stomach.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act
			("{RYou double over in agony, and fall to the ground gasping for breath.{x",
			 victim, NULL, NULL, TO_CHAR);
		act
			("{R$n{R doubles over in agony, and falls to the ground gasping for breath.{x",
			 victim, NULL, NULL, TO_ROOM);
		break;
	case 2:
		act("{RYou spin in a low circle, catching $N{R behind $S{R ankle.{x",
			ch, NULL, victim, TO_CHAR);
		act("{R$n{R spins in a low circle, catching you behind your ankle.{x",
			ch, NULL, victim, TO_VICT);
		act("{R$n{R spins in a low circle, catching $N{R behind $S{R ankle.{x",
			ch, NULL, victim, TO_NOTVICT);
		act("{RYou crash to the ground, stunned.{x", victim, NULL, NULL,
			TO_CHAR);
		act("{R$n{R crashes to the ground, stunned.{x", victim, NULL, NULL,
			TO_ROOM);
		break;
	case 3:
		act("{RYou roll between $N's{R legs and flip to your feet.{x", ch,
			NULL, victim, TO_CHAR);
		act("{R$n{R rolls between your legs and flips to $s{R feet.{x", ch,
			NULL, victim, TO_VICT);
		act("{R$n{R rolls between $N's{R legs and flips to $s{R feet.{x", ch,
			NULL, victim, TO_NOTVICT);
		act
			("{RYou spin around and smash your elbow into the back of $N's{R head.{x",
			 ch, NULL, victim, TO_CHAR);
		act
			("{R$n{R spins around and smashes $s{R elbow into the back of your head.{x",
			 ch, NULL, victim, TO_VICT);
		act
			("{R$n{R spins around and smashes $s{R elbow into the back of $N's{R head.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act("{RYou fall to the ground, stunned.{x", victim, NULL, NULL,
			TO_CHAR);
		act("{R$n{R falls to the ground, stunned.{x", victim, NULL, NULL,
			TO_ROOM);
		break;
	case 4:
		act
			("{RYou somersault over $N's{R head and land lightly on your toes.{x",
			ch, NULL, victim, TO_CHAR);
		act("{R$n{R somersaults over your head and lands lightly on $s toes.{x",
			ch, NULL, victim, TO_VICT);
		act
			("{R$n{R somersaults over $N's{R head and lands lightly on $s toes.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act
			("{RYou roll back onto your shoulders and kick both feet into $N's{R back.{x",
			 ch, NULL, victim, TO_CHAR);
		act
			("{R$n{R rolls back onto $s{R shoulders and kicks both feet into your back.{x",
			 ch, NULL, victim, TO_VICT);
		act
			("{R$n{R rolls back onto $s{R shoulders and kicks both feet into $N's{R back.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act("{RYou fall to the ground, stunned.", victim, NULL, NULL, TO_CHAR);
		act("{R$n{R falls to the ground, stunned.", victim, NULL, NULL,
			TO_ROOM);
		act("{RYou flip back up to your feet.", ch, NULL, NULL, TO_CHAR);
		act("{R$n{R flips back up to $s feet.", ch, NULL, NULL, TO_ROOM);
		break;
	case 5:
		act("{RYou grab $N{R by the waist and hoist $M{R above your head.{x",
			ch, NULL, victim, TO_CHAR);
		act("{R$n{R grabs $N{R by the waist and hoists $M{R above $s{R head.{x",
			ch, NULL, victim, TO_NOTVICT);
		act("{R$n{R grabs you by the waist and hoists you above $s{R head.{x",
			ch, NULL, victim, TO_VICT);
		act("{RYou crash to the ground, stunned.{x", victim, NULL, NULL,
			TO_CHAR);
		act("{R$n{R crashes to the ground, stunned.{x", victim, NULL, NULL,
			TO_ROOM);
		break;
	case 6:
		act("{RYou grab $N{R by the head and slam $S{R face into your knee.{x",
			ch, NULL, victim, TO_CHAR);
		act
			("{R$n{R grabs you by the head and slams your face into $s{R knee.{x",
			ch, NULL, victim, TO_VICT);
		act
			("{R$n{R grabs $N{R by the head and slams $S{R face into $s{R knee.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act("{RYou crash to the ground, stunned.{x", victim, NULL, NULL,
			TO_CHAR);
		act("{R$n{R crashes to the ground, stunned.{x", victim, NULL, NULL,
			TO_ROOM);
		act("{RYou flip back up to your feet.{x", ch, NULL, NULL, TO_CHAR);
		act("{R$n{R flips back up to $s{R feet.{x", ch, NULL, NULL, TO_ROOM);
		break;
	case 7:
		act
			("{RYou duck under $N's{R attack and pound your fist into $S{R stomach.{x",
			 ch, NULL, victim, TO_CHAR);
		act
			("{R$n{R ducks under your attack and pounds $s{R fist into your stomach.{x",
			 ch, NULL, victim, TO_VICT);
		act
			("{R$n{R ducks under $N's{R attack and pounds $s{R fist into $N's{R stomach.{x",
			 ch, NULL, victim, TO_NOTVICT);
		act("{RYou double over in agony.{x", victim, NULL, NULL, TO_CHAR);
		act("{R$n{R doubles over in agony.{x", victim, NULL, NULL, TO_ROOM);
		break;
	}

	stop_fighting(victim, TRUE);
	victim->position = POS_STUNNED;

	return;
}

/*
 * Do one group of attacks.
 */
void multi_hit(CHAR_DATA * ch, CHAR_DATA * victim, int dt)
{
	int chance;

	/* decrement the wait */
	if (ch->desc == NULL)
		ch->wait = UMAX(0, ch->wait - PULSE_VIOLENCE);

	if (ch->desc == NULL)
		ch->daze = UMAX(0, ch->daze - PULSE_VIOLENCE);

	/* no attacks for stunnies -- just a check */
	if (ch->position < POS_RESTING)
		return;

	if (IS_NPC(ch))
	{
		mob_hit(ch, victim, dt);
		return;
	}

	one_hit(ch, victim, dt, FALSE);

	if (get_eq_char(ch, WEAR_SECONDARY))
	{
		one_hit(ch, victim, dt, TRUE);
		if (ch->fighting != victim)
			return;
	}

	if (ch->fighting != victim)
		return;

	if (IS_AFFECTED(ch, AFF_HASTE))
		one_hit(ch, victim, dt, FALSE);

	if (ch->fighting != victim || dt == gsn_backstab)
		return;

	if (IS_VALID_STANCE(GET_STANCE(ch, STANCE_CURRENT))
		&& GET_STANCE(ch, STANCE_CURRENT) >= 200 && number_percent() == 50)
	{
		special_move(ch, victim);
		return;
	}

	chance = get_skill(ch, gsn_second_attack) / 2;

	if (IS_AFFECTED(ch, AFF_SLOW))
		chance /= 2;

	if (number_percent() < chance)
	{
		one_hit(ch, victim, dt, FALSE);
		check_improve(ch, gsn_second_attack, TRUE, 5);
		if (ch->fighting != victim)
			return;
	}

	chance = get_skill(ch, gsn_third_attack) / 4;

	if (IS_AFFECTED(ch, AFF_SLOW))
		chance = 0;

	if (number_percent() < chance)
	{
		one_hit(ch, victim, dt, FALSE);
		check_improve(ch, gsn_third_attack, TRUE, 6);
		if (ch->fighting != victim)
			return;
	}

	if (IS_STANCE(ch, STANCE_VIPER)
		&& number_percent() < GET_STANCE(ch, STANCE_VIPER) * 0.5)
	{
		one_hit(ch, victim, dt, FALSE);
		if (ch->fighting != victim)
			return;
	}
	else if (IS_STANCE(ch, STANCE_MANTIS)
			 && number_percent() < GET_STANCE(ch, STANCE_MANTIS) * 0.5)
	{
		one_hit(ch, victim, dt, FALSE);
		if (ch->fighting != victim)
			return;
	}
	else if (IS_STANCE(ch, STANCE_TIGER)
			 && number_percent() < GET_STANCE(ch, STANCE_TIGER) * 0.5)
	{
		one_hit(ch, victim, dt, FALSE);
		if (ch->fighting != victim)
			return;
	}

	return;
}

/* procedure for all mobile attacks */
void mob_hit(CHAR_DATA * ch, CHAR_DATA * victim, int dt)
{
	int chance, number;
	CHAR_DATA *vch, *vch_next;

	one_hit(ch, victim, dt, FALSE);

	if (ch->fighting != victim)
		return;

	/* Area attack -- BALLS nasty! */

	if (IS_SET(ch->off_flags, OFF_AREA_ATTACK))
	{
		for (vch = ch->in_room->first_person; vch != NULL; vch = vch_next)
		{
			vch_next = vch->next;
			if ((vch != victim && vch->fighting == ch))
				one_hit(ch, vch, dt, FALSE);
		}
	}

	if (IS_AFFECTED(ch, AFF_HASTE) ||
		(IS_SET(ch->off_flags, OFF_FAST) && !IS_AFFECTED(ch, AFF_SLOW)))
		one_hit(ch, victim, dt, FALSE);

	if (ch->fighting != victim || dt == gsn_backstab)
		return;

	chance = get_skill(ch, gsn_second_attack) / 2;

	if (IS_AFFECTED(ch, AFF_SLOW) && !IS_SET(ch->off_flags, OFF_FAST))
		chance /= 2;

	if (number_percent() < chance)
	{
		one_hit(ch, victim, dt, FALSE);
		if (ch->fighting != victim)
			return;
	}

	chance = get_skill(ch, gsn_third_attack) / 4;

	if (IS_AFFECTED(ch, AFF_SLOW) && !IS_SET(ch->off_flags, OFF_FAST))
		chance = 0;

	if (number_percent() < chance)
	{
		one_hit(ch, victim, dt, FALSE);
		if (ch->fighting != victim)
			return;
	}

	/* oh boy!  Fun stuff! */

	if (ch->wait > 0)
		return;

	number = number_range(0, 2);

	if (number == 1 && IS_SET(ch->act, ACT_MAGE))
	{
		/*  { mob_cast_mage(ch,victim); return; } */ ;
	}

	if (number == 2 && IS_SET(ch->act, ACT_CLERIC))
	{
		/* { mob_cast_cleric(ch,victim); return; } */ ;
	}

	/* now for the skills */

	number = number_range(0, 8);

	switch (number)
	{
	case (0):
		if (IS_SET(ch->off_flags, OFF_BASH))
			do_function(ch, &do_bash, "");
		break;

	case (1):
		if (IS_SET(ch->off_flags, OFF_BERSERK) && !IS_AFFECTED(ch, AFF_BERSERK))
			do_function(ch, &do_berserk, "");
		break;

	case (2):
		if (IS_SET(ch->off_flags, OFF_DISARM) ||
			(get_weapon_sn(ch) != gsn_hand_to_hand &&
			 (IS_SET(ch->act, ACT_WARRIOR) || IS_SET(ch->act, ACT_THIEF))))
			do_function(ch, &do_disarm, "");
		break;

	case (3):
		if (IS_SET(ch->off_flags, OFF_KICK))
			do_function(ch, &do_kick, "");
		break;

	case (4):
		if (IS_SET(ch->off_flags, OFF_KICK_DIRT))
			do_function(ch, &do_dirt, "");
		break;

	case (5):
		if (IS_SET(ch->off_flags, OFF_TAIL))
		{
			/* do_function(ch, &do_tail, "") */ ;
		}
		break;

	case (6):
		if (IS_SET(ch->off_flags, OFF_TRIP))
			do_function(ch, &do_trip, "");
		break;

	case (7):
		if (IS_SET(ch->off_flags, OFF_CRUSH))
		{
			/* do_function(ch, &do_crush, "") */ ;
		}
		break;
	case (8):
		if (IS_SET(ch->off_flags, OFF_BACKSTAB))
		{
			do_function(ch, &do_backstab, "");
		}
	}
}

/*
 * Hit one guy once.
 */
void one_hit(CHAR_DATA * ch, CHAR_DATA * victim, int dt, bool secondary)
{
	OBJ_DATA *wield;
	int victim_ac;
	int thac0;
	int thac0_00;
	int thac0_32;
	int dam;
	int diceroll;
	int sn, skill;
	int dam_type;
	bool result;

	sn = -1;

	/* just in case */
	if (victim == ch || ch == NULL || victim == NULL)
		return;

	/*
	 * Can't beat a dead char!
	 * Guard against weird room-leavings.
	 */
	if (victim->position == POS_DEAD || ch->in_room != victim->in_room)
		return;

	/*
	 * Figure out the type of damage message.
	 */
	if (!secondary)
		wield = get_eq_char(ch, WEAR_WIELD);
	else
		wield = get_eq_char(ch, WEAR_SECONDARY);

	if (dt == TYPE_UNDEFINED)
	{
		dt = TYPE_HIT;
		if (wield != NULL && wield->item_type == ITEM_WEAPON)
			dt += wield->value[3];
		else
			dt += ch->dam_type;
	}

	if (dt < TYPE_HIT)
		if (wield != NULL)
			dam_type = attack_table[wield->value[3]].damage;
		else
			dam_type = attack_table[ch->dam_type].damage;
	else
		dam_type = attack_table[dt - TYPE_HIT].damage;

	if (dam_type == -1)
		dam_type = DAM_BASH;

	/* get the weapon skill */
	sn = get_weapon_sn(ch);
	skill = 20 + get_weapon_skill(ch, sn);

	/*
	 * Calculate to-hit-armor-class-0 versus armor.
	 */
	if (IS_NPC(ch))
	{
		thac0_00 = 20;
		thac0_32 = -4;			/* as good as a thief */
		if (IS_SET(ch->act, ACT_WARRIOR))
			thac0_32 = -10;
		else if (IS_SET(ch->act, ACT_THIEF))
			thac0_32 = -4;
		else if (IS_SET(ch->act, ACT_CLERIC))
			thac0_32 = 2;
		else if (IS_SET(ch->act, ACT_MAGE))
			thac0_32 = 6;
	}
	else
	{
		thac0_00 = get_thac00(ch);
		thac0_32 = get_thac32(ch);
	}
	thac0 = interpolate(ch->level, thac0_00, thac0_32);

	if (thac0 < 0)
		thac0 = thac0 / 2;

	if (thac0 < -5)
		thac0 = -5 + (thac0 + 5) / 2;

	thac0 -= GET_HITROLL(ch) * skill / 100;
	thac0 += 5 * (100 - skill) / 100;

	if (dt == gsn_backstab)
		thac0 -= 10 * (100 - get_skill(ch, gsn_backstab));

	switch (dam_type)
	{
	case (DAM_PIERCE):
		victim_ac = GET_AC(victim, AC_PIERCE) / 10;
		break;
	case (DAM_BASH):
		victim_ac = GET_AC(victim, AC_BASH) / 10;
		break;
	case (DAM_SLASH):
		victim_ac = GET_AC(victim, AC_SLASH) / 10;
		break;
	default:
		victim_ac = GET_AC(victim, AC_EXOTIC) / 10;
		break;
	};

	if (victim_ac < -15)
		victim_ac = (victim_ac + 15) / 5 - 15;

	if (!can_see(ch, victim))
		victim_ac -= 4;

	if (victim->position < POS_FIGHTING)
		victim_ac += 4;

	if (victim->position < POS_RESTING)
		victim_ac += 6;

	/*
	 * The moment of excitement!
	 */
	while ((diceroll = number_bits(5)) >= 20)
		;

	if (diceroll == 0 || (diceroll != 19 && diceroll < thac0 - victim_ac))
	{
		/* Miss. */
		damage(ch, victim, 0, dt, dam_type, TRUE);
		improve_stance(ch);
		tail_chain();
		return;
	}

	/*
	 * Hit.
	 * Calc damage.
	 */
	if (IS_NPC(ch) && (!ch->pIndexData->new_format || wield == NULL))
		if (!ch->pIndexData->new_format)
		{
			dam = number_range(ch->level / 2, ch->level * 3 / 2);
			if (wield != NULL)
				dam += dam / 2;
		}
		else
			dam = dice(ch->damage[DICE_NUMBER], ch->damage[DICE_TYPE]);

	else
	{
		if (sn != -1)
			check_improve(ch, sn, TRUE, 5);
		if (wield != NULL)
		{
			if (wield->pIndexData->new_format)
				dam = dice(wield->value[1], wield->value[2]) * skill / 100;
			else
				dam =
					number_range(wield->value[1] * skill / 100,
								 wield->value[2] * skill / 100);

			if (get_eq_char(ch, WEAR_SHIELD) == NULL)	/* no shield = more */
				dam = dam * 11 / 10;

			/* sharpness! */
			if (IS_WEAPON_STAT(wield, WEAPON_SHARP))
			{
				int percent;

				if ((percent = number_percent()) <= (skill / 8))
					dam = 2 * dam + (dam * 2 * percent / 100);
			}
		}
		else
			dam =
				number_range(1 + 4 * skill / 100,
							 2 * ch->level / 3 * skill / 100);
	}

	/*
	 * Bonuses.
	 */

	if (IS_STANCE(ch, STANCE_NORMAL))
	{
		if (IS_NPC(ch))
			dam = dam * 113 / 100;
		else
			dam = dam * 115 / 100;
	}
	else
		dam = dambonus(ch, victim, dam, GET_STANCE(ch, STANCE_CURRENT));

	if (get_skill(ch, gsn_enhanced_damage) > 0)
	{
		diceroll = number_percent();
		if (diceroll <= get_skill(ch, gsn_enhanced_damage))
		{
			check_improve(ch, gsn_enhanced_damage, TRUE, 6);
			dam += 2 * (dam * diceroll / 300);
		}
	}

	if (!IS_AWAKE(victim))
		dam *= 2;
	else if (victim->position < POS_FIGHTING)
		dam = dam * 3 / 2;

	if (dt == gsn_backstab && wield != NULL)
	{
		if (wield->value[0] != 2)
			dam *= 2 + (ch->level / 10);
		else
			dam *= 2 + (ch->level / 8);
	}

	dam += GET_DAMROLL(ch) * UMIN(100, skill) / 100;

	if (dam <= 0)
		dam = 1;

	result = damage(ch, victim, dam, dt, dam_type, TRUE);

	/* but do we have a funky weapon? */
	if (result && wield != NULL)
	{
		int pdam;

		if (ch->fighting == victim && IS_WEAPON_STAT(wield, WEAPON_POISON))
		{
			int level;
			AFFECT_DATA *poison, af;

			if ((poison = affect_find(wield->first_affect, gsn_poison)) == NULL)
				level = wield->level;
			else
				level = poison->level;

			if (!saves_spell(level / 2, victim, DAM_POISON))
			{
				chprintln(victim,
						  "You feel poison coursing through your veins.");
				act("$n is poisoned by the venom on $p.", victim, wield,
					NULL, TO_ROOM);

				af.where = TO_AFFECTS;
				af.type = gsn_poison;
				af.level = level * 3 / 4;
				af.duration = level / 2;
				af.location = APPLY_STR;
				af.modifier = -1;
				af.bitvector = AFF_POISON;
				affect_join(victim, &af);
			}

			/* weaken the poison if it's temporary */
			if (poison != NULL)
			{
				poison->level = UMAX(0, poison->level - 2);
				poison->duration = UMAX(0, poison->duration - 1);

				if (poison->level == 0 || poison->duration == 0)
					act("The poison on $p has worn off.", ch,
						wield, NULL, TO_CHAR);
			}
		}

		if (ch->fighting == victim && IS_WEAPON_STAT(wield, WEAPON_VAMPIRIC))
		{
			pdam = number_range(1, wield->level / 5 + 1);
			act("$p draws life from $n.", victim, wield, NULL, TO_ROOM);
			act("You feel $p drawing your life away.", victim, wield,
				NULL, TO_CHAR);
			damage(ch, victim, pdam, 0, DAM_NEGATIVE, FALSE);
			ch->alignment = UMAX(-1000, ch->alignment - 1);
			ch->hit += pdam / 2;
		}

		if (ch->fighting == victim && IS_WEAPON_STAT(wield, WEAPON_FLAMING))
		{
			pdam = number_range(1, wield->level / 4 + 1);
			act("$n is burned by $p.", victim, wield, NULL, TO_ROOM);
			act("$p sears your flesh.", victim, wield, NULL, TO_CHAR);
			fire_effect((void *) victim, wield->level / 2, pdam, TARGET_CHAR);
			damage(ch, victim, pdam, 0, DAM_FIRE, FALSE);
		}

		if (ch->fighting == victim && IS_WEAPON_STAT(wield, WEAPON_FROST))
		{
			pdam = number_range(1, wield->level / 6 + 2);
			act("$p freezes $n.", victim, wield, NULL, TO_ROOM);
			act("The cold touch of $p surrounds you with ice.",
				victim, wield, NULL, TO_CHAR);
			cold_effect(victim, wield->level / 2, pdam, TARGET_CHAR);
			damage(ch, victim, pdam, 0, DAM_COLD, FALSE);
		}

		if (ch->fighting == victim && IS_WEAPON_STAT(wield, WEAPON_SHOCKING))
		{
			pdam = number_range(1, wield->level / 5 + 2);
			act("$n is struck by lightning from $p.", victim, wield,
				NULL, TO_ROOM);
			act("You are shocked by $p.", victim, wield, NULL, TO_CHAR);
			shock_effect(victim, wield->level / 2, pdam, TARGET_CHAR);
			damage(ch, victim, pdam, 0, DAM_LIGHTNING, FALSE);
		}
	}
	tail_chain();
	return;
}

/*
 * Inflict damage from a hit.
 */
bool damage(CHAR_DATA * ch, CHAR_DATA * victim, int dam, int dt,
			int dam_type, bool show)
{
	OBJ_DATA *corpse;
	bool immune;

	if (victim->position == POS_DEAD)
		return FALSE;

	/*
	 * Stop up any residual loopholes.
	 */
	if (dam > 1200 && dt >= TYPE_HIT)
	{
		bugf("Damage: %d: more than 1200 points!", dam);
		dam = 1200;
		if (!IS_IMMORTAL(ch))
		{
			OBJ_DATA *obj;
			obj = get_eq_char(ch, WEAR_WIELD);
			chprintln(ch, "You really shouldn't cheat.");
			if (obj != NULL)
				extract_obj(obj);
		}

	}

	/* damage reduction */
	if (dam > 35)
		dam = (dam - 35) / 2 + 35;
	if (dam > 80)
		dam = (dam - 80) / 2 + 80;

	if (victim != ch)
	{
		/*
		 * Certain attacks are forbidden.
		 * Most other attacks are returned.
		 */
		if (is_safe(ch, victim))
			return FALSE;
		check_killer(ch, victim);

		if (victim->position > POS_STUNNED)
		{
			if (victim->fighting == NULL)
			{
				set_fighting(victim, ch);
				if (IS_NPC(victim) && HAS_TRIGGER_MOB(victim, TRIG_KILL))
					p_percent_trigger(victim, NULL, NULL, ch, NULL, NULL,
									  TRIG_KILL);
			}
			if (victim->timer <= 4)
				victim->position = POS_FIGHTING;
		}

		if (victim->position > POS_STUNNED)
		{
			if (ch->fighting == NULL)
				set_fighting(ch, victim);
		}

		/*
		 * More charm stuff.
		 */
		if (victim->master == ch)
			stop_follower(victim);
	}

	/*
	 * Inviso attacks ... not.
	 */
	if (IS_AFFECTED(ch, AFF_INVISIBLE))
	{
		affect_strip(ch, gsn_invis);
		affect_strip(ch, gsn_mass_invis);
		REMOVE_BIT(ch->affected_by, AFF_INVISIBLE);
		act("$n fades into existence.", ch, NULL, NULL, TO_ROOM);
	}

	/*
	 * Damage modifiers.
	 */

	if (dam > 1 && !IS_NPC(victim) &&
		victim->pcdata->condition[COND_DRUNK] > 10)
		dam = 9 * dam / 10;

	if (dam > 1 && IS_AFFECTED(victim, AFF_SANCTUARY))
		dam /= 2;

	if (dam > 1 &&
		((IS_AFFECTED(victim, AFF_PROTECT_EVIL) && IS_EVIL(ch)) ||
		 (IS_AFFECTED(victim, AFF_PROTECT_GOOD) && IS_GOOD(ch))))
		dam -= dam / 4;

	immune = FALSE;

	/*
	 * Check for parry, and dodge.
	 */
	if (dt >= TYPE_HIT && ch != victim)
	{
		if (check_dodge(ch, victim))
			return FALSE;
		if (IS_STANCE(victim, STANCE_MONGOOSE)
			&& GET_STANCE(victim, STANCE_MONGOOSE) > 100 && !can_counter(ch)
			&& !can_bypass(ch, victim) && check_dodge(ch, victim))
			return FALSE;
		else if (IS_STANCE(victim, STANCE_SWALLOW)
				 && GET_STANCE(victim, STANCE_SWALLOW) > 100 && !can_counter(ch)
				 && !can_bypass(ch, victim) && check_dodge(ch, victim))
			return FALSE;
		if (check_parry(ch, victim))
			return FALSE;
		if (IS_STANCE(victim, STANCE_CRANE)
			&& GET_STANCE(victim, STANCE_CRANE) > 100 && !can_counter(ch)
			&& !can_bypass(ch, victim) && check_parry(ch, victim))
			return FALSE;
		else if (IS_STANCE(victim, STANCE_MANTIS)
				 && GET_STANCE(victim, STANCE_MANTIS) > 100 && !can_counter(ch)
				 && !can_bypass(ch, victim) && check_parry(ch, victim))
			return FALSE;
		if (check_shield_block(ch, victim))
			return FALSE;

	}

	switch (check_immune(victim, dam_type))
	{
	case (IS_IMMUNE):
		immune = TRUE;
		dam = 0;
		break;
	case (IS_RESISTANT):
		dam -= dam / 3;
		break;
	case (IS_VULNERABLE):
		dam += dam / 2;
		break;
	}

	if (show)
		dam_message(ch, victim, dam, dt, immune);

	if (dam == 0)
		return FALSE;

	/*
	 * Hurt the victim.
	 * Inform the victim of his new state.
	 */
	victim->hit -= dam;
	if (!IS_NPC(victim) && victim->level >= LEVEL_IMMORTAL && victim->hit < 1)
		victim->hit = 1;
	update_pos(victim);

	switch (victim->position)
	{
	case POS_MORTAL:
		act("$n is mortally wounded, and will die soon, if not aided.",
			victim, NULL, NULL, TO_ROOM);
		chprintln(victim,
				  "You are mortally wounded, and will die soon, if not aided.");
		break;

	case POS_INCAP:
		act("$n is incapacitated and will slowly die, if not aided.",
			victim, NULL, NULL, TO_ROOM);
		chprintln(victim,
				  "You are incapacitated and will slowly die, if not aided.");
		break;

	case POS_STUNNED:
		act("$n is stunned, but will probably recover.", victim, NULL,
			NULL, TO_ROOM);
		chprintln(victim, "You are stunned, but will probably recover.");
		break;

	case POS_DEAD:
		act("$n is DEAD!!", victim, 0, 0, TO_ROOM);
		chprintln(victim, "You have been KILLED!!\n\r");
		break;

	default:
		if (dam > victim->max_hit / 4)
			chprintln(victim, "That really did HURT!");
		if (victim->hit < victim->max_hit / 4)
			chprintln(victim, "You sure are BLEEDING!");
		break;
	}

	/*
	 * Sleep spells and extremely wounded folks.
	 */
	if (!IS_AWAKE(victim))
		stop_fighting(victim, FALSE);

	/*
	 * Payoff for killing things.
	 */
	if (victim->position == POS_DEAD)
	{
		if (IS_IN_ARENA(ch) && IS_IN_ARENA(victim))
		{
			check_arena(ch, victim);
			return TRUE;
		}
		if (IS_IN_WAR(ch) && IS_IN_WAR(victim))
		{
			check_war(ch, victim);
			return TRUE;
		}
		if (!IS_NPC(ch) && !IS_NPC(victim) &&
			IS_SET(ch->in_room->room_flags, ROOM_ARENA) &&
			IS_SET(victim->in_room->room_flags, ROOM_ARENA))
		{
			stop_fighting(victim, TRUE);
			death_cry(victim);
			char_from_room(victim);
			char_to_room(victim, get_room_index(ROOM_VNUM_TEMPLE));
			victim->hit = UMAX(1, victim->hit);
			victim->mana = UMAX(1, victim->mana);
			victim->move = UMAX(1, victim->move);
			update_pos(victim);
			do_look(victim, "auto");
			if (ch->in_room->area->nplayer == 1)
			{
				chprintln(ch, "You emerge victorious in the arena!");
				stop_fighting(ch, TRUE);
				char_from_room(ch);
				char_to_room(ch, get_room_index(ROOM_VNUM_TEMPLE));
				ch->hit = ch->max_hit;
				ch->mana = ch->max_mana;
				ch->move = ch->max_move;
				update_pos(ch);
				do_look(ch, "auto");
			}
			return TRUE;
		}

		group_gain(ch, victim);

		if (!IS_NPC(victim))
		{
			sprintf(log_buf, "%s killed by %s at %ld", victim->name,
					(IS_NPC(ch) ? ch->short_descr : ch->name),
					ch->in_room->vnum);
			log_string(log_buf);

			/*
			 * Dying penalty:
			 * 2/3 way back to previous level.
			 */
			if (!IS_QUESTOR(victim)
				|| (IS_NPC(ch)
					&& ch->pIndexData->vnum != victim->pcdata->questmob))
			{
				if (victim->exp >
					exp_per_level(victim,
								  victim->pcdata->points) * victim->level)
					gain_exp(victim,
							 (2 *
							  (exp_per_level
							   (victim,
								victim->pcdata->points) *
							   victim->level - victim->exp) / 3) + 50);
			}
			if (IS_NPC(ch))
				ADD_STAT(victim, MOB_DEATHS, 1);
			else
			{
				ADD_STAT(victim, PK_DEATHS, 1);
				ADD_STAT(ch, PK_KILLS, 1);
			}
		}
		else if (!IS_NPC(ch))
			ADD_STAT(ch, MOB_KILLS, 1);

		sprintf(log_buf, "%s got toasted by %s at %s [room %ld]",
				(IS_NPC(victim) ? victim->short_descr : victim->name),
				(IS_NPC(ch) ? ch->short_descr : ch->name),
				ch->in_room->name, ch->in_room->vnum);

		if (IS_NPC(victim))
			wiznet(log_buf, NULL, NULL, WIZ_MOBDEATHS, 0, 0);
		else
		{
			wiznet(log_buf, NULL, NULL, WIZ_DEATHS, 0, 0);
			announce(victim, INFO_DEATH, "$n got wasted by %s!",
					 IS_NPC(ch) ? ch->short_descr : ch->name);
		}

		/*
		 * Death trigger
		 */
		if (IS_NPC(victim) && HAS_TRIGGER_MOB(victim, TRIG_DEATH))
		{
			victim->position = POS_STANDING;
			p_percent_trigger(victim, NULL, NULL, ch, NULL, NULL, TRIG_DEATH);
		}

		raw_kill(victim);
		/* dump the flags */
		if (ch != victim && !IS_NPC(ch) && !is_same_clan(ch, victim))
		{
			if (IS_SET(victim->act, PLR_KILLER))
				REMOVE_BIT(victim->act, PLR_KILLER);
			else
				REMOVE_BIT(victim->act, PLR_THIEF);
		}

		/* RT new auto commands */

		if (!IS_NPC(ch) &&
			(corpse =
			 get_obj_list(ch, "corpse", ch->in_room->first_content)) != NULL
			&& corpse->item_type == ITEM_CORPSE_NPC && can_see_obj(ch, corpse))
		{
			OBJ_DATA *coins;

			corpse = get_obj_list(ch, "corpse", ch->in_room->first_content);

			if (IS_SET(ch->act, PLR_AUTOLOOT) && corpse && corpse->first_content)	/* exists and not empty */
			{
				do_function(ch, &do_get, "all corpse");
			}

			if (IS_SET(ch->act, PLR_AUTOGOLD) && corpse && corpse->first_content &&	/* exists and not empty */
				!IS_SET(ch->act, PLR_AUTOLOOT))
			{
				if ((coins =
					 get_obj_list(ch, "gcash", corpse->first_content)) != NULL)
				{
					do_function(ch, &do_get, "all.gcash corpse");
				}
			}

			if (IS_SET(ch->act, PLR_AUTOSAC))
			{
				if (IS_SET(ch->act, PLR_AUTOLOOT) && corpse
					&& corpse->first_content)
				{
					return TRUE;	/* leave if corpse has treasure */
				}
				else
				{
					do_function(ch, &do_sacrifice, "corpse");
				}
			}
		}

		return TRUE;
	}

	if (victim == ch)
		return TRUE;

	/*
	 * Take care of link dead people.
	 */
	if (!IS_NPC(victim) && victim->desc == NULL)
	{
		if (number_range(0, victim->wait) == 0)
		{
			do_function(victim, &do_recall, "");
			return TRUE;
		}
	}

	/*
	 * Wimp out?
	 */
	if (IS_NPC(victim) && dam > 0 && victim->wait < PULSE_VIOLENCE / 2)
	{
		if ((IS_SET(victim->act, ACT_WIMPY) && number_bits(2) == 0 &&
			 victim->hit < victim->max_hit / 5) ||
			(IS_AFFECTED(victim, AFF_CHARM) && victim->master != NULL &&
			 victim->master->in_room != victim->in_room))
		{
			do_function(victim, &do_flee, "");
		}
	}

	if (!IS_NPC(victim) && victim->hit > 0 && victim->hit <= victim->wimpy
		&& victim->wait < PULSE_VIOLENCE / 2)
	{
		do_function(victim, &do_flee, "");
	}

	tail_chain();
	return TRUE;
}

bool is_safe(CHAR_DATA * ch, CHAR_DATA * victim)
{
	if (victim->in_room == NULL || ch->in_room == NULL)
		return TRUE;

	if (victim->fighting == ch || victim == ch)
		return FALSE;

	if (IS_IMMORTAL(ch) && ch->level > LEVEL_IMMORTAL)
		return FALSE;

	/* killing mobiles */
	if (IS_NPC(victim))
	{

		/* safe room? */
		if (IS_SET(victim->in_room->room_flags, ROOM_SAFE))
		{
			chprintln(ch, "Not in this room.");
			return TRUE;
		}

		if (victim->pIndexData->pShop != NULL)
		{
			chprintln(ch, "The shopkeeper wouldn't like that.");
			return TRUE;
		}

		/* no killing healers, trainers, etc */
		if (IS_SET(victim->act, ACT_TRAIN) ||
			IS_SET(victim->act, ACT_PRACTICE) ||
			IS_SET(victim->act, ACT_IS_HEALER) ||
			IS_SET(victim->act, ACT_IS_CHANGER))
		{
			act("I don't think $g would approve.", ch, NULL, NULL, TO_CHAR);
			return TRUE;
		}

		if (!IS_NPC(ch))
		{
			/* no pets */
			if (IS_SET(victim->act, ACT_PET))
			{
				act("But $N looks so cute and cuddly...", ch,
					NULL, victim, TO_CHAR);
				return TRUE;
			}

			/* no charmed creatures unless owner */
			if (IS_AFFECTED(victim, AFF_CHARM) && ch != victim->master)
			{
				chprintln(ch, "You don't own that monster.");
				return TRUE;
			}
		}
	}
	/* killing players */
	else
	{
		/* NPC doing the killing */
		if (IS_NPC(ch))
		{
			/* safe room check */
			if (IS_SET(victim->in_room->room_flags, ROOM_SAFE))
			{
				chprintln(ch, "Not in this room.");
				return TRUE;
			}

			/* charmed mobs and pets cannot attack players while owned */
			if (IS_AFFECTED(ch, AFF_CHARM) && ch->master != NULL &&
				ch->master->fighting != victim)
			{
				chprintln(ch, "Players are your friends!");
				return TRUE;
			}
		}
		/* player doing the killing */
		else
		{

			if (is_safe_war(ch, victim))
			{
				chprintln(ch, "They're on YOUR team.");
				return TRUE;
			}

			if (IS_SET(victim->in_room->room_flags, ROOM_ARENA))
				return FALSE;

			if (!is_clan(ch))
			{
				chprintln(ch, "Join a clan if you want to kill players.");
				return TRUE;
			}

			if (IS_SET(victim->act, PLR_KILLER) ||
				IS_SET(victim->act, PLR_THIEF))
				return FALSE;

			if (!is_clan(victim))
			{
				chprintln(ch, "They aren't in a clan, leave them alone.");
				return TRUE;
			}

			if (IS_SET(victim->in_room->room_flags, ROOM_SAFE))
			{
				chprintln(ch, "Not in this room.");
				return TRUE;
			}

			if (ch->level > victim->level + 8)
			{
				chprintln(ch, "Pick on someone your own size.");
				return TRUE;
			}
			if (ON_GQUEST(ch))
			{
				chprintln(ch, "I don't beleive they are on the target list.");
				return TRUE;
			}

			if (ON_GQUEST(victim))
			{
				chprintln(ch,
						  "They are to closey involved in something right now.");
				return TRUE;
			}

		}
	}
	return FALSE;
}

bool is_safe_spell(CHAR_DATA * ch, CHAR_DATA * victim, bool area)
{
	if (victim->in_room == NULL || ch->in_room == NULL)
		return TRUE;

	if (victim == ch && area)
		return TRUE;

	if (victim->fighting == ch || victim == ch)
		return FALSE;

	if (IS_IMMORTAL(ch) && ch->level > LEVEL_IMMORTAL && !area)
		return FALSE;

	/* killing mobiles */
	if (IS_NPC(victim))
	{
		/* safe room? */
		if (IS_SET(victim->in_room->room_flags, ROOM_SAFE))
			return TRUE;

		if (victim->pIndexData->pShop != NULL)
			return TRUE;

		/* no killing healers, trainers, etc */
		if (IS_SET(victim->act, ACT_TRAIN) ||
			IS_SET(victim->act, ACT_PRACTICE) ||
			IS_SET(victim->act, ACT_IS_HEALER) ||
			IS_SET(victim->act, ACT_IS_CHANGER))
			return TRUE;

		if (!IS_NPC(ch))
		{
			/* no pets */
			if (IS_SET(victim->act, ACT_PET))
				return TRUE;

			/* no charmed creatures unless owner */
			if (IS_AFFECTED(victim, AFF_CHARM) &&
				(area || ch != victim->master))
				return TRUE;

			/* legal kill? -- cannot hit mob fighting non-group member */
			if (victim->fighting != NULL &&
				!is_same_group(ch, victim->fighting))
				return TRUE;
		}
		else
		{
			/* area effect spells do not hit other mobs */
			if (area && !is_same_group(victim, ch->fighting))
				return TRUE;
		}
	}
	/* killing players */
	else
	{
		if (area && IS_IMMORTAL(victim) && victim->level > LEVEL_IMMORTAL)
			return TRUE;

		/* NPC doing the killing */
		if (IS_NPC(ch))
		{
			/* charmed mobs and pets cannot attack players while owned */
			if (IS_AFFECTED(ch, AFF_CHARM) && ch->master != NULL &&
				ch->master->fighting != victim)
				return TRUE;

			/* safe room? */
			if (IS_SET(victim->in_room->room_flags, ROOM_SAFE))
				return TRUE;

			/* legal kill? -- mobs only hit players grouped with opponent */
			if (ch->fighting != NULL && !is_same_group(ch->fighting, victim))
				return TRUE;
		}

		/* player doing the killing */
		else
		{
			if (is_safe_war(ch, victim))
				return TRUE;

			if (IS_SET(victim->in_room->room_flags, ROOM_ARENA))
				return FALSE;

			if (!is_clan(ch))
				return TRUE;

			if (IS_SET(victim->act, PLR_KILLER) ||
				IS_SET(victim->act, PLR_THIEF))
				return FALSE;

			if (!is_clan(victim))
				return TRUE;

			if (ch->level > victim->level + 8)
				return TRUE;

			if (ON_GQUEST(ch) || ON_GQUEST(victim))
				return TRUE;

		}

	}
	return FALSE;
}

/*
 * See if an attack justifies a KILLER flag.
 */
void check_killer(CHAR_DATA * ch, CHAR_DATA * victim)
{
	char buf[MAX_STRING_LENGTH];

	if (IS_SET(ch->in_room->room_flags, ROOM_ARENA))
		return;
	/*
	 * Follow charm thread to responsible character.
	 * Attacking someone's charmed char is hostile!
	 */
	while (IS_AFFECTED(victim, AFF_CHARM) && victim->master != NULL)
		victim = victim->master;

	/*
	 * NPC's are fair game.
	 * So are killers and thieves.
	 */
	if (IS_NPC(victim) || IS_SET(victim->act, PLR_KILLER) ||
		IS_SET(victim->act, PLR_THIEF))
		return;

	/*
	 * Charm-o-rama.
	 */
	if (IS_SET(ch->affected_by, AFF_CHARM))
	{
		if (ch->master == NULL)
		{
			sprintf(buf, "Check_killer: %s bad AFF_CHARM",
					IS_NPC(ch) ? ch->short_descr : ch->name);
			bug(buf);
			affect_strip(ch, gsn_charm_person);
			REMOVE_BIT(ch->affected_by, AFF_CHARM);
			return;
		}
/*
	chprint( "*** You are now a KILLER!! ***\n\r", ch->master );
  	SET_BIT(ch->master->act, PLR_KILLER);
*/

		stop_follower(ch);
		return;
	}

	/*
	 * NPC's are cool of course (as long as not charmed).
	 * Hitting yourself is cool too (bleeding).
	 * So is being immortal (Alander's idea).
	 * And current killers stay as they are.
	 */
	if (IS_NPC(ch) || ch == victim || ch->level >= LEVEL_IMMORTAL ||
		!is_clan(ch) || IS_SET(ch->act, PLR_KILLER) || ch->fighting == victim)
		return;

	chprintln(ch, "*** You are now a KILLER!! ***");
	SET_BIT(ch->act, PLR_KILLER);
	sprintf(buf, "$N is attempting to murder %s", victim->name);
	wiznet(buf, ch, NULL, WIZ_FLAGS, 0, 0);
	save_char_obj(ch);
	return;
}

/*
 * Check for parry.
 */
bool check_parry(CHAR_DATA * ch, CHAR_DATA * victim)
{
	int chance;

	if (!IS_AWAKE(victim))
		return FALSE;

	chance = get_skill(victim, gsn_parry) / 2;

	if (get_eq_char(victim, WEAR_WIELD) == NULL)
	{
		if (IS_NPC(victim))
			chance /= 2;
		else
			return FALSE;
	}

	if (!can_see(ch, victim))
		chance /= 2;

	if (IS_STANCE(victim, STANCE_CRANE) && GET_STANCE(victim, STANCE_CRANE) > 0
		&& !can_counter(ch) && !can_bypass(ch, victim))
		chance += (GET_STANCE(victim, STANCE_CRANE) * 25 / 100);
	else if (IS_STANCE(victim, STANCE_MANTIS)
			 && GET_STANCE(victim, STANCE_MANTIS) > 0 && !can_counter(ch)
			 && !can_bypass(ch, victim))
		chance += (GET_STANCE(victim, STANCE_MANTIS) * 25 / 100);

	if (number_percent() >= chance + victim->level - ch->level)
		return FALSE;

	act("You parry $n's attack.", ch, NULL, victim, TO_VICT);
	act("$N parries your attack.", ch, NULL, victim, TO_CHAR);
	check_improve(victim, gsn_parry, TRUE, 6);
	return TRUE;
}

/*
 * Check for shield block.
 */
bool check_shield_block(CHAR_DATA * ch, CHAR_DATA * victim)
{
	int chance;

	if (!IS_AWAKE(victim))
		return FALSE;

	chance = get_skill(victim, gsn_shield_block) / 5 + 3;

	if (get_eq_char(victim, WEAR_SHIELD) == NULL)
		return FALSE;

	if (number_percent() >= chance + victim->level - ch->level)
		return FALSE;

	act("You block $n's attack with your shield.", ch, NULL, victim, TO_VICT);
	act("$N blocks your attack with a shield.", ch, NULL, victim, TO_CHAR);
	check_improve(victim, gsn_shield_block, TRUE, 6);
	return TRUE;
}

/*
 * Check for dodge.
 */
bool check_dodge(CHAR_DATA * ch, CHAR_DATA * victim)
{
	int chance;

	if (!IS_AWAKE(victim))
		return FALSE;

	chance = get_skill(victim, gsn_dodge) / 2;

	if (!can_see(victim, ch))
		chance /= 2;

	if (IS_STANCE(victim, STANCE_MONGOOSE)
		&& GET_STANCE(victim, STANCE_MONGOOSE) > 0 && !can_counter(ch)
		&& !can_bypass(ch, victim))
		(chance += GET_STANCE(victim, STANCE_MONGOOSE) * 25 / 100);
	if (IS_STANCE(victim, STANCE_SWALLOW)
		&& GET_STANCE(victim, STANCE_SWALLOW) > 0 && !can_counter(ch)
		&& !can_bypass(ch, victim))
		(chance += GET_STANCE(victim, STANCE_SWALLOW) * 25 / 100);

	if (number_percent() >= chance + victim->level - ch->level)
		return FALSE;

	act("You dodge $n's attack.", ch, NULL, victim, TO_VICT);
	act("$N dodges your attack.", ch, NULL, victim, TO_CHAR);
	check_improve(victim, gsn_dodge, TRUE, 6);
	return TRUE;
}

/*
 * Set position of a victim.
 */
void update_pos(CHAR_DATA * victim)
{
	if (victim->hit > 0)
	{
		if (victim->position <= POS_STUNNED)
			victim->position = POS_STANDING;
		return;
	}

	if (IS_NPC(victim) && victim->hit < 1)
	{
		victim->position = POS_DEAD;
		return;
	}

	if (victim->hit <= -11)
	{
		victim->position = POS_DEAD;
		return;
	}

	if (victim->hit <= -6)
		victim->position = POS_MORTAL;
	else if (victim->hit <= -3)
		victim->position = POS_INCAP;
	else
		victim->position = POS_STUNNED;

	return;
}

/*
 * Start fights.
 */
void set_fighting(CHAR_DATA * ch, CHAR_DATA * victim)
{
	if (ch->fighting != NULL)
	{
		bug("Set_fighting: already fighting");
		return;
	}

	if (IS_AFFECTED(ch, AFF_SLEEP))
		affect_strip(ch, gsn_sleep);

	ch->fighting = victim;
	ch->position = POS_FIGHTING;
	autodrop(ch);

	return;
}

/*
 * Stop fights.
 */
void stop_fighting(CHAR_DATA * ch, bool fBoth)
{
	CHAR_DATA *fch;

	for (fch = char_first; fch != NULL; fch = fch->next)
	{
		if (fch == ch || (fBoth && fch->fighting == ch))
		{
			fch->fighting = NULL;
			fch->position = IS_NPC(fch) ? fch->default_pos : POS_STANDING;
			update_pos(fch);
			SET_STANCE(fch, STANCE_CURRENT, STANCE_NONE);
		}
	}
	return;
}

/*
 * Make a corpse out of a character.
 */
void make_corpse(CHAR_DATA * ch)
{
	char buf[MAX_STRING_LENGTH];
	OBJ_DATA *corpse;
	OBJ_DATA *obj;
	OBJ_DATA *obj_next;
	const char *name;

	if (IS_SET(ch->in_room->room_flags, ROOM_ARENA))
		return;

	if (IS_NPC(ch))
	{
		name = ch->short_descr;
		corpse = create_object(get_obj_index(OBJ_VNUM_CORPSE_NPC), 0);
		corpse->timer = number_range(3, 6);
		if (ch->gold > 0)
		{
			obj_to_obj(create_money(ch->gold, ch->silver), corpse);
			ch->gold = 0;
			ch->silver = 0;
		}
		corpse->cost = 0;
	}
	else
	{
		name = ch->name;
		corpse = create_object(get_obj_index(OBJ_VNUM_CORPSE_PC), 0);
		corpse->timer = number_range(25, 40);
		REMOVE_BIT(ch->act, PLR_CANLOOT);
		corpse->owner = str_dup(ch->name);
		if (is_clan(ch))
		{
			if (ch->gold > 1 || ch->silver > 1)
			{
				obj_to_obj(create_money(ch->gold / 2, ch->silver / 2), corpse);
				ch->gold -= ch->gold / 2;
				ch->silver -= ch->silver / 2;
			}
		}

		corpse->cost = 0;
	}

	corpse->level = ch->level;

	sprintf(buf, corpse->short_descr, name);
	replace_string(corpse->short_descr, buf);

	sprintf(buf, corpse->description, name);
	replace_string(corpse->description, buf);

	for (obj = ch->first_carrying; obj != NULL; obj = obj_next)
	{
		bool floating = FALSE;

		obj_next = obj->next_content;
		if (obj->wear_loc == WEAR_FLOAT)
			floating = TRUE;
		obj_from_char(obj);
		if (obj->item_type == ITEM_POTION)
			obj->timer = number_range(500, 1000);
		if (obj->item_type == ITEM_SCROLL)
			obj->timer = number_range(1000, 2500);
		if (IS_SET(obj->extra_flags, ITEM_ROT_DEATH) && !floating)
		{
			obj->timer = number_range(5, 10);
			REMOVE_BIT(obj->extra_flags, ITEM_ROT_DEATH);
		}
		REMOVE_BIT(obj->extra_flags, ITEM_VIS_DEATH);

		if (IS_SET(obj->extra_flags, ITEM_INVENTORY))
			extract_obj(obj);
		else if (floating)
		{
			if (IS_OBJ_STAT(obj, ITEM_ROT_DEATH))	/* get rid of it! */
			{
				if (obj->first_content != NULL)
				{
					OBJ_DATA *in, *in_next;

					act("$p evaporates,scattering its contents.",
						ch, obj, NULL, TO_ROOM);
					for (in = obj->first_content; in != NULL; in = in_next)
					{
						in_next = in->next_content;
						obj_from_obj(in);
						obj_to_room(in, ch->in_room);
					}
				}
				else
					act("$p evaporates.", ch, obj, NULL, TO_ROOM);
				extract_obj(obj);
			}
			else
			{
				act("$p falls to the floor.", ch, obj, NULL, TO_ROOM);
				obj_to_room(obj, ch->in_room);
			}
		}
		else
			obj_to_obj(obj, corpse);
	}

	obj_to_room(corpse, ch->in_room);
	return;
}

/*
 * Improved Death_cry contributed by Diavolo.
 */
void death_cry(CHAR_DATA * ch)
{
	ROOM_INDEX_DATA *was_in_room;
	char *msg;
	int door;
	vnum_t vnum;

	vnum = 0;
	msg = "You hear $n's death cry.";

	switch (number_bits(4))
	{
	case 0:
		msg = "$n hits the ground ... DEAD.";
		break;
	case 1:
		if (ch->material == 0)
		{
			msg = "$n splatters blood on your armor.";
			break;
		}
	case 2:
		if (IS_SET(ch->parts, PART_GUTS))
		{
			msg = "$n spills $s guts all over the floor.";
			vnum = OBJ_VNUM_GUTS;
		}
		break;
	case 3:
		if (IS_SET(ch->parts, PART_HEAD))
		{
			msg = "$n's severed head plops on the ground.";
			vnum = OBJ_VNUM_SEVERED_HEAD;
		}
		break;
	case 4:
		if (IS_SET(ch->parts, PART_HEART))
		{
			msg = "$n's heart is torn from $s chest.";
			vnum = OBJ_VNUM_TORN_HEART;
		}
		break;
	case 5:
		if (IS_SET(ch->parts, PART_ARMS))
		{
			msg = "$n's arm is sliced from $s dead body.";
			vnum = OBJ_VNUM_SLICED_ARM;
		}
		break;
	case 6:
		if (IS_SET(ch->parts, PART_LEGS))
		{
			msg = "$n's leg is sliced from $s dead body.";
			vnum = OBJ_VNUM_SLICED_LEG;
		}
		break;
	case 7:
		if (IS_SET(ch->parts, PART_BRAINS))
		{
			msg = "$n's head is shattered, and $s brains splash all over you.";
			vnum = OBJ_VNUM_BRAINS;
		}
	}

	act(msg, ch, NULL, NULL, TO_ROOM);

	if (vnum != 0)
	{
		char buf[MAX_STRING_LENGTH];
		OBJ_DATA *obj;
		const char *name;

		name = IS_NPC(ch) ? ch->short_descr : ch->name;
		obj = create_object(get_obj_index(vnum), 0);
		obj->timer = number_range(4, 7);

		sprintf(buf, obj->short_descr, name);
		replace_string(obj->short_descr, buf);

		sprintf(buf, obj->description, name);
		replace_string(obj->description, buf);

		if (obj->item_type == ITEM_FOOD)
		{
			if (IS_SET(ch->form, FORM_POISON))
				obj->value[3] = 1;
			else if (!IS_SET(ch->form, FORM_EDIBLE))
				obj->item_type = ITEM_TRASH;
		}

		obj_to_room(obj, ch->in_room);
	}

	if (IS_NPC(ch))
		msg = "You hear something's death cry.";
	else
		msg = "You hear someone's death cry.";

	was_in_room = ch->in_room;
	for (door = 0; door <= 5; door++)
	{
		EXIT_DATA *pexit;

		if ((pexit = was_in_room->exit[door]) != NULL &&
			pexit->u1.to_room != NULL && pexit->u1.to_room != was_in_room)
		{
			ch->in_room = pexit->u1.to_room;
			act(msg, ch, NULL, NULL, TO_ROOM);
		}
	}
	ch->in_room = was_in_room;

	return;
}

void raw_kill(CHAR_DATA * victim)
{
	int i;

	stop_fighting(victim, TRUE);
	death_cry(victim);
	make_corpse(victim);

	if (IS_NPC(victim))
	{
		victim->pIndexData->killed++;
		kill_table[URANGE(0, victim->level, MAX_LEVEL - 1)].killed++;
		extract_char(victim, TRUE);
		return;
	}

	extract_char(victim, FALSE);
	while (victim->first_affect)
		affect_remove(victim, victim->first_affect);
	victim->affected_by = victim->race->aff;
	for (i = 0; i < 4; i++)
		victim->armor[i] = 100;
	victim->position = POS_RESTING;
	victim->hit = UMAX(1, victim->hit);
	victim->mana = UMAX(1, victim->mana);
	victim->move = UMAX(1, victim->move);
/*  save_char_obj( victim ); we're stable enough to not need this :) */
	update_all_qobjs(victim);
	return;
}

void group_gain(CHAR_DATA * ch, CHAR_DATA * victim)
{
	char buf[MAX_STRING_LENGTH];
	CHAR_DATA *gch;
	CHAR_DATA *lch;
	int xp;
	int members;
	int group_levels;
	int i;

	/*
	 * Monsters don't get kill xp's or alignment changes.
	 * P-killing doesn't help either.
	 * Dying of mortal wounds or poison doesn't give xp to anyone!
	 */
	if (victim == ch)
		return;

	members = 0;
	group_levels = 0;
	for (gch = ch->in_room->first_person; gch != NULL; gch = gch->next_in_room)
	{
		if (is_same_group(gch, ch))
		{
			members++;
			group_levels += IS_NPC(gch) ? gch->level / 2 : gch->level;
		}
	}

	if (members == 0)
	{
		bug("Group_gain: members == 0.");
		members = 1;
		group_levels = ch->level;
	}

	lch = (ch->leader != NULL) ? ch->leader : ch;

	for (gch = ch->in_room->first_person; gch != NULL; gch = gch->next_in_room)
	{
		OBJ_DATA *obj;
		OBJ_DATA *obj_next;

		if (!is_same_group(gch, ch) || IS_NPC(gch))
			continue;

/*	Taken out, add it back if you want it
	if ( gch->level - lch->level >= 5 )
	{
	    chprint( "You are too high for this group.\n\r", gch );
	    continue;
	}

	if ( gch->level - lch->level <= -5 )
	{
	    chprint( "You are too low for this group.\n\r", gch );
	    continue;
	}
*/

		xp = xp_compute(gch, victim, group_levels);
		chprintlnf(gch, "You receive %d experience points.", xp);
		gain_exp(gch, xp);

		if (IS_QUESTOR(ch) && IS_NPC(victim))
		{
			if (ch->pcdata->questmob == victim->pIndexData->vnum)
			{
				chprintln(ch, "You have almost completed your QUEST!");
				chprintln(ch,
						  "Return to the questmaster before your time runs out!");
				ch->pcdata->questmob = -1;
			}
		}

		if (IS_NPC(victim) && !IS_NPC(ch) &&
			gquest_info.running == GQUEST_RUNNING && ON_GQUEST(ch) &&
			(i = is_gqmob(ch->gquest, victim->pIndexData->vnum)) != -1)
		{
			ch->gquest->gq_mobs[i] = -1;
			chprintln
				(ch,
				 "Congratulations, that that mob was part of your global quest!");
			chprint(ch, "You receive an extra 3 Quest Points");
			ch->pcdata->questpoints += 3;
			if (chance(URANGE(5, gquest_info.mob_count, 95)))
			{
				chprintln(ch, " and a Trivia Point!");
				ch->pcdata->trivia += 1;
			}
			else
				chprintln(ch, ".");

			sprintf(buf, "$N has killed %s, a global questmob.",
					victim->short_descr);
			wiznet(buf, ch, NULL, 0, 0, 0);

			if (count_gqmobs(ch->gquest) == gquest_info.mob_count)
				chprintln(ch,
						  "You are now ready to complete the global quest. Type 'GQUEST COMPLETE' to finish.");
		}

		for (obj = ch->first_carrying; obj != NULL; obj = obj_next)
		{
			obj_next = obj->next_content;
			if (obj->wear_loc == WEAR_NONE)
				continue;

			if ((IS_OBJ_STAT(obj, ITEM_ANTI_EVIL) && IS_EVIL(ch)) ||
				(IS_OBJ_STAT(obj, ITEM_ANTI_GOOD) && IS_GOOD(ch)) ||
				(IS_OBJ_STAT(obj, ITEM_ANTI_NEUTRAL) && IS_NEUTRAL(ch)))
			{
				act("You are zapped by $p.", ch, obj, NULL, TO_CHAR);
				act("$n is zapped by $p.", ch, obj, NULL, TO_ROOM);
				obj_from_char(obj);
				obj_to_room(obj, ch->in_room);
			}
		}
	}

	return;
}

/*
 * Compute xp for a kill.
 * Also adjust alignment of killer.
 * Edit this function to change xp computations.
 */
int xp_compute(CHAR_DATA * gch, CHAR_DATA * victim, int total_levels)
{
	int xp, base_exp;
	int align, level_range;
	int change;
	int time_per_level;

	level_range = victim->level - gch->level;

	/* compute the base exp */
	switch (level_range)
	{
	default:
		base_exp = 0;
		break;
	case -9:
		base_exp = 1;
		break;
	case -8:
		base_exp = 2;
		break;
	case -7:
		base_exp = 5;
		break;
	case -6:
		base_exp = 9;
		break;
	case -5:
		base_exp = 11;
		break;
	case -4:
		base_exp = 22;
		break;
	case -3:
		base_exp = 33;
		break;
	case -2:
		base_exp = 50;
		break;
	case -1:
		base_exp = 66;
		break;
	case 0:
		base_exp = 83;
		break;
	case 1:
		base_exp = 99;
		break;
	case 2:
		base_exp = 121;
		break;
	case 3:
		base_exp = 143;
		break;
	case 4:
		base_exp = 165;
		break;
	}

	if (level_range > 4)
		base_exp = 160 + 20 * (level_range - 4);

	/* do alignment computations */

	align = victim->alignment - gch->alignment;

	if (IS_SET(victim->act, ACT_NOALIGN))
	{
		/* no change */
	}

	else if (align > 500)		/* monster is more good than slayer */
	{
		change = (align - 500) * base_exp / 500 * gch->level / total_levels;
		change = UMAX(1, change);
		gch->alignment = UMAX(-1000, gch->alignment - change);
	}

	else if (align < -500)		/* monster is more evil than slayer */
	{
		change =
			(-1 * align - 500) * base_exp / 500 * gch->level / total_levels;
		change = UMAX(1, change);
		gch->alignment = UMIN(1000, gch->alignment + change);
	}

	else						/* improve this someday */
	{
		change = gch->alignment * base_exp / 500 * gch->level / total_levels;
		gch->alignment -= change;
	}

	/* calculate exp multiplier */
	if (IS_SET(victim->act, ACT_NOALIGN))
		xp = base_exp;

	else if (gch->alignment > 500)	/* for goodie two shoes */
	{
		if (victim->alignment < -750)
			xp = (base_exp * 4) / 3;

		else if (victim->alignment < -500)
			xp = (base_exp * 5) / 4;

		else if (victim->alignment > 750)
			xp = base_exp / 4;

		else if (victim->alignment > 500)
			xp = base_exp / 2;

		else if (victim->alignment > 250)
			xp = (base_exp * 3) / 4;

		else
			xp = base_exp;
	}

	else if (gch->alignment < -500)	/* for baddies */
	{
		if (victim->alignment > 750)
			xp = (base_exp * 5) / 4;

		else if (victim->alignment > 500)
			xp = (base_exp * 11) / 10;

		else if (victim->alignment < -750)
			xp = base_exp / 2;

		else if (victim->alignment < -500)
			xp = (base_exp * 3) / 4;

		else if (victim->alignment < -250)
			xp = (base_exp * 9) / 10;

		else
			xp = base_exp;
	}

	else if (gch->alignment > 200)	/* a little good */
	{

		if (victim->alignment < -500)
			xp = (base_exp * 6) / 5;

		else if (victim->alignment > 750)
			xp = base_exp / 2;

		else if (victim->alignment > 0)
			xp = (base_exp * 3) / 4;

		else
			xp = base_exp;
	}

	else if (gch->alignment < -200)	/* a little bad */
	{
		if (victim->alignment > 500)
			xp = (base_exp * 6) / 5;

		else if (victim->alignment < -750)
			xp = base_exp / 2;

		else if (victim->alignment < 0)
			xp = (base_exp * 3) / 4;

		else
			xp = base_exp;
	}

	else						/* neutral */
	{

		if (victim->alignment > 500 || victim->alignment < -500)
			xp = (base_exp * 4) / 3;

		else if (victim->alignment < 200 && victim->alignment > -200)
			xp = base_exp / 2;

		else
			xp = base_exp;
	}

	/* more exp at the low levels */
	if (gch->level < 6)
		xp = 10 * xp / (gch->level + 4);

	/* less at high */
	if (gch->level > 35)
		xp = 15 * xp / (gch->level - 25);

	/* reduce for playing time */

	{
		/* compute quarter-hours per level */
		time_per_level =
			4 * (gch->played +
				 (int) (current_time - gch->logon)) / 3600 / gch->level;

		time_per_level = URANGE(2, time_per_level, 12);
		if (gch->level < 15)	/* make it a curve */
			time_per_level = UMAX(time_per_level, (15 - gch->level));
		xp = xp * time_per_level / 12;
	}

	/* randomize the rewards */
	xp = number_range(xp * 3 / 4, xp * 5 / 4);

	/* adjust for grouping */
	xp = xp * gch->level / (UMAX(1, total_levels - 1));

	return xp;
}

void dam_message(CHAR_DATA * ch, CHAR_DATA * victim, int dam, int dt,
				 bool immune)
{
	char buf1[256], buf2[256], buf3[256], chmesg[256], vmesg[256], omesg[256];
	const char *vs;
	const char *vp;
	const char *attack;
	char punct;

	if (ch == NULL || victim == NULL)
		return;

	sprintf(chmesg, "{W [{R%d{W]{x", dam);
	sprintf(vmesg, "{W [{R%d{W]{x", dam);
	sprintf(omesg, "{W [{R%d{W]{x", dam);

	if (dam == 0)
	{
		vs = "miss";
		vp = "misses";
	}
	else if (dam <= 4)
	{
		vs = "scratch";
		vp = "scratches";
	}
	else if (dam <= 8)
	{
		vs = "graze";
		vp = "grazes";
	}
	else if (dam <= 12)
	{
		vs = "hit";
		vp = "hits";
	}
	else if (dam <= 16)
	{
		vs = "injure";
		vp = "injures";
	}
	else if (dam <= 20)
	{
		vs = "wound";
		vp = "wounds";
	}
	else if (dam <= 24)
	{
		vs = "maul";
		vp = "mauls";
	}
	else if (dam <= 28)
	{
		vs = "decimate";
		vp = "decimates";
	}
	else if (dam <= 32)
	{
		vs = "devastate";
		vp = "devastates";
	}
	else if (dam <= 36)
	{
		vs = "maim";
		vp = "maims";
	}
	else if (dam <= 40)
	{
		vs = "MUTILATE";
		vp = "MUTILATES";
	}
	else if (dam <= 44)
	{
		vs = "DISEMBOWEL";
		vp = "DISEMBOWELS";
	}
	else if (dam <= 48)
	{
		vs = "DISMEMBER";
		vp = "DISMEMBERS";
	}
	else if (dam <= 52)
	{
		vs = "MASSACRE";
		vp = "MASSACRES";
	}
	else if (dam <= 56)
	{
		vs = "MANGLE";
		vp = "MANGLES";
	}
	else if (dam <= 60)
	{
		vs = "*** DEMOLISH ***";
		vp = "*** DEMOLISHES ***";
	}
	else if (dam <= 75)
	{
		vs = "*** DEVASTATE ***";
		vp = "*** DEVASTATES ***";
	}
	else if (dam <= 100)
	{
		vs = "=== OBLITERATE ===";
		vp = "=== OBLITERATES ===";
	}
	else if (dam <= 125)
	{
		vs = ">>> ANNIHILATE <<<";
		vp = ">>> ANNIHILATES <<<";
	}
	else if (dam <= 150)
	{
		vs = "<<< ERADICATE >>>";
		vp = "<<< ERADICATES >>>";
	}
	else
	{
		vs = "do UNSPEAKABLE things to";
		vp = "does UNSPEAKABLE things to";
	}

	punct = (dam <= 24) ? '.' : '!';

#define SEE_DAMAGE(ch) (!IS_NPC(ch) && IS_SET(ch->act, PLR_AUTODAMAGE))

	if (dt == TYPE_HIT)
	{
		if (ch == victim)
		{
			sprintf(buf1, CTAG(_OHIT) "$n %s" CTAG(_OHIT) " $melf%c$t", vp,
					punct);
			sprintf(buf2, CTAG(_YHIT) "You %s" CTAG(_YHIT) " yourself%c%s", vs,
					punct, SEE_DAMAGE(ch) ? chmesg : "{x");
		}
		else
		{
			sprintf(buf1, CTAG(_OHIT) "$n %s" CTAG(_OHIT) " $N%c$t", vp, punct);
			sprintf(buf2, CTAG(_YHIT) "You %s" CTAG(_YHIT) " $N%c%s", vs, punct,
					SEE_DAMAGE(ch) ? chmesg : "{x");
			sprintf(buf3, CTAG(_VHIT) "$n %s" CTAG(_VHIT) " you%c%s", vp, punct,
					SEE_DAMAGE(victim) ? vmesg : "{x");
		}
	}
	else
	{
		if (dt >= 0 && dt < maxSkill)
			attack = skill_table[dt].noun_damage;
		else if (dt >= TYPE_HIT && dt < TYPE_HIT + MAX_DAMAGE_MESSAGE)
			attack = attack_table[dt - TYPE_HIT].noun;
		else
		{
			bugf("Dam_message: bad dt %d.", dt);
			dt = TYPE_HIT;
			attack = attack_table[0].name;
		}

		if (immune)
		{
			if (ch == victim)
			{
				sprintf(buf1, CTAG(_OHIT) "$n is unaffected by $s own %s.{x",
						attack);
				sprintf(buf2, CTAG(_YHIT) "Luckily, you are immune to that.{x");
			}
			else
			{
				sprintf(buf1, CTAG(_OHIT) "$N is unaffected by $n's %s!{x",
						attack);
				sprintf(buf2, CTAG(_YHIT) "$N is unaffected by your %s!{x",
						attack);
				sprintf(buf3, CTAG(_VHIT) "$n's %s is powerless against you.{x",
						attack);
			}
		}
		else
		{
			if (ch == victim)
			{
				sprintf(buf1, CTAG(_OHIT) "$n's %s %s" CTAG(_OHIT) " $m%c$t",
						attack, vp, punct);
				sprintf(buf2, CTAG(_YHIT) "Your %s %s" CTAG(_YHIT) " you%c%s",
						attack, vp, punct, SEE_DAMAGE(ch) ? chmesg : "{x");
			}
			else
			{
				sprintf(buf1, CTAG(_OHIT) "$n's %s %s" CTAG(_OHIT) " $N%c$t",
						attack, vp, punct);
				sprintf(buf2, CTAG(_YHIT) "Your %s %s" CTAG(_YHIT) " $N%c%s",
						attack, vp, punct, SEE_DAMAGE(ch) ? chmesg : "{x");
				sprintf(buf3, CTAG(_VHIT) "$n's %s %s" CTAG(_VHIT) " you%c%s",
						attack, vp, punct, SEE_DAMAGE(victim) ? vmesg : "{x");
			}
		}
	}

	if (ch == victim)
	{
		act(buf1, ch, omesg, NULL, TO_ROOM | TO_DAMAGE);
		act(buf2, ch, NULL, NULL, TO_CHAR);
	}
	else
	{
		act(buf1, ch, omesg, victim, TO_NOTVICT | TO_DAMAGE);
		act(buf2, ch, NULL, victim, TO_CHAR);
		act(buf3, ch, NULL, victim, TO_VICT);
	}

	return;
}

/*
 * Disarm a creature.
 * Caller must check for successful attack.
 */
void disarm(CHAR_DATA * ch, CHAR_DATA * victim)
{
	OBJ_DATA *obj;

	if ((obj = get_eq_char(victim, WEAR_WIELD)) == NULL)
		return;

	if (IS_OBJ_STAT(obj, ITEM_NOREMOVE))
	{
		act("$S weapon won't budge!", ch, NULL, victim, TO_CHAR);
		act("$n tries to disarm you, but your weapon won't budge!", ch,
			NULL, victim, TO_VICT);
		act("$n tries to disarm $N, but fails.", ch, NULL, victim, TO_NOTVICT);
		return;
	}

	act("$n DISARMS you and sends your weapon flying!", ch, NULL, victim,
		TO_VICT);
	act("You disarm $N!", ch, NULL, victim, TO_CHAR);
	act("$n disarms $N!", ch, NULL, victim, TO_NOTVICT);

	obj_from_char(obj);
	if (IS_OBJ_STAT(obj, ITEM_NODROP) || IS_OBJ_STAT(obj, ITEM_INVENTORY) ||
		IS_SET(victim->in_room->room_flags, ROOM_ARENA))
		obj_to_char(obj, victim);
	else
	{
		obj_to_room(obj, victim->in_room);
		if (IS_NPC(victim) && victim->wait == 0 && can_see_obj(victim, obj))
			get_obj(victim, obj, NULL);
	}

	return;
}

CH_CMD(do_berserk)
{
	int chance, hp_percent;

	if ((chance = get_skill(ch, gsn_berserk)) == 0 ||
		(IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_BERSERK)) ||
		(!IS_NPC(ch) && !can_use_skpell(ch, gsn_berserk)))
	{
		chprintln(ch, "You turn red in the face, but nothing happens.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_BERSERK) || is_affected(ch, gsn_berserk) ||
		is_affected(ch, skill_lookup("frenzy")))
	{
		chprintln(ch, "You get a little madder.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CALM))
	{
		chprintln(ch, "You're feeling to mellow to berserk.");
		return;
	}

	if (ch->mana < 50)
	{
		chprintln(ch, "You can't get up enough energy.");
		return;
	}

	/* modifiers */

	/* fighting */
	if (ch->position == POS_FIGHTING)
		chance += 10;

	/* damage -- below 50% of hp helps, above hurts */
	hp_percent = 100 * ch->hit / ch->max_hit;
	chance += 25 - hp_percent / 2;

	if (number_percent() < chance)
	{
		AFFECT_DATA af;

		WAIT_STATE(ch, PULSE_VIOLENCE);
		ch->mana -= 50;
		ch->move /= 2;

		/* heal a little damage */
		ch->hit += ch->level * 2;
		ch->hit = UMIN(ch->hit, ch->max_hit);

		chprintln(ch, "Your pulse races as you are consumed by rage!");
		act("$n gets a wild look in $s eyes.", ch, NULL, NULL, TO_ROOM);
		check_improve(ch, gsn_berserk, TRUE, 2);

		af.where = TO_AFFECTS;
		af.type = gsn_berserk;
		af.level = ch->level;
		af.duration = number_fuzzy(ch->level / 8);
		af.modifier = UMAX(1, ch->level / 5);
		af.bitvector = AFF_BERSERK;

		af.location = APPLY_HITROLL;
		affect_to_char(ch, &af);

		af.location = APPLY_DAMROLL;
		affect_to_char(ch, &af);

		af.modifier = UMAX(10, 10 * (ch->level / 5));
		af.location = APPLY_AC;
		affect_to_char(ch, &af);
	}

	else
	{
		WAIT_STATE(ch, 3 * PULSE_VIOLENCE);
		ch->mana -= 25;
		ch->move /= 2;

		chprintln(ch, "Your pulse speeds up, but nothing happens.");
		check_improve(ch, gsn_berserk, FALSE, 2);
	}
}

CH_CMD(do_bash)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	int chance;

	one_argument(argument, arg);

	if ((chance = get_skill(ch, gsn_bash)) == 0 ||
		(IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_BASH)) ||
		(!IS_NPC(ch) && !can_use_skpell(ch, gsn_bash)))
	{
		chprintln(ch, "Bashing? What's that?");
		return;
	}

	if (IS_NULLSTR(arg))
	{
		victim = ch->fighting;
		if (victim == NULL)
		{
			chprintln(ch, "But you aren't fighting anyone!");
			return;
		}
	}

	else if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (victim->position < POS_FIGHTING)
	{
		act("You'll have to let $M get back up first.", ch, NULL, victim,
			TO_CHAR);
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "You try to bash your brains out, but fail.");
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (IS_NPC(victim) && victim->fighting != NULL &&
		!is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim)
	{
		act("But $N is your friend!", ch, NULL, victim, TO_CHAR);
		return;
	}

	/* modifiers */

	/* size  and weight */
	chance += ch->carry_weight / 250;
	chance -= victim->carry_weight / 200;

	if (ch->size < victim->size)
		chance += (ch->size - victim->size) * 15;
	else
		chance += (ch->size - victim->size) * 10;

	/* stats */
	chance += get_curr_stat(ch, STAT_STR);
	chance -= (get_curr_stat(victim, STAT_DEX) * 4) / 3;
	chance -= GET_AC(victim, AC_BASH) / 25;
	/* speed */
	if (IS_SET(ch->off_flags, OFF_FAST) || IS_AFFECTED(ch, AFF_HASTE))
		chance += 10;
	if (IS_SET(victim->off_flags, OFF_FAST) || IS_AFFECTED(victim, AFF_HASTE))
		chance -= 30;

	/* level */
	chance += (ch->level - victim->level);

	if (!IS_NPC(victim) && chance < get_skill(victim, gsn_dodge))
	{							/*
								   act("$n tries to bash you, but you dodge it.",ch,NULL,victim,TO_VICT);
								   act("$N dodges your bash, you fall flat on your face.",ch,NULL,victim,TO_CHAR);
								   WAIT_STATE(ch,skill_table[gsn_bash].beats);
								   return; */
		chance -= 3 * (get_skill(victim, gsn_dodge) - chance);
	}

	/* now the attack */
	if (number_percent() < chance)
	{

		act("$n sends you sprawling with a powerful bash!", ch, NULL,
			victim, TO_VICT);
		act("You slam into $N, and send $M flying!", ch, NULL, victim, TO_CHAR);
		act("$n sends $N sprawling with a powerful bash.", ch, NULL,
			victim, TO_NOTVICT);
		check_improve(ch, gsn_bash, TRUE, 1);

		DAZE_STATE(victim, 3 * PULSE_VIOLENCE);
		WAIT_STATE(ch, skill_table[gsn_bash].beats);
		victim->position = POS_RESTING;
		damage(ch, victim,
			   number_range(2, 2 + 2 * ch->size + chance / 20),
			   gsn_bash, DAM_BASH, FALSE);

	}
	else
	{
		damage(ch, victim, 0, gsn_bash, DAM_BASH, FALSE);
		act("You fall flat on your face!", ch, NULL, victim, TO_CHAR);
		act("$n falls flat on $s face.", ch, NULL, victim, TO_NOTVICT);
		act("You evade $n's bash, causing $m to fall flat on $s face.",
			ch, NULL, victim, TO_VICT);
		check_improve(ch, gsn_bash, FALSE, 1);
		ch->position = POS_RESTING;
		WAIT_STATE(ch, skill_table[gsn_bash].beats * 3 / 2);
	}
	check_killer(ch, victim);
}

CH_CMD(do_dirt)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	int chance;

	one_argument(argument, arg);

	if ((chance = get_skill(ch, gsn_dirt)) == 0 ||
		(IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_KICK_DIRT)) ||
		(!IS_NPC(ch) && !can_use_skpell(ch, gsn_dirt)))
	{
		chprintln(ch, "You get your feet dirty.");
		return;
	}

	if (IS_NULLSTR(arg))
	{
		victim = ch->fighting;
		if (victim == NULL)
		{
			chprintln(ch, "But you aren't in combat!");
			return;
		}
	}

	else if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (IS_AFFECTED(victim, AFF_BLIND))
	{
		act("$E's already been blinded.", ch, NULL, victim, TO_CHAR);
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "Very funny.");
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (IS_NPC(victim) && victim->fighting != NULL &&
		!is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim)
	{
		act("But $N is such a good friend!", ch, NULL, victim, TO_CHAR);
		return;
	}

	/* modifiers */

	/* dexterity */
	chance += get_curr_stat(ch, STAT_DEX);
	chance -= 2 * get_curr_stat(victim, STAT_DEX);

	/* speed  */
	if (IS_SET(ch->off_flags, OFF_FAST) || IS_AFFECTED(ch, AFF_HASTE))
		chance += 10;
	if (IS_SET(victim->off_flags, OFF_FAST) || IS_AFFECTED(victim, AFF_HASTE))
		chance -= 25;

	/* level */
	chance += (ch->level - victim->level) * 2;

	/* sloppy hack to prevent FALSE zeroes */
	if (chance % 5 == 0)
		chance += 1;

	/* terrain */

	switch (ch->in_room->sector_type)
	{
	case (SECT_INSIDE):
		chance -= 20;
		break;
	case (SECT_CITY):
		chance -= 10;
		break;
	case (SECT_FIELD):
		chance += 5;
		break;
	case (SECT_FOREST):
		break;
	case (SECT_HILLS):
		break;
	case (SECT_MOUNTAIN):
		chance -= 10;
		break;
	case (SECT_WATER_SWIM):
		chance = 0;
		break;
	case (SECT_WATER_NOSWIM):
		chance = 0;
		break;
	case (SECT_AIR):
		chance = 0;
		break;
	case (SECT_DESERT):
		chance += 10;
		break;
	default:
		break;
	}

	if (chance == 0)
	{
		chprintln(ch, "There isn't any dirt to kick.");
		return;
	}

	/* now the attack */
	if (number_percent() < chance)
	{
		AFFECT_DATA af;
		act("$n is blinded by the dirt in $s eyes!", victim, NULL, NULL,
			TO_ROOM);
		act("$n kicks dirt in your eyes!", ch, NULL, victim, TO_VICT);
		damage(ch, victim, number_range(2, 5), gsn_dirt, DAM_NONE, FALSE);
		chprintln(victim, "You can't see a thing!");
		check_improve(ch, gsn_dirt, TRUE, 2);
		WAIT_STATE(ch, skill_table[gsn_dirt].beats);

		af.where = TO_AFFECTS;
		af.type = gsn_dirt;
		af.level = ch->level;
		af.duration = 0;
		af.location = APPLY_HITROLL;
		af.modifier = -4;
		af.bitvector = AFF_BLIND;

		affect_to_char(victim, &af);
	}
	else
	{
		damage(ch, victim, 0, gsn_dirt, DAM_NONE, TRUE);
		check_improve(ch, gsn_dirt, FALSE, 2);
		WAIT_STATE(ch, skill_table[gsn_dirt].beats);
	}
	check_killer(ch, victim);
}

CH_CMD(do_trip)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	int chance;

	one_argument(argument, arg);

	if ((chance = get_skill(ch, gsn_trip)) == 0 ||
		(IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_TRIP)) ||
		(!IS_NPC(ch) && !can_use_skpell(ch, gsn_trip)))
	{
		chprintln(ch, "Tripping?  What's that?");
		return;
	}

	if (IS_NULLSTR(arg))
	{
		victim = ch->fighting;
		if (victim == NULL)
		{
			chprintln(ch, "But you aren't fighting anyone!");
			return;
		}
	}

	else if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (IS_NPC(victim) && victim->fighting != NULL &&
		!is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if (IS_AFFECTED(victim, AFF_FLYING))
	{
		act("$S feet aren't on the ground.", ch, NULL, victim, TO_CHAR);
		return;
	}

	if (victim->position < POS_FIGHTING)
	{
		act("$N is already down.", ch, NULL, victim, TO_CHAR);
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "You fall flat on your face!");
		WAIT_STATE(ch, 2 * skill_table[gsn_trip].beats);
		act("$n trips over $s own feet!", ch, NULL, NULL, TO_ROOM);
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim)
	{
		act("$N is your beloved master.", ch, NULL, victim, TO_CHAR);
		return;
	}

	/* modifiers */

	/* size */
	if (ch->size < victim->size)
		chance += (ch->size - victim->size) * 10;	/* bigger = harder to trip */

	/* dex */
	chance += get_curr_stat(ch, STAT_DEX);
	chance -= get_curr_stat(victim, STAT_DEX) * 3 / 2;

	/* speed */
	if (IS_SET(ch->off_flags, OFF_FAST) || IS_AFFECTED(ch, AFF_HASTE))
		chance += 10;
	if (IS_SET(victim->off_flags, OFF_FAST) || IS_AFFECTED(victim, AFF_HASTE))
		chance -= 20;

	/* level */
	chance += (ch->level - victim->level) * 2;

	/* now the attack */
	if (number_percent() < chance)
	{
		act("$n trips you and you go down!", ch, NULL, victim, TO_VICT);
		act("You trip $N and $N goes down!", ch, NULL, victim, TO_CHAR);
		act("$n trips $N, sending $M to the ground.", ch, NULL, victim,
			TO_NOTVICT);
		check_improve(ch, gsn_trip, TRUE, 1);

		DAZE_STATE(victim, 2 * PULSE_VIOLENCE);
		WAIT_STATE(ch, skill_table[gsn_trip].beats);
		victim->position = POS_RESTING;
		damage(ch, victim, number_range(2, 2 + 2 * victim->size),
			   gsn_trip, DAM_BASH, TRUE);
		if (number_percent() < chance - 5
			&& IS_VALID_STANCE(GET_STANCE(victim, STANCE_CURRENT)))
		{
			SET_STANCE(victim, STANCE_CURRENT, STANCE_NONE);
			act("You trip up $N's stance!", ch, NULL, victim, TO_CHAR);
			act("$n trips up $N's stance!", ch, NULL, victim, TO_NOTVICT);
			act("$n trips up your stance!", ch, NULL, victim, TO_VICT);
		}
	}
	else
	{
		damage(ch, victim, 0, gsn_trip, DAM_BASH, TRUE);
		WAIT_STATE(ch, skill_table[gsn_trip].beats * 2 / 3);
		check_improve(ch, gsn_trip, FALSE, 1);
	}
	check_killer(ch, victim);
}

CH_CMD(do_kill)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;

	one_argument(argument, arg);

	if (IS_NULLSTR(arg))
	{
		chprintln(ch, "Kill whom?");
		return;
	}

	if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}
/*  Allow player killing
    if ( !IS_NPC(victim) )
    {
        if ( !IS_SET(victim->act, PLR_KILLER)
        &&   !IS_SET(victim->act, PLR_THIEF) )
        {
            chprint( "You must MURDER a player.\n\r", ch );
            return;
        }
    }
*/
	if (victim == ch)
	{
		chprintln(ch, "You hit yourself.  Ouch!");
		multi_hit(ch, ch, TYPE_UNDEFINED);
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (victim->fighting != NULL && !is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim)
	{
		act("$N is your beloved master.", ch, NULL, victim, TO_CHAR);
		return;
	}

	if (ch->position == POS_FIGHTING)
	{
		chprintln(ch, "You do the best you can!");
		return;
	}

	WAIT_STATE(ch, 1 * PULSE_VIOLENCE);
	check_killer(ch, victim);
	multi_hit(ch, victim, TYPE_UNDEFINED);
	return;
}

CH_CMD(do_murde)
{
	chprintln(ch, "If you want to MURDER, spell it out.");
	return;
}

CH_CMD(do_murder)
{
	char buf[MAX_STRING_LENGTH];
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;

	one_argument(argument, arg);

	if (IS_NULLSTR(arg))
	{
		chprintln(ch, "Murder whom?");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) || (IS_NPC(ch) && IS_SET(ch->act, ACT_PET)))
		return;

	if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "Suicide is a mortal sin.");
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (IS_NPC(victim) && victim->fighting != NULL &&
		!is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if (IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim)
	{
		act("$N is your beloved master.", ch, NULL, victim, TO_CHAR);
		return;
	}

	if (ch->position == POS_FIGHTING)
	{
		chprintln(ch, "You do the best you can!");
		return;
	}

	WAIT_STATE(ch, 1 * PULSE_VIOLENCE);
	if (IS_NPC(ch))
		sprintf(buf, "Help! I am being attacked by %s!", ch->short_descr);
	else
		sprintf(buf, "Help!  I am being attacked by %s!", ch->name);
	do_function(victim, &do_yell, buf);
	check_killer(ch, victim);
	multi_hit(ch, victim, TYPE_UNDEFINED);
	return;
}

CH_CMD(do_backstab)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	OBJ_DATA *obj;

	one_argument(argument, arg);

	if (IS_NULLSTR(arg))
	{
		chprintln(ch, "Backstab whom?");
		return;
	}

	if (ch->fighting != NULL)
	{
		chprintln(ch, "You're facing the wrong end.");
		return;
	}

	else if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "How can you sneak up on yourself?");
		return;
	}

	if (is_safe(ch, victim))
		return;

	if (IS_NPC(victim) && victim->fighting != NULL &&
		!is_same_group(ch, victim->fighting))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	if ((obj = get_eq_char(ch, WEAR_WIELD)) == NULL)
	{
		chprintln(ch, "You need to wield a weapon to backstab.");
		return;
	}

	if (victim->hit < victim->max_hit / 3)
	{
		act("$N is hurt and suspicious ... you can't sneak up.", ch,
			NULL, victim, TO_CHAR);
		return;
	}

	check_killer(ch, victim);
	WAIT_STATE(ch, skill_table[gsn_backstab].beats);
	if (number_percent() < get_skill(ch, gsn_backstab) ||
		(get_skill(ch, gsn_backstab) >= 2 && !IS_AWAKE(victim)))
	{
		check_improve(ch, gsn_backstab, TRUE, 1);
		multi_hit(ch, victim, gsn_backstab);
	}
	else
	{
		check_improve(ch, gsn_backstab, FALSE, 1);
		damage(ch, victim, 0, gsn_backstab, DAM_NONE, TRUE);
	}

	return;
}

CH_CMD(do_flee)
{
	ROOM_INDEX_DATA *was_in;
	ROOM_INDEX_DATA *now_in;
	CHAR_DATA *victim;
	int attempt;

	if ((victim = ch->fighting) == NULL)
	{
		if (ch->position == POS_FIGHTING)
			ch->position = POS_STANDING;
		chprintln(ch, "You aren't fighting anyone.");
		return;
	}

	if (IS_SET(ch->in_room->room_flags, ROOM_ARENA))
		return;

	was_in = ch->in_room;
	for (attempt = 0; attempt < 6; attempt++)
	{
		EXIT_DATA *pexit;
		int door;

		door = number_door();
		if ((pexit = was_in->exit[door]) == 0 || pexit->u1.to_room == NULL
			|| IS_SET(pexit->exit_info, EX_CLOSED) ||
			number_range(0, ch->daze) != 0 || (IS_NPC(ch) &&
											   IS_SET(pexit->u1.
													  to_room->room_flags,
													  ROOM_NO_MOB)))
			continue;

		move_char(ch, door, FALSE);
		if ((now_in = ch->in_room) == was_in)
			continue;

		ch->in_room = was_in;
		act("$n has fled!", ch, NULL, NULL, TO_ROOM);
		ch->in_room = now_in;

		if (!IS_NPC(ch))
		{
			chprintln(ch, "You flee from combat!");
			if ((is_class(ch, 2)) && (number_percent() < 3 * (ch->level / 2)))
				chprintln(ch, "You snuck away safely.");
			else
			{
				chprintln(ch, "You lost 10 exp.");
				gain_exp(ch, -10);
			}
		}

		stop_fighting(ch, TRUE);
		return;
	}

	chprintln(ch, "PANIC! You couldn't escape!");
	return;
}

CH_CMD(do_rescue)
{
	char arg[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	CHAR_DATA *fch;

	one_argument(argument, arg);
	if (IS_NULLSTR(arg))
	{
		chprintln(ch, "Rescue whom?");
		return;
	}

	if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (victim == ch)
	{
		chprintln(ch, "What about fleeing instead?");
		return;
	}

	if (!IS_NPC(ch) && IS_NPC(victim))
	{
		chprintln(ch, "Doesn't need your help!");
		return;
	}

	if (ch->fighting == victim)
	{
		chprintln(ch, "Too late.");
		return;
	}

	if ((fch = victim->fighting) == NULL)
	{
		chprintln(ch, "That person is not fighting right now.");
		return;
	}

	if (IS_NPC(fch) && !is_same_group(ch, victim))
	{
		chprintln(ch, "Kill stealing is not permitted.");
		return;
	}

	WAIT_STATE(ch, skill_table[gsn_rescue].beats);
	if (number_percent() > get_skill(ch, gsn_rescue))
	{
		chprintln(ch, "You fail the rescue.");
		check_improve(ch, gsn_rescue, FALSE, 1);
		return;
	}

	act("You rescue $N!", ch, NULL, victim, TO_CHAR);
	act("$n rescues you!", ch, NULL, victim, TO_VICT);
	act("$n rescues $N!", ch, NULL, victim, TO_NOTVICT);
	check_improve(ch, gsn_rescue, TRUE, 1);

	stop_fighting(fch, FALSE);
	stop_fighting(victim, FALSE);

	check_killer(ch, fch);
	set_fighting(ch, fch);
	set_fighting(fch, ch);
	return;
}

CH_CMD(do_kick)
{
	CHAR_DATA *victim;

	if (!IS_NPC(ch) && !can_use_skpell(ch, gsn_kick))
	{
		chprintln(ch, "You better leave the martial arts to fighters.");
		return;
	}

	if (IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_KICK))
		return;

	if ((victim = ch->fighting) == NULL)
	{
		chprintln(ch, "You aren't fighting anyone.");
		return;
	}

	WAIT_STATE(ch, skill_table[gsn_kick].beats);
	if (get_skill(ch, gsn_kick) > number_percent())
	{
		damage(ch, victim, number_range(1, ch->level), gsn_kick,
			   DAM_BASH, TRUE);
		check_improve(ch, gsn_kick, TRUE, 1);
	}
	else
	{
		damage(ch, victim, 0, gsn_kick, DAM_BASH, TRUE);
		check_improve(ch, gsn_kick, FALSE, 1);
	}
	check_killer(ch, victim);
	return;
}

CH_CMD(do_disarm)
{
	CHAR_DATA *victim;
	OBJ_DATA *obj;
	int chance, hth, ch_weapon, vict_weapon, ch_vict_weapon;

	hth = 0;

	if ((chance = get_skill(ch, gsn_disarm)) == 0)
	{
		chprintln(ch, "You don't know how to disarm opponents.");
		return;
	}

	if (get_eq_char(ch, WEAR_WIELD) == NULL &&
		((hth = get_skill(ch, gsn_hand_to_hand)) == 0 ||
		 (IS_NPC(ch) && !IS_SET(ch->off_flags, OFF_DISARM))))
	{
		chprintln(ch, "You must wield a weapon to disarm.");
		return;
	}

	if ((victim = ch->fighting) == NULL)
	{
		chprintln(ch, "You aren't fighting anyone.");
		return;
	}

	if ((obj = get_eq_char(victim, WEAR_WIELD)) == NULL)
	{
		chprintln(ch, "Your opponent is not wielding a weapon.");
		return;
	}

	/* find weapon skills */
	ch_weapon = get_weapon_skill(ch, get_weapon_sn(ch));
	vict_weapon = get_weapon_skill(victim, get_weapon_sn(victim));
	ch_vict_weapon = get_weapon_skill(ch, get_weapon_sn(victim));

	/* modifiers */

	/* skill */
	if (get_eq_char(ch, WEAR_WIELD) == NULL)
		chance = chance * hth / 150;
	else
		chance = chance * ch_weapon / 100;

	chance += (ch_vict_weapon / 2 - vict_weapon) / 2;

	/* dex vs. strength */
	chance += get_curr_stat(ch, STAT_DEX);
	chance -= 2 * get_curr_stat(victim, STAT_STR);

	/* level */
	chance += (ch->level - victim->level) * 2;

	/* and now the attack */
	if (number_percent() < chance)
	{
		WAIT_STATE(ch, skill_table[gsn_disarm].beats);
		disarm(ch, victim);
		check_improve(ch, gsn_disarm, TRUE, 1);
	}
	else
	{
		WAIT_STATE(ch, skill_table[gsn_disarm].beats);
		act("You fail to disarm $N.", ch, NULL, victim, TO_CHAR);
		act("$n tries to disarm you, but fails.", ch, NULL, victim, TO_VICT);
		act("$n tries to disarm $N, but fails.", ch, NULL, victim, TO_NOTVICT);
		check_improve(ch, gsn_disarm, FALSE, 1);
	}
	check_killer(ch, victim);
	return;
}

CH_CMD(do_surrender)
{
	CHAR_DATA *mob;
	if ((mob = ch->fighting) == NULL)
	{
		chprintln(ch, "But you're not fighting!");
		return;
	}
	act("You surrender to $N!", ch, NULL, mob, TO_CHAR);
	act("$n surrenders to you!", ch, NULL, mob, TO_VICT);
	act("$n tries to surrender to $N!", ch, NULL, mob, TO_NOTVICT);
	stop_fighting(ch, TRUE);

	if (!IS_NPC(ch) && IS_NPC(mob) &&
		(!HAS_TRIGGER_MOB(mob, TRIG_SURR) ||
		 !p_percent_trigger(mob, NULL, NULL, ch, NULL, NULL, TRIG_SURR)))
	{
		act("$N seems to ignore your cowardly act!", ch, NULL, mob, TO_CHAR);
		multi_hit(mob, ch, TYPE_UNDEFINED);
	}
}

CH_CMD(do_sla)
{
	chprintln(ch, "If you want to SLAY, spell it out.");
	return;
}

CH_CMD(do_slay)
{
	CHAR_DATA *victim;
	char arg[MAX_INPUT_LENGTH];

	one_argument(argument, arg);
	if (IS_NULLSTR(arg))
	{
		chprintln(ch, "Slay whom?");
		return;
	}

	if ((victim = get_char_room(ch, NULL, arg)) == NULL)
	{
		chprintln(ch, "They aren't here.");
		return;
	}

	if (ch == victim)
	{
		chprintln(ch, "Suicide is a mortal sin.");
		return;
	}

	if (!IS_NPC(victim) && victim->level >= get_trust(ch))
	{
		chprintln(ch, "You failed.");
		return;
	}

	act("You slay $M in cold blood!", ch, NULL, victim, TO_CHAR);
	act("$n slays you in cold blood!", ch, NULL, victim, TO_VICT);
	act("$n slays $N in cold blood!", ch, NULL, victim, TO_NOTVICT);
	raw_kill(victim);
	return;
}

CH_CMD(do_sskill)
{
	int ast = GET_STANCE(ch, STANCE_AUTODROP);

	chprintlnf(ch, "{W%s", draw_line(ch, NULL, 0));
	chprint(ch, stringf
			(ch, 0, ALIGN_CENTER, NULL, "{G-==[ {RBasic Styles{G ]==-{x"));
	chprintlnf(ch, "\n\r%s",
			   stringf(ch, 0, ALIGN_CENTER, NULL,
					   FORMATF
					   ("Viper: {Y%-3d{x Crane: {Y%-3d{x Crab: {Y%-3d{x Mongoose: {Y%-3d{x Bull: {Y%-3d{x",
						GET_STANCE(ch, STANCE_VIPER), GET_STANCE(ch,
																 STANCE_CRANE),
						GET_STANCE(ch, STANCE_CRAB), GET_STANCE(ch,
																STANCE_MONGOOSE),
						GET_STANCE(ch, STANCE_BULL))));
	chprintlnf(ch, "{W%s", draw_line(ch, NULL, 0));
	chprint(ch, stringf
			(ch, 0, ALIGN_CENTER, NULL, "{G-==[ {RAdvanced Styles{G ]==-{x"));
	chprintlnf(ch, "\n\r%s",
			   stringf(ch, 0, ALIGN_CENTER, NULL,
					   FORMATF
					   ("Mantis: {Y%-3d{x Dragon: {Y%-3d{x Tiger: {Y%-3d{x Monkey: {Y%-3d{x Swallow: {Y%-3d{x",
						GET_STANCE(ch, STANCE_MANTIS), GET_STANCE(ch,
																  STANCE_DRAGON),
						GET_STANCE(ch, STANCE_TIGER), GET_STANCE(ch,
																 STANCE_MONKEY),
						GET_STANCE(ch, STANCE_SWALLOW))));
	chprintlnf(ch, "{W%s", draw_line(ch, NULL, 0));
	chprintlnf(ch, "%s",
			   stringf(ch, 0, ALIGN_CENTER, NULL,
					   FORMATF("{RAutoStance: {x%s", get_stance_name(ast))));
	return;
}

long dambonus(CHAR_DATA * ch, CHAR_DATA * victim, long dam, int stance)
{
	if (dam < 1)
		return 0;

	if (!IS_VALID_STANCE(stance))
		return dam;

	if (!can_counter(victim))
	{
		if (IS_STANCE(ch, STANCE_MONKEY))
		{
			int mindam = dam * 25 / 100;

			dam *= (GET_STANCE(ch, STANCE_MONKEY) + 1) / 200;
			if (dam < mindam)
				dam = mindam;
		}
		else if (IS_STANCE(ch, STANCE_BULL)
				 && GET_STANCE(ch, STANCE_BULL) > 100)
			dam += dam * (GET_STANCE(ch, STANCE_BULL) / 100);
		else if (IS_STANCE(ch, STANCE_DRAGON)
				 && GET_STANCE(ch, STANCE_DRAGON) > 100)
			dam += dam * (GET_STANCE(ch, STANCE_DRAGON) / 100);
		else if (IS_STANCE(ch, STANCE_TIGER)
				 && GET_STANCE(ch, STANCE_TIGER) > 100)
			dam += dam * (GET_STANCE(ch, STANCE_TIGER) / 100);
		else if (GET_STANCE(ch, STANCE_CURRENT) > 0
				 && GET_STANCE(ch, stance) < 100)
			dam = dam * 5 / 10;
	}
	if (!can_counter(ch))
	{
		if (IS_STANCE(victim, STANCE_CRAB)
			&& GET_STANCE(victim, STANCE_CRAB) > 100)
			dam /= GET_STANCE(victim, STANCE_CRAB) / 100;
		else if (IS_STANCE(victim, STANCE_DRAGON)
				 && GET_STANCE(victim, STANCE_DRAGON) > 100)
			dam /= GET_STANCE(victim, STANCE_DRAGON) / 100;
		else if (IS_STANCE(victim, STANCE_SWALLOW)
				 && GET_STANCE(victim, STANCE_SWALLOW) > 100)
			dam /= GET_STANCE(victim, STANCE_SWALLOW) / 100;
	}
	return dam;
}

bool can_counter(CHAR_DATA * ch)
{
	if (IS_STANCE(ch, STANCE_MONKEY))
		return TRUE;

	return FALSE;
}

bool can_bypass(CHAR_DATA * ch, CHAR_DATA * victim)
{
	if (IS_STANCE(ch, STANCE_VIPER))
		return TRUE;

	if (IS_STANCE(ch, STANCE_MANTIS))
		return TRUE;

	if (IS_STANCE(ch, STANCE_TIGER))
		return TRUE;

	return FALSE;
}

char *get_stance_name(int stance)
{
	static char stancename[2][100];
	static int i;

	i++;
	i %= 2;

	switch (stance)
	{
	case STANCE_VIPER:
		sprintf(stancename[i], "viper");
		break;
	case STANCE_CRANE:
		sprintf(stancename[i], "crane");
		break;
	case STANCE_CRAB:
		sprintf(stancename[i], "crab");
		break;
	case STANCE_MONGOOSE:
		sprintf(stancename[i], "mongoose");
		break;
	case STANCE_BULL:
		sprintf(stancename[i], "bull");
		break;
	case STANCE_MANTIS:
		sprintf(stancename[i], "mantis");
		break;
	case STANCE_DRAGON:
		sprintf(stancename[i], "dragon");
		break;
	case STANCE_TIGER:
		sprintf(stancename[i], "tiger");
		break;
	case STANCE_MONKEY:
		sprintf(stancename[i], "monkey");
		break;
	case STANCE_SWALLOW:
		sprintf(stancename[i], "swallow");
		break;
	default:
		sprintf(stancename[i], "unknown");
		break;
	}
	return (stancename[i]);
}

void improve_stance(CHAR_DATA * ch)
{
	char bufskill[25];
	int dice1;
	int dice2;
	int stance;
	int skill;

	dice1 = number_percent();
	dice2 = number_percent();

	stance = GET_STANCE(ch, STANCE_CURRENT);
	if (!IS_VALID_STANCE(stance))
		return;
	skill = GET_STANCE(ch, stance);
	if (skill >= 200)
	{
		SET_STANCE(ch, stance, 200);
		return;
	}
	if ((dice1 > skill && dice2 > skill) || (dice1 == 100 || dice2 == 100))
		ch->stance[stance] += 1;
	else
		return;
	if (skill == GET_STANCE(ch, stance))
		return;

	if (IS_NPC(ch))
		return;

	switch (GET_STANCE(ch, stance))
	{

	case 1:
		sprintf(bufskill, "an apprentice of");
		break;
	case 26:
		sprintf(bufskill, "a trainee of");
		break;
	case 51:
		sprintf(bufskill, "a student of");
		break;
	case 76:
		sprintf(bufskill, "fairly experienced in");
		break;
	case 101:
		sprintf(bufskill, "well trained in");
		break;
	case 126:
		sprintf(bufskill, "highly skilled in");
		break;
	case 151:
		sprintf(bufskill, "an expert of");
		break;
	case 176:
		sprintf(bufskill, "a master of");
		break;
	case 200:
		sprintf(bufskill, "a grand master of");
		break;
	default:
		return;
	}

	chprintlnf(ch, "{RYou are now %s the %s stance.{x", bufskill,
			   get_stance_name(stance));
	return;
}

CH_CMD(do_stance)
{
	char arg[MIL];
	int selection;

	argument = one_argument(argument, arg);

	if (IS_NULLSTR(arg))
	{
		if (!IS_VALID_STANCE(GET_STANCE(ch, STANCE_CURRENT)))
		{
			SET_STANCE(ch, STANCE_CURRENT, STANCE_NORMAL);
			chprintln(ch, "You drop into a fighting stance.");
			act("$n drops into a fighting stance.", ch, NULL, NULL, TO_ROOM);
		}
		else
		{
			SET_STANCE(ch, STANCE_CURRENT, STANCE_NONE);
			chprintln(ch, "You relax from your fighting stance.");
			act("$n relaxes from $s fighting stance.", ch, NULL, NULL, TO_ROOM);
		}
		return;
	}
	if (IS_VALID_STANCE(GET_STANCE(ch, STANCE_CURRENT)))
	{
		chprintln
			(ch,
			 "You cannot change stances until you come up from the one you are currently in.");
		return;
	}
	if (!str_cmp(arg, "none"))
	{
		selection = STANCE_NONE;
		chprintln(ch, "You drop into a general fighting stance.");
		act("$n drops into a general fighting stance.", ch, NULL, NULL,
			TO_ROOM);
	}
	else if (!str_cmp(arg, "viper"))
	{
		selection = STANCE_VIPER;
		chprintln(ch, "You arch your body into the viper fighting stance.");
		act("$n arches $s body into the viper fighting stance.", ch, NULL,
			NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "crane"))
	{
		selection = STANCE_CRANE;
		chprintln(ch, "You swing your body into the crane fighting stance.");
		act("$n swings $s body into the crane fighting stance.", ch, NULL,
			NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "crab"))
	{
		selection = STANCE_CRAB;
		chprintln(ch, "You squat down into the crab fighting stance.");
		act("$n squats down into the crab fighting stance. ", ch, NULL, NULL,
			TO_ROOM);
	}
	else if (!str_cmp(arg, "mongoose"))
	{
		selection = STANCE_MONGOOSE;
		chprintln(ch, "You twist into the mongoose fighting stance.");
		act("$n twists into the mongoose fighting stance. ", ch, NULL, NULL,
			TO_ROOM);
	}
	else if (!str_cmp(arg, "bull"))
	{
		selection = STANCE_BULL;
		chprintln(ch, "You hunch down into the bull fighting stance.");
		act("$n hunches down into the bull fighting stance. ", ch, NULL,
			NULL, TO_ROOM);
	}
	else
	{
		if (!str_cmp(arg, "mantis") && GET_STANCE(ch, STANCE_CRANE) >= 200
			&& GET_STANCE(ch, STANCE_VIPER) >= 200)
		{
			selection = STANCE_MANTIS;
			chprintln(ch,
					  "You spin your body into the mantis fighting stance.");
			act("$n spins $s body into the mantis fighting stance.", ch, NULL,
				NULL, TO_ROOM);
		}
		else if (!str_cmp(arg, "dragon") && GET_STANCE(ch, STANCE_BULL) >= 200
				 && GET_STANCE(ch, STANCE_CRAB) >= 200)
		{
			selection = STANCE_DRAGON;
			chprintln(ch,
					  "You coil your body into the dragon fighting stance.");
			act("$n coils $s body into the dragon fighting stance.", ch, NULL,
				NULL, TO_ROOM);
		}
		else if (!str_cmp(arg, "tiger") && GET_STANCE(ch, STANCE_BULL) >= 200
				 && GET_STANCE(ch, STANCE_VIPER) >= 200)
		{
			selection = STANCE_TIGER;
			chprintln(ch, "You lunge into the tiger fighting stance.");
			act("$n lunges into the tiger fighting stance.", ch, NULL, NULL,
				TO_ROOM);
		}
		else if (!str_cmp(arg, "monkey") && GET_STANCE(ch, STANCE_CRANE) >= 200
				 && GET_STANCE(ch, STANCE_MONGOOSE) >= 200)
		{
			selection = STANCE_MONKEY;
			chprintln
				(ch, "You rotate your body into the monkey fighting stance.");
			act("$n rotates $s body into the monkey fighting stance.", ch,
				NULL, NULL, TO_ROOM);
		}
		else if (!str_cmp(arg, "swallow") && GET_STANCE(ch, STANCE_CRAB) >= 200
				 && GET_STANCE(ch, STANCE_MONGOOSE) >= 200)
		{
			selection = STANCE_SWALLOW;
			chprintln(ch, "You slide into the swallow fighting stance.");
			act("$n slides into the swallow fighting stance.", ch, NULL,
				NULL, TO_ROOM);
		}
		else
		{
			chprintln(ch, "Syntax is: stance <stance>.");
			chprintln
				(ch,
				 "Stance being one of: None, Viper, Crane, Crab, Mongoose, Bull.");
			chprintln(ch,
					  "If all basic mastered Stance may be: Swallow, Mantis, Tiger, Dragon, Monkey.");
			return;
		}
	}
	SET_STANCE(ch, STANCE_CURRENT, selection);
	if (IS_NPC(ch))
		SET_STANCE(ch, selection, UMIN(ch->level, 200));
	return;
}

CH_CMD(do_autostance)
{
	char arg[MIL];

	if (IS_NPC(ch))
		return;

	argument = one_argument(argument, arg);

	if (!str_cmp(arg, "none"))
	{
		chprintln(ch, "You no longer autostance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_NONE);
	}
	else if (!str_cmp(arg, "normal"))
	{
		chprintln(ch, "You now autostance into a normal stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_NORMAL);
	}
	else if (!str_cmp(arg, "viper"))
	{
		chprintln(ch, "You now autostance into the viper stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_VIPER);
	}
	else if (!str_cmp(arg, "crane"))
	{
		chprintln(ch, "You now autostance into the crane stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_CRANE);
	}
	else if (!str_cmp(arg, "crab"))
	{
		chprintln(ch, "You now autostance into the crab stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_CRAB);
	}
	else if (!str_cmp(arg, "mongoose"))
	{
		chprintln(ch, "You now autostance into the mongoose stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_MONGOOSE);
	}
	else if (!str_cmp(arg, "bull"))
	{
		chprintln(ch, "You now autostance into the bull stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_BULL);
	}
	else if (!str_cmp(arg, "mantis") && GET_STANCE(ch, STANCE_CRANE) >= 200
			 && GET_STANCE(ch, STANCE_VIPER) >= 200)
	{
		chprintln(ch, "You now autostance into the mantis stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_MANTIS);
	}
	else if (!str_cmp(arg, "monkey") && GET_STANCE(ch, STANCE_CRANE) >= 200
			 && GET_STANCE(ch, STANCE_MONGOOSE) >= 200)
	{
		chprintln(ch, "You now autostance into the monkey stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_MONKEY);
	}
	else if (!str_cmp(arg, "swallow") && GET_STANCE(ch, STANCE_CRAB) >= 200
			 && GET_STANCE(ch, STANCE_MONGOOSE) >= 200)
	{
		chprintln(ch, "You now autostance into the swallow stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_SWALLOW);
	}
	else if (!str_cmp(arg, "tiger") && GET_STANCE(ch, STANCE_BULL) >= 200
			 && GET_STANCE(ch, STANCE_VIPER) >= 200)
	{
		chprintln(ch, "You now autostance into the tiger stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_TIGER);
	}
	else if (!str_cmp(arg, "dragon") && GET_STANCE(ch, STANCE_CRAB) >= 200
			 && GET_STANCE(ch, STANCE_BULL) >= 200)
	{
		chprintln(ch, "You now autostance into the dragon stance.");
		SET_STANCE(ch, STANCE_AUTODROP, STANCE_DRAGON);
	}
	else
		chprintln(ch, "You can't set your autostance to that!");
}

void autodrop(CHAR_DATA * ch)
{
	int stance;

	stance = GET_STANCE(ch, STANCE_AUTODROP);

	if (!IS_VALID_STANCE(stance))
		return;

	if (!IS_VALID_STANCE(GET_STANCE(ch, STANCE_CURRENT)))
	{
		SET_STANCE(ch, STANCE_CURRENT, stance);
		chprintlnf(ch, "You autodrop into the %s stance. (%d%%)",
				   get_stance_name(stance), GET_STANCE(ch, stance));
		act("$n autodrops into the $T stance.", ch, NULL,
			get_stance_name(stance), TO_ROOM);
	}
}
